/*
*********************************************************************************
*										*
* UniformX.C.m4									*
*										*
* Function UniformPartition() breaks up a region of space into a number of	*
* uniformly sized regions using recursive bisection.  We assume the workload	*
* distribution is uniform across the region.					*
*										*
* Note that since uniform partitioning is so simple, we do not attempt to	*
* parallelize it, as the communication in the end would probably be more	*
* expensive than the calculation itself.					*
*										*
* Author:  Scott Kohn (skohn-at-cs.ucsd.edu)					*
*										*
*********************************************************************************
*/

//#include "Array.h"
#include "PointX.h"
#include "RegionX.h"

/*
*********************************************************************************
*										*
* static void divide_processor_set(const int n, int& left, int& right)		*
*										*
* Static function divide_processor_set() divides n into a group which is a	*
* power of two (left) and a group which represents the remainder (right).	*
*										*
*********************************************************************************
*/

static void divide_processor_set(const int n, int& left, int& right)
  {
   for (int l = 1; l < n; l <<= 1);
   left = l >> 1;
   right = n - left;
  }

/*
*********************************************************************************
*										*
* static int get_longest_dimension(const RegionX& region)			*
*										*
* Static function get_longest_dimension() finds the longest dimension of region	*
* and returns that dimension number.						*
*										*
*********************************************************************************
*/

static int get_longest_dimension(const RegionX& region)
  {
   PointX extents = region.upper()-region.lower()+1;
   int maxdim = 0, maxval = extents(0);
   for (int d = 1; d < NDIM; d++)
      if (extents(d) > maxval)
        {
         maxval = extents(d);
         maxdim = d;
        }
   return(maxdim);
  }

/*
*********************************************************************************
*										*
* static void uniform_partition_recursively(const RegionX& region,		*
*                                           const int n,			*
*					    RegionX *regionlist,		*
*                                           const int mincut)			*
*										*
* Function uniform_partition_recursively() recursively partitions region into	*
* n domains, each of approximately the same size.  The regions are returned in	*
* array regionlist.								*
*										*
*********************************************************************************
*/

static void uniform_partition_recursively(const RegionX& region,
                                          const int n,
                                          RegionX *regionlist,
                                          const PointX mincut)
  {
   /*
     If we have only one division, then we are finished -- set the region
   */

   if ((n == 1) || region.empty()) { regionlist->setregion(region); return; }

   /*
     Split the processor set and the domain and then do recursive call
   */

	PointX m_mincut;
	for(int i=0; i < NDIM; ++i) 
		m_mincut(i) = mincut(i)+(mincut(i)%2); // GC-make sure mincut is even
	
   int left, right;
   divide_processor_set(n, left, right);
   const int cut = get_longest_dimension(region);

   const int width = region.upper(cut)-region.lower(cut)+1;
//GC   if (width < 2*mincut(cut)) { regionlist->setregion(region); return; }
   if (width < 2*m_mincut(cut)) { regionlist->setregion(region); return; }

//GC   int split = (width*left)/(left+right);
//GC   if (split < mincut(cut)) split = mincut(cut);
//GC   if (split > width-mincut(cut)) split = width-mincut(cut);
   int split = (width*left)/(left+right);
   split = split + (split%2);
   if (split < m_mincut(cut)) split = m_mincut(cut);
   if (split > width-m_mincut(cut)) {
		split = width-m_mincut(cut);
   	split = split + (split%2);
	}

   PointX left_cutplane = region.upper();
   left_cutplane(cut) = split+region.lower(cut)-1;
   const RegionX left_region(region.lower(), left_cutplane);
   uniform_partition_recursively(left_region, left, regionlist, m_mincut);

   PointX right_cutplane = region.lower();
   right_cutplane(cut) = split+region.lower(cut);
   const RegionX right_region(right_cutplane, region.upper());
   uniform_partition_recursively(right_region, right,
                                 &regionlist[left], m_mincut);
  }

/*
*********************************************************************************
*										*
* RegionX *UniformPartition(const RegionX& region, int& n, const int mincut)	*
*										*
* Function UniformPartition() performs a uniform partitioning of the region	*
* into n partitions with a specified minimum cut.  A list of the partitions is	*
* returned.  The callee is responsible for delete [] ing the partition list.	*
* The number of non-empty regions is returned in n and all empty regions are	*
* moved to the end of the list.							*
*										*
*********************************************************************************
*/

RegionX *MyUniformPartition(const RegionX& region, int& n, const PointX mincut)
  {
   RegionX *regionlist = new RegionX[n];
   uniform_partition_recursively(region, n, regionlist, mincut);

   for (int i = 0; i < n; i++)
      if (regionlist[i].empty())
        {
         regionlist[i--] = regionlist[--n];
         regionlist[n].setempty();
        }

   return(regionlist);
  }
