c*******************************************************************************
c* Coarse to fine grid interpolation using 3X3 bi-quadratic interpolation
c*     fine: The fine grid.
c*   coarse: The coarse grid.
c*  fext(6): The "extent" of the fine grid,
c*           fine(fext(1):fext(4),fext(2):fext(5),fext(3):fext(6))
c*  cint(6): The "interior" of the coarse grid.  Used to determine how much
c*           of the fine grid to fill (see implimentation notes below).
c*  cext(6): The "extent" of the coarse grid,
c*           coarse(cext(1):cext(4),cext(2):cext(5),cext(3):cext(6))
c*  cfil(6): The "fill" of the coarse grid.  The range of indices for which
c*           the coarse grid containts valid points for use in interpolation.
c*******************************************************************************
c* Implimentation notes:
c*    cint must always be a superset of fint, ie.
c*          fint(1) >= 2*cint(1) - 1,
c*          fint(2) >= 2*cint(2) - 1,
c*          fint(3) >= 2*cint(3) - 1,
c*          fint(4) <= 2*cint(4)    ,
c*          fint(5) <= 2*cint(5)    .
c*          fint(6) <= 2*cint(6)    .
c*    Usually, cint "=" fint.  When adaptive gridding is used, fint may be
c*    defined one fine zone interior to cint.  This buffer zone is filled
c*    because cint is used to define the fine grid interpolation fill region
c*    instead of fint.  These filled buffer zones constitute the Dirichlet
c*    boundary points for the fine grid.
c*******************************************************************************
      subroutine ctof3x3x3(fine,coarse,fext,cint,cext,cfil)
      implicit none
      integer fext(6),cint(6),cext(6),cfil(6)
      real*8 fine(fext(1):fext(4),fext(2):fext(5),fext(3):fext(6))
      real*8 coarse(cext(1):cext(4),cext(2):cext(5),cext(3):cext(6))
      real*8 tmp0,tmp1,tmp2,tmp3,tmp4
      integer c1i,c1f,c2i,c2f,c3i,c3f
      integer f1i,f1f,f2i,f2f,f3i,f3f
      integer c1,c2,c3
      integer f10,f11,f20,f21,f30,f31
      real*8 F1o32768,
     &       F91125, F60750, F42525, F40500, F36450, F28350, F27000,
     &       F24300, F19845, F18900, F17010, F16200, F14580, F13230,
     &       F12600, F11340, F10125, F9720, F9261, F8820, F7938, F7560,
     &       F6804, F6750, F6174, F6075, F5880, F5832, F5292, F4725,
     &       F4536, F4500, F4116, F4050, F3528, F3150, F2835, F2744,
     &       F2700, F2430, F2205, F2100, F1890, F1620, F1470, F1323,
     &       F1260, F1134, F1125, F980, F972, F882, F756, F750, F675,
     &       F588, F525, F450, F405, F350, F315, F270, F210, F189, F162,
     &       F126, F125, F75, F45, F27
      parameter(F1o32768 = 1.D0/32768.D0,
     &          F91125 = 91125.D0, F60750 = 60750.D0, F42525 = 42525.D0,
     &          F40500 = 40500.D0, F36450 = 36450.D0, F28350 = 28350.D0,
     &          F27000 = 27000.D0, F24300 = 24300.D0, F19845 = 19845.D0,
     &          F18900 = 18900.D0, F17010 = 17010.D0, F16200 = 16200.D0,
     &          F14580 = 14580.D0, F13230 = 13230.D0, F12600 = 12600.D0,
     &          F11340 = 11340.D0, F10125 = 10125.D0, F9720 = 9720.D0,
     &          F9261 = 9261.D0, F8820 = 8820.D0, F7938 = 7938.D0,
     &          F7560 = 7560.D0, F6804 = 6804.D0, F6750 = 6750.D0,
     &          F6174 = 6174.D0, F6075 = 6075.D0, F5880 = 5880.D0,
     &          F5832 = 5832.D0, F5292 = 5292.D0, F4725 = 4725.D0,
     &          F4536 = 4536.D0, F4500 = 4500.D0, F4116 = 4116.D0,
     &          F4050 = 4050.D0, F3528 = 3528.D0, F3150 = 3150.D0,
     &          F2835 = 2835.D0, F2744 = 2744.D0, F2700 = 2700.D0,
     &          F2430 = 2430.D0, F2205 = 2205.D0, F2100 = 2100.D0,
     &          F1890 = 1890.D0, F1620 = 1620.D0, F1470 = 1470.D0,
     &          F1323 = 1323.D0, F1260 = 1260.D0, F1134 = 1134.D0,
     &          F1125 = 1125.D0, F980 = 980.D0, F972 = 972.D0,
     &          F882 = 882.D0, F756 = 756.D0, F750 = 750.D0,
     &          F675 = 675.D0, F588 = 588.D0, F525 = 525.D0,
     &          F450 = 450.D0, F405 = 405.D0, F350 = 350.D0,
     &          F315 = 315.D0, F270 = 270.D0, F210 = 210.D0,
     &          F189 = 189.D0, F162 = 162.D0, F126 = 126.D0,
     &          F125 = 125.D0, F75 = 75.D0, F45 = 45.D0, F27 = 27.D0)
c
c Make sure the "fill" region of the coarse grid is at least 3X3X3.
      if(((cfil(4) - cfil(1)).lt.2).or.((cfil(5) - cfil(2)).lt.2).or.
     .                                 ((cfil(6) - cfil(3)).lt.2)) then
        write(*,*) "ctof3x3x3: fill region smaller than 3X3X3!"
        stop
      end if
c
c Set the coarse interior which will be used to fill the fine grid using
c "centered" 3X3X3 interpolation.
      c1i = MAX(cint(1),cfil(1) + 1)
      f1i = 2*c1i
      c2i = MAX(cint(2),cfil(2) + 1)
      f2i = 2*c2i
      c3i = MAX(cint(3),cfil(3) + 1)
      f3i = 2*c3i
      c1f = MIN(cint(4),cfil(4) - 1)
      f1f = 2*c1f
      c2f = MIN(cint(5),cfil(5) - 1)
      f2f = 2*c2f
      c3f = MIN(cint(6),cfil(6) - 1)
      f3f = 2*c3f
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Fill interior.
      f31 = f3i
      do c3 = c3i,c3f
        f21 = f2i
        do c2 = c2i,c2f
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F27000*coarse( c1  , c2  , c3  ) 
     & +     F4500*coarse( c1  , c2  , c3-1) 
     & +     F4500*coarse( c1  , c2-1, c3  ) 
     & +     F4500*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1+1, c2  , c3  ) 
     & -     F2700*coarse( c1  , c2+1, c3  ) 
     & -     F2700*coarse( c1  , c2  , c3+1) 
     & +      F750*coarse( c1  , c2-1, c3-1) 
     & +      F750*coarse( c1-1, c2  , c3-1) 
     & +      F750*coarse( c1-1, c2-1, c3  ) 
     & -      F450*coarse( c1+1, c2  , c3-1) 
     & -      F450*coarse( c1  , c2+1, c3-1) 
     & -      F450*coarse( c1+1, c2-1, c3  ) 
     & -      F450*coarse( c1-1, c2+1, c3  ) 
     & -      F450*coarse( c1  , c2-1, c3+1) 
     & -      F450*coarse( c1-1, c2  , c3+1) 
     & +      F270*coarse( c1+1, c2+1, c3  ) 
     & +      F270*coarse( c1+1, c2  , c3+1) 
     & +      F270*coarse( c1  , c2+1, c3+1) 
     & +      F125*coarse( c1-1, c2-1, c3-1) 
     & -       F75*coarse( c1+1, c2-1, c3-1) 
     & -       F75*coarse( c1-1, c2+1, c3-1) 
     & -       F75*coarse( c1-1, c2-1, c3+1) 
     & +       F45*coarse( c1+1, c2+1, c3-1) 
     & +       F45*coarse( c1+1, c2-1, c3+1) 
     & +       F45*coarse( c1-1, c2+1, c3+1) 
     & -       F27*coarse( c1+1, c2+1, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F27000*coarse( c1  , c2  , c3  ) 
     & +     F4500*coarse( c1  , c2  , c3-1) 
     & +     F4500*coarse( c1+1, c2  , c3  ) 
     & +     F4500*coarse( c1  , c2-1, c3  ) 
     & -     F2700*coarse( c1-1, c2  , c3  ) 
     & -     F2700*coarse( c1  , c2+1, c3  ) 
     & -     F2700*coarse( c1  , c2  , c3+1) 
     & +      F750*coarse( c1  , c2-1, c3-1) 
     & +      F750*coarse( c1+1, c2  , c3-1) 
     & +      F750*coarse( c1+1, c2-1, c3  ) 
     & -      F450*coarse( c1-1, c2  , c3-1) 
     & -      F450*coarse( c1  , c2+1, c3-1) 
     & -      F450*coarse( c1-1, c2-1, c3  ) 
     & -      F450*coarse( c1+1, c2+1, c3  ) 
     & -      F450*coarse( c1  , c2-1, c3+1) 
     & -      F450*coarse( c1+1, c2  , c3+1) 
     & +      F270*coarse( c1-1, c2+1, c3  ) 
     & +      F270*coarse( c1-1, c2  , c3+1) 
     & +      F270*coarse( c1  , c2+1, c3+1) 
     & +      F125*coarse( c1+1, c2-1, c3-1) 
     & -       F75*coarse( c1-1, c2-1, c3-1) 
     & -       F75*coarse( c1+1, c2+1, c3-1) 
     & -       F75*coarse( c1+1, c2-1, c3+1) 
     & +       F45*coarse( c1-1, c2+1, c3-1) 
     & +       F45*coarse( c1-1, c2-1, c3+1) 
     & +       F45*coarse( c1+1, c2+1, c3+1)
     & -       F27*coarse( c1-1, c2+1, c3+1) 
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F27000*coarse( c1  , c2  , c3  ) 
     & +     F4500*coarse( c1  , c2  , c3-1) 
     & +     F4500*coarse( c1-1, c2  , c3  ) 
     & +     F4500*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2-1, c3  ) 
     & -     F2700*coarse( c1+1, c2  , c3  ) 
     & -     F2700*coarse( c1  , c2  , c3+1) 
     & +      F750*coarse( c1-1, c2  , c3-1) 
     & +      F750*coarse( c1  , c2+1, c3-1) 
     & +      F750*coarse( c1-1, c2+1, c3  ) 
     & -      F450*coarse( c1  , c2-1, c3-1) 
     & -      F450*coarse( c1+1, c2  , c3-1) 
     & -      F450*coarse( c1-1, c2-1, c3  ) 
     & -      F450*coarse( c1+1, c2+1, c3  ) 
     & -      F450*coarse( c1-1, c2  , c3+1) 
     & -      F450*coarse( c1  , c2+1, c3+1) 
     & +      F270*coarse( c1+1, c2-1, c3  ) 
     & +      F270*coarse( c1  , c2-1, c3+1) 
     & +      F270*coarse( c1+1, c2  , c3+1) 
     & +      F125*coarse( c1-1, c2+1, c3-1) 
     & -       F75*coarse( c1-1, c2-1, c3-1) 
     & -       F75*coarse( c1+1, c2+1, c3-1) 
     & -       F75*coarse( c1-1, c2+1, c3+1) 
     & +       F45*coarse( c1+1, c2-1, c3-1) 
     & +       F45*coarse( c1-1, c2-1, c3+1) 
     & +       F45*coarse( c1+1, c2+1, c3+1)
     & -       F27*coarse( c1+1, c2-1, c3+1) 
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F27000*coarse( c1  , c2  , c3  )
     & +     F4500*coarse( c1  , c2  , c3-1) 
     & +     F4500*coarse( c1+1, c2  , c3  ) 
     & +     F4500*coarse( c1  , c2+1, c3  ) 
     & -     F2700*coarse( c1  , c2-1, c3  ) 
     & -     F2700*coarse( c1-1, c2  , c3  ) 
     & -     F2700*coarse( c1  , c2  , c3+1) 
     & +      F750*coarse( c1+1, c2  , c3-1) 
     & +      F750*coarse( c1  , c2+1, c3-1) 
     & +      F750*coarse( c1+1, c2+1, c3  ) 
     & -      F450*coarse( c1  , c2-1, c3-1) 
     & -      F450*coarse( c1-1, c2  , c3-1) 
     & -      F450*coarse( c1+1, c2-1, c3  ) 
     & -      F450*coarse( c1-1, c2+1, c3  ) 
     & -      F450*coarse( c1+1, c2  , c3+1) 
     & -      F450*coarse( c1  , c2+1, c3+1) 
     & +      F270*coarse( c1-1, c2-1, c3  ) 
     & +      F270*coarse( c1  , c2-1, c3+1) 
     & +      F270*coarse( c1-1, c2  , c3+1) 
     & +      F125*coarse( c1+1, c2+1, c3-1) 
     & -       F75*coarse( c1+1, c2-1, c3-1) 
     & -       F75*coarse( c1-1, c2+1, c3-1) 
     & -       F75*coarse( c1+1, c2+1, c3+1)
     & +       F45*coarse( c1-1, c2-1, c3-1) 
     & +       F45*coarse( c1+1, c2-1, c3+1) 
     & +       F45*coarse( c1-1, c2+1, c3+1) 
     & -       F27*coarse( c1-1, c2-1, c3+1) 
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F27000*coarse( c1  , c2  , c3  ) 
     & +     F4500*coarse( c1  , c2-1, c3  ) 
     & +     F4500*coarse( c1-1, c2  , c3  ) 
     & +     F4500*coarse( c1  , c2  , c3+1) 
     & -     F2700*coarse( c1  , c2  , c3-1) 
     & -     F2700*coarse( c1+1, c2  , c3  ) 
     & -     F2700*coarse( c1  , c2+1, c3  ) 
     & +      F750*coarse( c1-1, c2-1, c3  ) 
     & +      F750*coarse( c1  , c2-1, c3+1) 
     & +      F750*coarse( c1-1, c2  , c3+1) 
     & -      F450*coarse( c1  , c2-1, c3-1) 
     & -      F450*coarse( c1-1, c2  , c3-1) 
     & -      F450*coarse( c1+1, c2-1, c3  ) 
     & -      F450*coarse( c1-1, c2+1, c3  ) 
     & -      F450*coarse( c1+1, c2  , c3+1) 
     & -      F450*coarse( c1  , c2+1, c3+1) 
     & +      F270*coarse( c1+1, c2  , c3-1) 
     & +      F270*coarse( c1  , c2+1, c3-1) 
     & +      F270*coarse( c1+1, c2+1, c3  ) 
     & +      F125*coarse( c1-1, c2-1, c3+1) 
     & -       F75*coarse( c1-1, c2-1, c3-1) 
     & -       F75*coarse( c1+1, c2-1, c3+1) 
     & -       F75*coarse( c1-1, c2+1, c3+1) 
     & +       F45*coarse( c1+1, c2-1, c3-1) 
     & +       F45*coarse( c1-1, c2+1, c3-1) 
     & +       F45*coarse( c1+1, c2+1, c3+1)
     & -       F27*coarse( c1+1, c2+1, c3-1) 
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F27000*coarse( c1  , c2  , c3  ) 
     & +     F4500*coarse( c1  , c2-1, c3  ) 
     & +     F4500*coarse( c1+1, c2  , c3  ) 
     & +     F4500*coarse( c1  , c2  , c3+1) 
     & -     F2700*coarse( c1  , c2  , c3-1) 
     & -     F2700*coarse( c1-1, c2  , c3  ) 
     & -     F2700*coarse( c1  , c2+1, c3  ) 
     & +      F750*coarse( c1+1, c2-1, c3  ) 
     & +      F750*coarse( c1  , c2-1, c3+1) 
     & +      F750*coarse( c1+1, c2  , c3+1) 
     & -      F450*coarse( c1  , c2-1, c3-1) 
     & -      F450*coarse( c1+1, c2  , c3-1) 
     & -      F450*coarse( c1-1, c2-1, c3  ) 
     & -      F450*coarse( c1+1, c2+1, c3  ) 
     & -      F450*coarse( c1-1, c2  , c3+1) 
     & -      F450*coarse( c1  , c2+1, c3+1) 
     & +      F270*coarse( c1-1, c2  , c3-1) 
     & +      F270*coarse( c1  , c2+1, c3-1) 
     & +      F270*coarse( c1-1, c2+1, c3  ) 
     & +      F125*coarse( c1+1, c2-1, c3+1) 
     & -       F75*coarse( c1+1, c2-1, c3-1) 
     & -       F75*coarse( c1-1, c2-1, c3+1) 
     & -       F75*coarse( c1+1, c2+1, c3+1)
     & +       F45*coarse( c1-1, c2-1, c3-1) 
     & +       F45*coarse( c1+1, c2+1, c3-1) 
     & +       F45*coarse( c1-1, c2+1, c3+1) 
     & -       F27*coarse( c1-1, c2+1, c3-1) 
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F27000*coarse( c1  , c2  , c3  ) 
     & +     F4500*coarse( c1-1, c2  , c3  ) 
     & +     F4500*coarse( c1  , c2+1, c3  ) 
     & +     F4500*coarse( c1  , c2  , c3+1) 
     & -     F2700*coarse( c1  , c2  , c3-1) 
     & -     F2700*coarse( c1  , c2-1, c3  ) 
     & -     F2700*coarse( c1+1, c2  , c3  ) 
     & +      F750*coarse( c1-1, c2+1, c3  ) 
     & +      F750*coarse( c1-1, c2  , c3+1) 
     & +      F750*coarse( c1  , c2+1, c3+1) 
     & -      F450*coarse( c1-1, c2  , c3-1) 
     & -      F450*coarse( c1  , c2+1, c3-1) 
     & -      F450*coarse( c1-1, c2-1, c3  ) 
     & -      F450*coarse( c1+1, c2+1, c3  ) 
     & -      F450*coarse( c1  , c2-1, c3+1) 
     & -      F450*coarse( c1+1, c2  , c3+1) 
     & +      F270*coarse( c1  , c2-1, c3-1) 
     & +      F270*coarse( c1+1, c2  , c3-1) 
     & +      F270*coarse( c1+1, c2-1, c3  ) 
     & +      F125*coarse( c1-1, c2+1, c3+1) 
     & -       F75*coarse( c1-1, c2+1, c3-1) 
     & -       F75*coarse( c1-1, c2-1, c3+1) 
     & -       F75*coarse( c1+1, c2+1, c3+1)
     & +       F45*coarse( c1-1, c2-1, c3-1) 
     & +       F45*coarse( c1+1, c2+1, c3-1) 
     & +       F45*coarse( c1+1, c2-1, c3+1) 
     & -       F27*coarse( c1+1, c2-1, c3-1) 
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F27000*coarse( c1  , c2  , c3  ) 
     & +     F4500*coarse( c1+1, c2  , c3  ) 
     & +     F4500*coarse( c1  , c2+1, c3  ) 
     & +     F4500*coarse( c1  , c2  , c3+1) 
     & -     F2700*coarse( c1  , c2  , c3-1) 
     & -     F2700*coarse( c1  , c2-1, c3  ) 
     & -     F2700*coarse( c1-1, c2  , c3  ) 
     & +      F750*coarse( c1+1, c2+1, c3  ) 
     & +      F750*coarse( c1+1, c2  , c3+1) 
     & +      F750*coarse( c1  , c2+1, c3+1) 
     & -      F450*coarse( c1+1, c2  , c3-1) 
     & -      F450*coarse( c1  , c2+1, c3-1) 
     & -      F450*coarse( c1+1, c2-1, c3  ) 
     & -      F450*coarse( c1-1, c2+1, c3  ) 
     & -      F450*coarse( c1  , c2-1, c3+1) 
     & -      F450*coarse( c1-1, c2  , c3+1) 
     & +      F270*coarse( c1  , c2-1, c3-1) 
     & +      F270*coarse( c1-1, c2  , c3-1) 
     & +      F270*coarse( c1-1, c2-1, c3  ) 
     & +      F125*coarse( c1+1, c2+1, c3+1)
     & -       F75*coarse( c1+1, c2+1, c3-1) 
     & -       F75*coarse( c1+1, c2-1, c3+1) 
     & -       F75*coarse( c1-1, c2+1, c3+1) 
     & +       F45*coarse( c1+1, c2-1, c3-1) 
     & +       F45*coarse( c1-1, c2+1, c3-1) 
     & +       F45*coarse( c1-1, c2-1, c3+1) 
     & -       F27*coarse( c1-1, c2-1, c3-1) 
     & )
            f11 = f11 + 2
          end do
          f21 = f21 + 2
        end do
        f31 = f31 + 2
      end do
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Set cint(1) boundary
      if(c1i.gt.cint(1)) then
        c1 = cint(1)
        f11 = 2*c1
c
c Set cint(1)-cint(3) edge
        if(c3i.gt.cint(3)) then
          c3 = cint(3)
          f31 = 2*c3
          f21 = f2i
          do c2 = c2i,c2f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1+1, c2  , c3  )
     & -    F24300*coarse( c1  , c2  , c3+1)
     & +    F10125*coarse( c1  , c2-1, c3  )
     & +     F9720*coarse( c1+1, c2  , c3+1)
     & +     F6750*coarse( c1+2, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3+2)
     & -     F6075*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1+1, c2-1, c3  )
     & -     F4050*coarse( c1  , c2-1, c3+1)
     & -     F2700*coarse( c1+2, c2  , c3+1)
     & -     F2700*coarse( c1+1, c2  , c3+2)
     & +     F2430*coarse( c1+1, c2+1, c3  )
     & +     F2430*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1+1, c2-1, c3+1)
     & +     F1125*coarse( c1+2, c2-1, c3  )
     & +     F1125*coarse( c1  , c2-1, c3+2)
     & -      F972*coarse( c1+1, c2+1, c3+1)
     & +      F750*coarse( c1+2, c2  , c3+2)
     & -      F675*coarse( c1+2, c2+1, c3  )
     & -      F675*coarse( c1  , c2+1, c3+2)
     & -      F450*coarse( c1+2, c2-1, c3+1)
     & -      F450*coarse( c1+1, c2-1, c3+2)
     & +      F270*coarse( c1+2, c2+1, c3+1)
     & +      F270*coarse( c1+1, c2+1, c3+2)
     & +      F125*coarse( c1+2, c2-1, c3+2)
     & -       F75*coarse( c1+2, c2+1, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1+1, c2  , c3  )
     & -    F11340*coarse( c1  , c2  , c3+1)
     & -     F7560*coarse( c1+1, c2  , c3+1)
     & +     F4725*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1+1, c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+2)
     & -     F2835*coarse( c1  , c2+1, c3  )
     & +     F2100*coarse( c1+1, c2  , c3+2)
     & -     F1890*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1+2, c2  , c3+1)
     & -     F1260*coarse( c1+1, c2-1, c3+1)
     & +     F1134*coarse( c1  , c2+1, c3+1)
     & +      F756*coarse( c1+1, c2+1, c3+1)
     & -      F675*coarse( c1+2, c2-1, c3  )
     & +      F525*coarse( c1  , c2-1, c3+2)
     & -      F450*coarse( c1+2, c2  , c3+2)
     & +      F405*coarse( c1+2, c2+1, c3  )
     & +      F350*coarse( c1+1, c2-1, c3+2)
     & -      F315*coarse( c1  , c2+1, c3+2)
     & +      F270*coarse( c1+2, c2-1, c3+1)
     & -      F210*coarse( c1+1, c2+1, c3+2)
     & -      F162*coarse( c1+2, c2+1, c3+1)
     & -       F75*coarse( c1+2, c2-1, c3+2)
     & +       F45*coarse( c1+2, c2+1, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2  , c3+1)
     & -    F24300*coarse( c1+1, c2  , c3  )
     & +    F10125*coarse( c1  , c2+1, c3  )
     & +     F9720*coarse( c1+1, c2  , c3+1)
     & +     F6750*coarse( c1+2, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3+2)
     & -     F6075*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1+1, c2+1, c3  )
     & -     F4050*coarse( c1  , c2+1, c3+1)
     & -     F2700*coarse( c1+2, c2  , c3+1)
     & -     F2700*coarse( c1+1, c2  , c3+2)
     & +     F2430*coarse( c1+1, c2-1, c3  )
     & +     F2430*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1+1, c2+1, c3+1)
     & +     F1125*coarse( c1+2, c2+1, c3  )
     & +     F1125*coarse( c1  , c2+1, c3+2)
     & -      F972*coarse( c1+1, c2-1, c3+1)
     & +      F750*coarse( c1+2, c2  , c3+2)
     & -      F675*coarse( c1+2, c2-1, c3  )
     & -      F675*coarse( c1  , c2-1, c3+2)
     & -      F450*coarse( c1+2, c2+1, c3+1)
     & -      F450*coarse( c1+1, c2+1, c3+2)
     & +      F270*coarse( c1+2, c2-1, c3+1)
     & +      F270*coarse( c1+1, c2-1, c3+2)
     & +      F125*coarse( c1+2, c2+1, c3+2)
     & -       F75*coarse( c1+2, c2-1, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1+1, c2  , c3  )
     & -    F11340*coarse( c1  , c2  , c3+1)
     & -     F7560*coarse( c1+1, c2  , c3+1)
     & +     F4725*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1+1, c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+2)
     & -     F2835*coarse( c1  , c2-1, c3  )
     & +     F2100*coarse( c1+1, c2  , c3+2)
     & -     F1890*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1+2, c2  , c3+1)
     & -     F1260*coarse( c1+1, c2+1, c3+1)
     & +     F1134*coarse( c1  , c2-1, c3+1)
     & +      F756*coarse( c1+1, c2-1, c3+1)
     & -      F675*coarse( c1+2, c2+1, c3  )
     & +      F525*coarse( c1  , c2+1, c3+2)
     & -      F450*coarse( c1+2, c2  , c3+2)
     & +      F405*coarse( c1+2, c2-1, c3  )
     & +      F350*coarse( c1+1, c2+1, c3+2)
     & -      F315*coarse( c1  , c2-1, c3+2)
     & +      F270*coarse( c1+2, c2+1, c3+1)
     & -      F210*coarse( c1+1, c2-1, c3+2)
     & -      F162*coarse( c1+2, c2-1, c3+1)
     & -       F75*coarse( c1+2, c2+1, c3+2)
     & +       F45*coarse( c1+2, c2-1, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1+1, c2  , c3  )
     & -     F7560*coarse( c1+1, c2  , c3+1)
     & +     F4725*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1  , c2-1, c3+1)
     & +     F3150*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2+1, c3  )
     & +     F2100*coarse( c1+2, c2  , c3+1)
     & -     F1890*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1+1, c2  , c3+2)
     & -     F1260*coarse( c1+1, c2-1, c3+1)
     & +     F1134*coarse( c1+1, c2+1, c3  )
     & +      F756*coarse( c1+1, c2+1, c3+1)
     & -      F675*coarse( c1  , c2-1, c3+2)
     & +      F525*coarse( c1+2, c2-1, c3  )
     & -      F450*coarse( c1+2, c2  , c3+2)
     & +      F405*coarse( c1  , c2+1, c3+2)
     & +      F350*coarse( c1+2, c2-1, c3+1)
     & -      F315*coarse( c1+2, c2+1, c3  )
     & +      F270*coarse( c1+1, c2-1, c3+2)
     & -      F210*coarse( c1+2, c2+1, c3+1)
     & -      F162*coarse( c1+1, c2+1, c3+2)
     & -       F75*coarse( c1+2, c2-1, c3+2)
     & +       F45*coarse( c1+2, c2+1, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1+1, c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3+1)
     & +     F5880*coarse( c1+1, c2  , c3+1)
     & +     F2205*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3+2)
     & +     F1470*coarse( c1+1, c2-1, c3  )
     & +     F1470*coarse( c1  , c2-1, c3+1)
     & -     F1323*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1+2, c2  , c3+1)
     & -     F1260*coarse( c1+1, c2  , c3+2)
     & +      F980*coarse( c1+1, c2-1, c3+1)
     & -      F882*coarse( c1+1, c2+1, c3  )
     & -      F882*coarse( c1  , c2+1, c3+1)
     & -      F588*coarse( c1+1, c2+1, c3+1)
     & -      F315*coarse( c1+2, c2-1, c3  )
     & -      F315*coarse( c1  , c2-1, c3+2)
     & +      F270*coarse( c1+2, c2  , c3+2)
     & -      F210*coarse( c1+2, c2-1, c3+1)
     & -      F210*coarse( c1+1, c2-1, c3+2)
     & +      F189*coarse( c1+2, c2+1, c3  )
     & +      F189*coarse( c1  , c2+1, c3+2)
     & +      F126*coarse( c1+2, c2+1, c3+1)
     & +      F126*coarse( c1+1, c2+1, c3+2)
     & +       F45*coarse( c1+2, c2-1, c3+2)
     & -       F27*coarse( c1+2, c2+1, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1+1, c2  , c3  )
     & -     F7560*coarse( c1+1, c2  , c3+1)
     & +     F4725*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3+1)
     & -     F2835*coarse( c1  , c2-1, c3  )
     & +     F2100*coarse( c1+2, c2  , c3+1)
     & -     F1890*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1+1, c2  , c3+2)
     & -     F1260*coarse( c1+1, c2+1, c3+1)
     & +     F1134*coarse( c1+1, c2-1, c3  )
     & +      F756*coarse( c1+1, c2-1, c3+1)
     & -      F675*coarse( c1  , c2+1, c3+2)
     & +      F525*coarse( c1+2, c2+1, c3  )
     & -      F450*coarse( c1+2, c2  , c3+2)
     & +      F405*coarse( c1  , c2-1, c3+2)
     & +      F350*coarse( c1+2, c2+1, c3+1)
     & -      F315*coarse( c1+2, c2-1, c3  )
     & +      F270*coarse( c1+1, c2+1, c3+2)
     & -      F210*coarse( c1+2, c2-1, c3+1)
     & -      F162*coarse( c1+1, c2-1, c3+2)
     & -       F75*coarse( c1+2, c2+1, c3+2)
     & +       F45*coarse( c1+2, c2-1, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1+1, c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3+1)
     & +     F5880*coarse( c1+1, c2  , c3+1)
     & +     F2205*coarse( c1  , c2+1, c3  )
     & -     F1890*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3+2)
     & +     F1470*coarse( c1+1, c2+1, c3  )
     & +     F1470*coarse( c1  , c2+1, c3+1)
     & -     F1323*coarse( c1  , c2-1, c3  )
     & -     F1260*coarse( c1+2, c2  , c3+1)
     & -     F1260*coarse( c1+1, c2  , c3+2)
     & +      F980*coarse( c1+1, c2+1, c3+1)
     & -      F882*coarse( c1+1, c2-1, c3  )
     & -      F882*coarse( c1  , c2-1, c3+1)
     & -      F588*coarse( c1+1, c2-1, c3+1)
     & -      F315*coarse( c1+2, c2+1, c3  )
     & -      F315*coarse( c1  , c2+1, c3+2)
     & +      F270*coarse( c1+2, c2  , c3+2)
     & -      F210*coarse( c1+2, c2+1, c3+1)
     & -      F210*coarse( c1+1, c2+1, c3+2)
     & +      F189*coarse( c1+2, c2-1, c3  )
     & +      F189*coarse( c1  , c2-1, c3+2)
     & +      F126*coarse( c1+2, c2-1, c3+1)
     & +      F126*coarse( c1+1, c2-1, c3+2)
     & +       F45*coarse( c1+2, c2+1, c3+2)
     & -       F27*coarse( c1+2, c2-1, c3+2)
     & )
            f21 = f21 + 2
          end do
        end if
c
c Set cint(1) face
        f31 = f3i
        do c3 = c3i,c3f
          f21 = f2i
          do c2 = c2i,c2f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1  , c2-1, c3  )
     & +     F4500*coarse( c1+2, c2  , c3  )
     & -     F4050*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1+1, c2  , c3-1)
     & -     F2700*coarse( c1+1, c2-1, c3  )
     & +     F1620*coarse( c1+1, c2+1, c3  )
     & +     F1620*coarse( c1+1, c2  , c3+1)
     & +     F1125*coarse( c1  , c2-1, c3-1)
     & +      F750*coarse( c1+2, c2  , c3-1)
     & +      F750*coarse( c1+2, c2-1, c3  )
     & -      F675*coarse( c1  , c2+1, c3-1)
     & -      F675*coarse( c1  , c2-1, c3+1)
     & -      F450*coarse( c1+1, c2-1, c3-1)
     & -      F450*coarse( c1+2, c2+1, c3  )
     & -      F450*coarse( c1+2, c2  , c3+1)
     & +      F405*coarse( c1  , c2+1, c3+1)
     & +      F270*coarse( c1+1, c2+1, c3-1)
     & +      F270*coarse( c1+1, c2-1, c3+1)
     & -      F162*coarse( c1+1, c2+1, c3+1)
     & +      F125*coarse( c1+2, c2-1, c3-1)
     & -       F75*coarse( c1+2, c2+1, c3-1)
     & -       F75*coarse( c1+2, c2-1, c3+1)
     & +       F45*coarse( c1+2, c2+1, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1+1, c2  , c3  )
     & +     F3150*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1  , c2-1, c3  )
     & -     F2700*coarse( c1+2, c2  , c3  )
     & +     F2100*coarse( c1+1, c2  , c3-1)
     & +     F2100*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3  )
     & -     F1890*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1+1, c2+1, c3  )
     & -     F1260*coarse( c1+1, c2  , c3+1)
     & +      F525*coarse( c1  , c2-1, c3-1)
     & -      F450*coarse( c1+2, c2  , c3-1)
     & -      F450*coarse( c1+2, c2-1, c3  )
     & +      F350*coarse( c1+1, c2-1, c3-1)
     & -      F315*coarse( c1  , c2+1, c3-1)
     & -      F315*coarse( c1  , c2-1, c3+1)
     & +      F270*coarse( c1+2, c2+1, c3  )
     & +      F270*coarse( c1+2, c2  , c3+1)
     & -      F210*coarse( c1+1, c2+1, c3-1)
     & -      F210*coarse( c1+1, c2-1, c3+1)
     & +      F189*coarse( c1  , c2+1, c3+1)
     & +      F126*coarse( c1+1, c2+1, c3+1)
     & -       F75*coarse( c1+2, c2-1, c3-1)
     & +       F45*coarse( c1+2, c2+1, c3-1)
     & +       F45*coarse( c1+2, c2-1, c3+1)
     & -       F27*coarse( c1+2, c2+1, c3+1)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1  , c2+1, c3  )
     & +     F4500*coarse( c1+2, c2  , c3  )
     & -     F4050*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1+1, c2  , c3-1)
     & -     F2700*coarse( c1+1, c2+1, c3  )
     & +     F1620*coarse( c1+1, c2-1, c3  )
     & +     F1620*coarse( c1+1, c2  , c3+1)
     & +     F1125*coarse( c1  , c2+1, c3-1)
     & +      F750*coarse( c1+2, c2  , c3-1)
     & +      F750*coarse( c1+2, c2+1, c3  )
     & -      F675*coarse( c1  , c2-1, c3-1)
     & -      F675*coarse( c1  , c2+1, c3+1)
     & -      F450*coarse( c1+1, c2+1, c3-1)
     & -      F450*coarse( c1+2, c2-1, c3  )
     & -      F450*coarse( c1+2, c2  , c3+1)
     & +      F405*coarse( c1  , c2-1, c3+1)
     & +      F270*coarse( c1+1, c2-1, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3+1)
     & -      F162*coarse( c1+1, c2-1, c3+1)
     & +      F125*coarse( c1+2, c2+1, c3-1)
     & -       F75*coarse( c1+2, c2-1, c3-1)
     & -       F75*coarse( c1+2, c2+1, c3+1)
     & +       F45*coarse( c1+2, c2-1, c3+1)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1+1, c2  , c3  )
     & +     F3150*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1+2, c2  , c3  )
     & +     F2100*coarse( c1+1, c2  , c3-1)
     & +     F2100*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1+1, c2-1, c3  )
     & -     F1260*coarse( c1+1, c2  , c3+1)
     & +      F525*coarse( c1  , c2+1, c3-1)
     & -      F450*coarse( c1+2, c2  , c3-1)
     & -      F450*coarse( c1+2, c2+1, c3  )
     & +      F350*coarse( c1+1, c2+1, c3-1)
     & -      F315*coarse( c1  , c2-1, c3-1)
     & -      F315*coarse( c1  , c2+1, c3+1)
     & +      F270*coarse( c1+2, c2-1, c3  )
     & +      F270*coarse( c1+2, c2  , c3+1)
     & -      F210*coarse( c1+1, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3+1)
     & +      F189*coarse( c1  , c2-1, c3+1)
     & +      F126*coarse( c1+1, c2-1, c3+1)
     & -       F75*coarse( c1+2, c2+1, c3-1)
     & +       F45*coarse( c1+2, c2-1, c3-1)
     & +       F45*coarse( c1+2, c2+1, c3+1)
     & -       F27*coarse( c1+2, c2-1, c3+1)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1  , c2  , c3+1)
     & +     F4500*coarse( c1+2, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1+1, c2-1, c3  )
     & -     F2700*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1+1, c2+1, c3  )
     & +     F1125*coarse( c1  , c2-1, c3+1)
     & +      F750*coarse( c1+2, c2-1, c3  )
     & +      F750*coarse( c1+2, c2  , c3+1)
     & -      F675*coarse( c1  , c2-1, c3-1)
     & -      F675*coarse( c1  , c2+1, c3+1)
     & -      F450*coarse( c1+2, c2  , c3-1)
     & -      F450*coarse( c1+2, c2+1, c3  )
     & -      F450*coarse( c1+1, c2-1, c3+1)
     & +      F405*coarse( c1  , c2+1, c3-1)
     & +      F270*coarse( c1+1, c2-1, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3+1)
     & -      F162*coarse( c1+1, c2+1, c3-1)
     & +      F125*coarse( c1+2, c2-1, c3+1)
     & -       F75*coarse( c1+2, c2-1, c3-1)
     & -       F75*coarse( c1+2, c2+1, c3+1)
     & +       F45*coarse( c1+2, c2+1, c3-1)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1+1, c2  , c3  )
     & +     F3150*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1+2, c2  , c3  )
     & +     F2100*coarse( c1+1, c2-1, c3  )
     & +     F2100*coarse( c1+1, c2  , c3+1)
     & -     F1890*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1+1, c2  , c3-1)
     & -     F1260*coarse( c1+1, c2+1, c3  )
     & +      F525*coarse( c1  , c2-1, c3+1)
     & -      F450*coarse( c1+2, c2-1, c3  )
     & -      F450*coarse( c1+2, c2  , c3+1)
     & +      F350*coarse( c1+1, c2-1, c3+1)
     & -      F315*coarse( c1  , c2-1, c3-1)
     & -      F315*coarse( c1  , c2+1, c3+1)
     & +      F270*coarse( c1+2, c2  , c3-1)
     & +      F270*coarse( c1+2, c2+1, c3  )
     & -      F210*coarse( c1+1, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3+1)
     & +      F189*coarse( c1  , c2+1, c3-1)
     & +      F126*coarse( c1+1, c2+1, c3-1)
     & -       F75*coarse( c1+2, c2-1, c3+1)
     & +       F45*coarse( c1+2, c2-1, c3-1)
     & +       F45*coarse( c1+2, c2+1, c3+1)
     & -       F27*coarse( c1+2, c2+1, c3-1)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+1, c3  )
     & +     F6750*coarse( c1  , c2  , c3+1)
     & +     F4500*coarse( c1+2, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1  , c2-1, c3  )
     & -     F2700*coarse( c1+1, c2+1, c3  )
     & -     F2700*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1+1, c2-1, c3  )
     & +     F1125*coarse( c1  , c2+1, c3+1)
     & +      F750*coarse( c1+2, c2+1, c3  )
     & +      F750*coarse( c1+2, c2  , c3+1)
     & -      F675*coarse( c1  , c2+1, c3-1)
     & -      F675*coarse( c1  , c2-1, c3+1)
     & -      F450*coarse( c1+2, c2  , c3-1)
     & -      F450*coarse( c1+2, c2-1, c3  )
     & -      F450*coarse( c1+1, c2+1, c3+1)
     & +      F405*coarse( c1  , c2-1, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3-1)
     & +      F270*coarse( c1+1, c2-1, c3+1)
     & -      F162*coarse( c1+1, c2-1, c3-1)
     & +      F125*coarse( c1+2, c2+1, c3+1)
     & -       F75*coarse( c1+2, c2+1, c3-1)
     & -       F75*coarse( c1+2, c2-1, c3+1)
     & +       F45*coarse( c1+2, c2-1, c3-1)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1+1, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1+2, c2  , c3  )
     & +     F2100*coarse( c1+1, c2+1, c3  )
     & +     F2100*coarse( c1+1, c2  , c3+1)
     & -     F1890*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1  , c2-1, c3  )
     & -     F1260*coarse( c1+1, c2  , c3-1)
     & -     F1260*coarse( c1+1, c2-1, c3  )
     & +      F525*coarse( c1  , c2+1, c3+1)
     & -      F450*coarse( c1+2, c2+1, c3  )
     & -      F450*coarse( c1+2, c2  , c3+1)
     & +      F350*coarse( c1+1, c2+1, c3+1)
     & -      F315*coarse( c1  , c2+1, c3-1)
     & -      F315*coarse( c1  , c2-1, c3+1)
     & +      F270*coarse( c1+2, c2  , c3-1)
     & +      F270*coarse( c1+2, c2-1, c3  )
     & -      F210*coarse( c1+1, c2+1, c3-1)
     & -      F210*coarse( c1+1, c2-1, c3+1)
     & +      F189*coarse( c1  , c2-1, c3-1)
     & +      F126*coarse( c1+1, c2-1, c3-1)
     & -       F75*coarse( c1+2, c2+1, c3+1)
     & +       F45*coarse( c1+2, c2+1, c3-1)
     & +       F45*coarse( c1+2, c2-1, c3+1)
     & -       F27*coarse( c1+2, c2-1, c3-1)
     & )
            f21 = f21 + 2
          end do
          f31 = f31 + 2
        end do
c
c Set cint(1)-cint(6) edge
        if(c3f.lt.cint(6)) then
          c3 = cint(6)
          f31 = 2*c3
          f21 = f2i
          do c2 = c2i,c2f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1+1, c2  , c3  )
     & -     F7560*coarse( c1+1, c2  , c3-1)
     & +     F4725*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1  , c2-1, c3-1)
     & +     F3150*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2+1, c3  )
     & +     F2100*coarse( c1+2, c2  , c3-1)
     & -     F1890*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1+1, c2  , c3-2)
     & -     F1260*coarse( c1+1, c2-1, c3-1)
     & +     F1134*coarse( c1+1, c2+1, c3  )
     & +      F756*coarse( c1+1, c2+1, c3-1)
     & -      F675*coarse( c1  , c2-1, c3-2)
     & +      F525*coarse( c1+2, c2-1, c3  )
     & -      F450*coarse( c1+2, c2  , c3-2)
     & +      F405*coarse( c1  , c2+1, c3-2)
     & +      F350*coarse( c1+2, c2-1, c3-1)
     & -      F315*coarse( c1+2, c2+1, c3  )
     & +      F270*coarse( c1+1, c2-1, c3-2)
     & -      F210*coarse( c1+2, c2+1, c3-1)
     & -      F162*coarse( c1+1, c2+1, c3-2)
     & -       F75*coarse( c1+2, c2-1, c3-2)
     & +       F45*coarse( c1+2, c2+1, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1+1, c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3-1)
     & +     F5880*coarse( c1+1, c2  , c3-1)
     & +     F2205*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3-2)
     & +     F1470*coarse( c1+1, c2-1, c3  )
     & +     F1470*coarse( c1  , c2-1, c3-1)
     & -     F1323*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1+2, c2  , c3-1)
     & -     F1260*coarse( c1+1, c2  , c3-2)
     & +      F980*coarse( c1+1, c2-1, c3-1)
     & -      F882*coarse( c1+1, c2+1, c3  )
     & -      F882*coarse( c1  , c2+1, c3-1)
     & -      F588*coarse( c1+1, c2+1, c3-1)
     & -      F315*coarse( c1+2, c2-1, c3  )
     & -      F315*coarse( c1  , c2-1, c3-2)
     & +      F270*coarse( c1+2, c2  , c3-2)
     & -      F210*coarse( c1+2, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2-1, c3-2)
     & +      F189*coarse( c1+2, c2+1, c3  )
     & +      F189*coarse( c1  , c2+1, c3-2)
     & +      F126*coarse( c1+2, c2+1, c3-1)
     & +      F126*coarse( c1+1, c2+1, c3-2)
     & +       F45*coarse( c1+2, c2-1, c3-2)
     & -       F27*coarse( c1+2, c2+1, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1+1, c2  , c3  )
     & -     F7560*coarse( c1+1, c2  , c3-1)
     & +     F4725*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3-1)
     & -     F2835*coarse( c1  , c2-1, c3  )
     & +     F2100*coarse( c1+2, c2  , c3-1)
     & -     F1890*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1+1, c2  , c3-2)
     & -     F1260*coarse( c1+1, c2+1, c3-1)
     & +     F1134*coarse( c1+1, c2-1, c3  )
     & +      F756*coarse( c1+1, c2-1, c3-1)
     & -      F675*coarse( c1  , c2+1, c3-2)
     & +      F525*coarse( c1+2, c2+1, c3  )
     & -      F450*coarse( c1+2, c2  , c3-2)
     & +      F405*coarse( c1  , c2-1, c3-2)
     & +      F350*coarse( c1+2, c2+1, c3-1)
     & -      F315*coarse( c1+2, c2-1, c3  )
     & +      F270*coarse( c1+1, c2+1, c3-2)
     & -      F210*coarse( c1+2, c2-1, c3-1)
     & -      F162*coarse( c1+1, c2-1, c3-2)
     & -       F75*coarse( c1+2, c2+1, c3-2)
     & +       F45*coarse( c1+2, c2-1, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1+1, c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3-1)
     & +     F5880*coarse( c1+1, c2  , c3-1)
     & +     F2205*coarse( c1  , c2+1, c3  )
     & -     F1890*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3-2)
     & +     F1470*coarse( c1+1, c2+1, c3  )
     & +     F1470*coarse( c1  , c2+1, c3-1)
     & -     F1323*coarse( c1  , c2-1, c3  )
     & -     F1260*coarse( c1+2, c2  , c3-1)
     & -     F1260*coarse( c1+1, c2  , c3-2)
     & +      F980*coarse( c1+1, c2+1, c3-1)
     & -      F882*coarse( c1+1, c2-1, c3  )
     & -      F882*coarse( c1  , c2-1, c3-1)
     & -      F588*coarse( c1+1, c2-1, c3-1)
     & -      F315*coarse( c1+2, c2+1, c3  )
     & -      F315*coarse( c1  , c2+1, c3-2)
     & +      F270*coarse( c1+2, c2  , c3-2)
     & -      F210*coarse( c1+2, c2+1, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3-2)
     & +      F189*coarse( c1+2, c2-1, c3  )
     & +      F189*coarse( c1  , c2-1, c3-2)
     & +      F126*coarse( c1+2, c2-1, c3-1)
     & +      F126*coarse( c1+1, c2-1, c3-2)
     & +       F45*coarse( c1+2, c2+1, c3-2)
     & -       F27*coarse( c1+2, c2-1, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1+1, c2  , c3  )
     & -    F24300*coarse( c1  , c2  , c3-1)
     & +    F10125*coarse( c1  , c2-1, c3  )
     & +     F9720*coarse( c1+1, c2  , c3-1)
     & +     F6750*coarse( c1+2, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3-2)
     & -     F6075*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1+1, c2-1, c3  )
     & -     F4050*coarse( c1  , c2-1, c3-1)
     & -     F2700*coarse( c1+2, c2  , c3-1)
     & -     F2700*coarse( c1+1, c2  , c3-2)
     & +     F2430*coarse( c1+1, c2+1, c3  )
     & +     F2430*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1+1, c2-1, c3-1)
     & +     F1125*coarse( c1+2, c2-1, c3  )
     & +     F1125*coarse( c1  , c2-1, c3-2)
     & -      F972*coarse( c1+1, c2+1, c3-1)
     & +      F750*coarse( c1+2, c2  , c3-2)
     & -      F675*coarse( c1+2, c2+1, c3  )
     & -      F675*coarse( c1  , c2+1, c3-2)
     & -      F450*coarse( c1+2, c2-1, c3-1)
     & -      F450*coarse( c1+1, c2-1, c3-2)
     & +      F270*coarse( c1+2, c2+1, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3-2)
     & +      F125*coarse( c1+2, c2-1, c3-2)
     & -       F75*coarse( c1+2, c2+1, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1+1, c2  , c3  )
     & -    F11340*coarse( c1  , c2  , c3-1)
     & -     F7560*coarse( c1+1, c2  , c3-1)
     & +     F4725*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1+1, c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-2)
     & -     F2835*coarse( c1  , c2+1, c3  )
     & +     F2100*coarse( c1+1, c2  , c3-2)
     & -     F1890*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1+2, c2  , c3-1)
     & -     F1260*coarse( c1+1, c2-1, c3-1)
     & +     F1134*coarse( c1  , c2+1, c3-1)
     & +      F756*coarse( c1+1, c2+1, c3-1)
     & -      F675*coarse( c1+2, c2-1, c3  )
     & +      F525*coarse( c1  , c2-1, c3-2)
     & -      F450*coarse( c1+2, c2  , c3-2)
     & +      F405*coarse( c1+2, c2+1, c3  )
     & +      F350*coarse( c1+1, c2-1, c3-2)
     & -      F315*coarse( c1  , c2+1, c3-2)
     & +      F270*coarse( c1+2, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3-2)
     & -      F162*coarse( c1+2, c2+1, c3-1)
     & -       F75*coarse( c1+2, c2-1, c3-2)
     & +       F45*coarse( c1+2, c2+1, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2  , c3-1)
     & -    F24300*coarse( c1+1, c2  , c3  )
     & +    F10125*coarse( c1  , c2+1, c3  )
     & +     F9720*coarse( c1+1, c2  , c3-1)
     & +     F6750*coarse( c1+2, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3-2)
     & -     F6075*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1+1, c2+1, c3  )
     & -     F4050*coarse( c1  , c2+1, c3-1)
     & -     F2700*coarse( c1+2, c2  , c3-1)
     & -     F2700*coarse( c1+1, c2  , c3-2)
     & +     F2430*coarse( c1+1, c2-1, c3  )
     & +     F2430*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1+1, c2+1, c3-1)
     & +     F1125*coarse( c1+2, c2+1, c3  )
     & +     F1125*coarse( c1  , c2+1, c3-2)
     & -      F972*coarse( c1+1, c2-1, c3-1)
     & +      F750*coarse( c1+2, c2  , c3-2)
     & -      F675*coarse( c1+2, c2-1, c3  )
     & -      F675*coarse( c1  , c2-1, c3-2)
     & -      F450*coarse( c1+2, c2+1, c3-1)
     & -      F450*coarse( c1+1, c2+1, c3-2)
     & +      F270*coarse( c1+2, c2-1, c3-1)
     & +      F270*coarse( c1+1, c2-1, c3-2)
     & +      F125*coarse( c1+2, c2+1, c3-2)
     & -       F75*coarse( c1+2, c2-1, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1+1, c2  , c3  )
     & -    F11340*coarse( c1  , c2  , c3-1)
     & -     F7560*coarse( c1+1, c2  , c3-1)
     & +     F4725*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1+1, c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-2)
     & -     F2835*coarse( c1  , c2-1, c3  )
     & +     F2100*coarse( c1+1, c2  , c3-2)
     & -     F1890*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1+2, c2  , c3-1)
     & -     F1260*coarse( c1+1, c2+1, c3-1)
     & +     F1134*coarse( c1  , c2-1, c3-1)
     & +      F756*coarse( c1+1, c2-1, c3-1)
     & -      F675*coarse( c1+2, c2+1, c3  )
     & +      F525*coarse( c1  , c2+1, c3-2)
     & -      F450*coarse( c1+2, c2  , c3-2)
     & +      F405*coarse( c1+2, c2-1, c3  )
     & +      F350*coarse( c1+1, c2+1, c3-2)
     & -      F315*coarse( c1  , c2-1, c3-2)
     & +      F270*coarse( c1+2, c2+1, c3-1)
     & -      F210*coarse( c1+1, c2-1, c3-2)
     & -      F162*coarse( c1+2, c2-1, c3-1)
     & -       F75*coarse( c1+2, c2+1, c3-2)
     & +       F45*coarse( c1+2, c2-1, c3-2)
     & )
            f21 = f21 + 2
          end do
        end if
c
c Set cint(1)--cint(5) corner
        if(c2f.lt.cint(5)) then
          c2 = cint(5)
          f21 = 2*c2
c
c Set cint(1)-cint(5)-cint(3) corner
          if(c3i.gt.cint(3)) then
            c3 = cint(3)
            f31 = 2*c3
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2-1, c3  )
     & -    F17010*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1+1, c2-1, c3  )
     & -    F11340*coarse( c1  , c2-1, c3+1)
     & +     F6804*coarse( c1+1, c2  , c3+1)
     & -     F6075*coarse( c1  , c2-2, c3  )
     & +     F4725*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2  , c3+2)
     & +     F4536*coarse( c1+1, c2-1, c3+1)
     & +     F3150*coarse( c1+2, c2-1, c3  )
     & +     F3150*coarse( c1  , c2-1, c3+2)
     & +     F2430*coarse( c1+1, c2-2, c3  )
     & +     F2430*coarse( c1  , c2-2, c3+1)
     & -     F1890*coarse( c1+2, c2  , c3+1)
     & -     F1890*coarse( c1+1, c2  , c3+2)
     & -     F1260*coarse( c1+2, c2-1, c3+1)
     & -     F1260*coarse( c1+1, c2-1, c3+2)
     & -      F972*coarse( c1+1, c2-2, c3+1)
     & -      F675*coarse( c1+2, c2-2, c3  )
     & -      F675*coarse( c1  , c2-2, c3+2)
     & +      F525*coarse( c1+2, c2  , c3+2)
     & +      F350*coarse( c1+2, c2-1, c3+2)
     & +      F270*coarse( c1+2, c2-2, c3+1)
     & +      F270*coarse( c1+1, c2-2, c3+2)
     & -       F75*coarse( c1+2, c2-2, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1+1, c2  , c3  )
     & +    F13230*coarse( c1  , c2-1, c3  )
     & +     F8820*coarse( c1+1, c2-1, c3  )
     & -     F7938*coarse( c1  , c2  , c3+1)
     & -     F5292*coarse( c1+1, c2  , c3+1)
     & -     F5292*coarse( c1  , c2-1, c3+1)
     & -     F3528*coarse( c1+1, c2-1, c3+1)
     & -     F2835*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2-2, c3  )
     & +     F2205*coarse( c1  , c2  , c3+2)
     & -     F1890*coarse( c1+2, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2-2, c3  )
     & +     F1470*coarse( c1+1, c2  , c3+2)
     & +     F1470*coarse( c1  , c2-1, c3+2)
     & +     F1134*coarse( c1+2, c2  , c3+1)
     & +     F1134*coarse( c1  , c2-2, c3+1)
     & +      F980*coarse( c1+1, c2-1, c3+2)
     & +      F756*coarse( c1+2, c2-1, c3+1)
     & +      F756*coarse( c1+1, c2-2, c3+1)
     & +      F405*coarse( c1+2, c2-2, c3  )
     & -      F315*coarse( c1+2, c2  , c3+2)
     & -      F315*coarse( c1  , c2-2, c3+2)
     & -      F210*coarse( c1+2, c2-1, c3+2)
     & -      F210*coarse( c1+1, c2-2, c3+2)
     & -      F162*coarse( c1+2, c2-2, c3+1)
     & +       F45*coarse( c1+2, c2-2, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F91125*coarse( c1  , c2  , c3  )
     & -    F36450*coarse( c1+1, c2  , c3  )
     & -    F36450*coarse( c1  , c2-1, c3  )
     & -    F36450*coarse( c1  , c2  , c3+1)
     & +    F14580*coarse( c1+1, c2-1, c3  )
     & +    F14580*coarse( c1+1, c2  , c3+1)
     & +    F14580*coarse( c1  , c2-1, c3+1)
     & +    F10125*coarse( c1+2, c2  , c3  )
     & +    F10125*coarse( c1  , c2-2, c3  )
     & +    F10125*coarse( c1  , c2  , c3+2)
     & -     F5832*coarse( c1+1, c2-1, c3+1)
     & -     F4050*coarse( c1+2, c2-1, c3  )
     & -     F4050*coarse( c1+1, c2-2, c3  )
     & -     F4050*coarse( c1+2, c2  , c3+1)
     & -     F4050*coarse( c1  , c2-2, c3+1)
     & -     F4050*coarse( c1+1, c2  , c3+2)
     & -     F4050*coarse( c1  , c2-1, c3+2)
     & +     F1620*coarse( c1+2, c2-1, c3+1)
     & +     F1620*coarse( c1+1, c2-2, c3+1)
     & +     F1620*coarse( c1+1, c2-1, c3+2)
     & +     F1125*coarse( c1+2, c2-2, c3  )
     & +     F1125*coarse( c1+2, c2  , c3+2)
     & +     F1125*coarse( c1  , c2-2, c3+2)
     & -      F450*coarse( c1+2, c2-2, c3+1)
     & -      F450*coarse( c1+2, c2-1, c3+2)
     & -      F450*coarse( c1+1, c2-2, c3+2)
     & +      F125*coarse( c1+2, c2-2, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2-1, c3  )
     & -    F17010*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1+1, c2-1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3+1)
     & +     F6804*coarse( c1  , c2-1, c3+1)
     & -     F6075*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2-2, c3  )
     & +     F4725*coarse( c1  , c2  , c3+2)
     & +     F4536*coarse( c1+1, c2-1, c3+1)
     & +     F3150*coarse( c1+1, c2-2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3+2)
     & +     F2430*coarse( c1+2, c2-1, c3  )
     & +     F2430*coarse( c1+2, c2  , c3+1)
     & -     F1890*coarse( c1  , c2-2, c3+1)
     & -     F1890*coarse( c1  , c2-1, c3+2)
     & -     F1260*coarse( c1+1, c2-2, c3+1)
     & -     F1260*coarse( c1+1, c2-1, c3+2)
     & -      F972*coarse( c1+2, c2-1, c3+1)
     & -      F675*coarse( c1+2, c2-2, c3  )
     & -      F675*coarse( c1+2, c2  , c3+2)
     & +      F525*coarse( c1  , c2-2, c3+2)
     & +      F350*coarse( c1+1, c2-2, c3+2)
     & +      F270*coarse( c1+2, c2-2, c3+1)
     & +      F270*coarse( c1+2, c2-1, c3+2)
     & -       F75*coarse( c1+2, c2-2, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1  , c2-1, c3  )
     & +    F13230*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1  , c2-1, c3+1)
     & -     F7938*coarse( c1+1, c2  , c3  )
     & -     F5292*coarse( c1+1, c2-1, c3  )
     & -     F5292*coarse( c1+1, c2  , c3+1)
     & -     F3528*coarse( c1+1, c2-1, c3+1)
     & -     F2835*coarse( c1  , c2-2, c3  )
     & -     F2835*coarse( c1  , c2  , c3+2)
     & +     F2205*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3+1)
     & -     F1890*coarse( c1  , c2-1, c3+2)
     & +     F1470*coarse( c1+2, c2-1, c3  )
     & +     F1470*coarse( c1+2, c2  , c3+1)
     & +     F1134*coarse( c1+1, c2-2, c3  )
     & +     F1134*coarse( c1+1, c2  , c3+2)
     & +      F980*coarse( c1+2, c2-1, c3+1)
     & +      F756*coarse( c1+1, c2-2, c3+1)
     & +      F756*coarse( c1+1, c2-1, c3+2)
     & +      F405*coarse( c1  , c2-2, c3+2)
     & -      F315*coarse( c1+2, c2-2, c3  )
     & -      F315*coarse( c1+2, c2  , c3+2)
     & -      F210*coarse( c1+2, c2-2, c3+1)
     & -      F210*coarse( c1+2, c2-1, c3+2)
     & -      F162*coarse( c1+1, c2-2, c3+2)
     & +       F45*coarse( c1+2, c2-2, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &       F9261*coarse( c1  , c2  , c3  )
     & +     F6174*coarse( c1+1, c2  , c3  )
     & +     F6174*coarse( c1  , c2-1, c3  )
     & +     F6174*coarse( c1  , c2  , c3+1)
     & +     F4116*coarse( c1+1, c2-1, c3  )
     & +     F4116*coarse( c1+1, c2  , c3+1)
     & +     F4116*coarse( c1  , c2-1, c3+1)
     & +     F2744*coarse( c1+1, c2-1, c3+1)
     & -     F1323*coarse( c1+2, c2  , c3  )
     & -     F1323*coarse( c1  , c2-2, c3  )
     & -     F1323*coarse( c1  , c2  , c3+2)
     & -      F882*coarse( c1+2, c2-1, c3  )
     & -      F882*coarse( c1+1, c2-2, c3  )
     & -      F882*coarse( c1+2, c2  , c3+1)
     & -      F882*coarse( c1  , c2-2, c3+1)
     & -      F882*coarse( c1+1, c2  , c3+2)
     & -      F882*coarse( c1  , c2-1, c3+2)
     & -      F588*coarse( c1+2, c2-1, c3+1)
     & -      F588*coarse( c1+1, c2-2, c3+1)
     & -      F588*coarse( c1+1, c2-1, c3+2)
     & +      F189*coarse( c1+2, c2-2, c3  )
     & +      F189*coarse( c1+2, c2  , c3+2)
     & +      F189*coarse( c1  , c2-2, c3+2)
     & +      F126*coarse( c1+2, c2-2, c3+1)
     & +      F126*coarse( c1+2, c2-1, c3+2)
     & +      F126*coarse( c1+1, c2-2, c3+2)
     & -       F27*coarse( c1+2, c2-2, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2  , c3+1)
     & -    F17010*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3+1)
     & -    F11340*coarse( c1  , c2-1, c3+1)
     & +     F6804*coarse( c1+1, c2-1, c3  )
     & -     F6075*coarse( c1  , c2  , c3+2)
     & +     F4725*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2-2, c3  )
     & +     F4536*coarse( c1+1, c2-1, c3+1)
     & +     F3150*coarse( c1+2, c2  , c3+1)
     & +     F3150*coarse( c1  , c2-2, c3+1)
     & +     F2430*coarse( c1+1, c2  , c3+2)
     & +     F2430*coarse( c1  , c2-1, c3+2)
     & -     F1890*coarse( c1+2, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2-2, c3  )
     & -     F1260*coarse( c1+2, c2-1, c3+1)
     & -     F1260*coarse( c1+1, c2-2, c3+1)
     & -      F972*coarse( c1+1, c2-1, c3+2)
     & -      F675*coarse( c1+2, c2  , c3+2)
     & -      F675*coarse( c1  , c2-2, c3+2)
     & +      F525*coarse( c1+2, c2-2, c3  )
     & +      F350*coarse( c1+2, c2-2, c3+1)
     & +      F270*coarse( c1+2, c2-1, c3+2)
     & +      F270*coarse( c1+1, c2-2, c3+2)
     & -       F75*coarse( c1+2, c2-2, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1+1, c2  , c3  )
     & +    F13230*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1+1, c2  , c3+1)
     & -     F7938*coarse( c1  , c2-1, c3  )
     & -     F5292*coarse( c1+1, c2-1, c3  )
     & -     F5292*coarse( c1  , c2-1, c3+1)
     & -     F3528*coarse( c1+1, c2-1, c3+1)
     & -     F2835*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3+2)
     & +     F2205*coarse( c1  , c2-2, c3  )
     & -     F1890*coarse( c1+2, c2  , c3+1)
     & -     F1890*coarse( c1+1, c2  , c3+2)
     & +     F1470*coarse( c1+1, c2-2, c3  )
     & +     F1470*coarse( c1  , c2-2, c3+1)
     & +     F1134*coarse( c1+2, c2-1, c3  )
     & +     F1134*coarse( c1  , c2-1, c3+2)
     & +      F980*coarse( c1+1, c2-2, c3+1)
     & +      F756*coarse( c1+2, c2-1, c3+1)
     & +      F756*coarse( c1+1, c2-1, c3+2)
     & +      F405*coarse( c1+2, c2  , c3+2)
     & -      F315*coarse( c1+2, c2-2, c3  )
     & -      F315*coarse( c1  , c2-2, c3+2)
     & -      F210*coarse( c1+2, c2-2, c3+1)
     & -      F210*coarse( c1+1, c2-2, c3+2)
     & -      F162*coarse( c1+2, c2-1, c3+2)
     & +       F45*coarse( c1+2, c2-2, c3+2)
     & )
          end if
c
c Set cint(1)-cint(5) edge
          f31 = f3i
          do c3 = c3i,c3f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3  )
     & -     F7560*coarse( c1+1, c2-1, c3  )
     & +     F4725*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1  , c2-1, c3-1)
     & +     F3150*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3+1)
     & +     F2100*coarse( c1+2, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3-1)
     & -     F1890*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1+1, c2-2, c3  )
     & -     F1260*coarse( c1+1, c2-1, c3-1)
     & +     F1134*coarse( c1+1, c2  , c3+1)
     & +      F756*coarse( c1+1, c2-1, c3+1)
     & -      F675*coarse( c1  , c2-2, c3-1)
     & +      F525*coarse( c1+2, c2  , c3-1)
     & -      F450*coarse( c1+2, c2-2, c3  )
     & +      F405*coarse( c1  , c2-2, c3+1)
     & +      F350*coarse( c1+2, c2-1, c3-1)
     & -      F315*coarse( c1+2, c2  , c3+1)
     & +      F270*coarse( c1+1, c2-2, c3-1)
     & -      F210*coarse( c1+2, c2-1, c3+1)
     & -      F162*coarse( c1+1, c2-2, c3+1)
     & -       F75*coarse( c1+2, c2-2, c3-1)
     & +       F45*coarse( c1+2, c2-2, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     & +    F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1+1, c2  , c3  )
     & +     F8820*coarse( c1  , c2-1, c3  )
     & +     F5880*coarse( c1+1, c2-1, c3  )
     & +     F2205*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3  )
     & +     F1470*coarse( c1+1, c2  , c3-1)
     & +     F1470*coarse( c1  , c2-1, c3-1)
     & -     F1323*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1+2, c2-1, c3  )
     & -     F1260*coarse( c1+1, c2-2, c3  )
     & +      F980*coarse( c1+1, c2-1, c3-1)
     & -      F882*coarse( c1+1, c2  , c3+1)
     & -      F882*coarse( c1  , c2-1, c3+1)
     & -      F588*coarse( c1+1, c2-1, c3+1)
     & -      F315*coarse( c1+2, c2  , c3-1)
     & -      F315*coarse( c1  , c2-2, c3-1)
     & +      F270*coarse( c1+2, c2-2, c3  )
     & -      F210*coarse( c1+2, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2-2, c3-1)
     & +      F189*coarse( c1+2, c2  , c3+1)
     & +      F189*coarse( c1  , c2-2, c3+1)
     & +      F126*coarse( c1+2, c2-1, c3+1)
     & +      F126*coarse( c1+1, c2-2, c3+1)
     & +       F45*coarse( c1+2, c2-2, c3-1)
     & -       F27*coarse( c1+2, c2-2, c3+1)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     & +    F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1+1, c2  , c3  )
     & -    F24300*coarse( c1  , c2-1, c3  )
     & +    F10125*coarse( c1  , c2  , c3-1)
     & +     F9720*coarse( c1+1, c2-1, c3  )
     & +     F6750*coarse( c1+2, c2  , c3  )
     & +     F6750*coarse( c1  , c2-2, c3  )
     & -     F6075*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1+1, c2  , c3-1)
     & -     F4050*coarse( c1  , c2-1, c3-1)
     & -     F2700*coarse( c1+2, c2-1, c3  )
     & -     F2700*coarse( c1+1, c2-2, c3  )
     & +     F2430*coarse( c1+1, c2  , c3+1)
     & +     F2430*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1+1, c2-1, c3-1)
     & +     F1125*coarse( c1+2, c2  , c3-1)
     & +     F1125*coarse( c1  , c2-2, c3-1)
     & -      F972*coarse( c1+1, c2-1, c3+1)
     & +      F750*coarse( c1+2, c2-2, c3  )
     & -      F675*coarse( c1+2, c2  , c3+1)
     & -      F675*coarse( c1  , c2-2, c3+1)
     & -      F450*coarse( c1+2, c2-1, c3-1)
     & -      F450*coarse( c1+1, c2-2, c3-1)
     & +      F270*coarse( c1+2, c2-1, c3+1)
     & +      F270*coarse( c1+1, c2-2, c3+1)
     & +      F125*coarse( c1+2, c2-2, c3-1)
     & -       F75*coarse( c1+2, c2-2, c3+1)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1+1, c2  , c3  )
     & -    F11340*coarse( c1  , c2-1, c3  )
     & -     F7560*coarse( c1+1, c2-1, c3  )
     & +     F4725*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1+1, c2  , c3-1)
     & +     F3150*coarse( c1  , c2-2, c3  )
     & -     F2835*coarse( c1  , c2  , c3+1)
     & +     F2100*coarse( c1+1, c2-2, c3  )
     & -     F1890*coarse( c1  , c2-1, c3-1)
     & -     F1890*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1+2, c2-1, c3  )
     & -     F1260*coarse( c1+1, c2-1, c3-1)
     & +     F1134*coarse( c1  , c2-1, c3+1)
     & +      F756*coarse( c1+1, c2-1, c3+1)
     & -      F675*coarse( c1+2, c2  , c3-1)
     & +      F525*coarse( c1  , c2-2, c3-1)
     & -      F450*coarse( c1+2, c2-2, c3  )
     & +      F405*coarse( c1+2, c2  , c3+1)
     & +      F350*coarse( c1+1, c2-2, c3-1)
     & -      F315*coarse( c1  , c2-2, c3+1)
     & +      F270*coarse( c1+2, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2-2, c3+1)
     & -      F162*coarse( c1+2, c2-1, c3+1)
     & -       F75*coarse( c1+2, c2-2, c3-1)
     & +       F45*coarse( c1+2, c2-2, c3+1)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3  )
     & -     F7560*coarse( c1+1, c2-1, c3  )
     & +     F4725*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1  , c2-1, c3+1)
     & -     F2835*coarse( c1  , c2  , c3-1)
     & +     F2100*coarse( c1+2, c2-1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3-1)
     & -     F1890*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1+1, c2-2, c3  )
     & -     F1260*coarse( c1+1, c2-1, c3+1)
     & +     F1134*coarse( c1+1, c2  , c3-1)
     & +      F756*coarse( c1+1, c2-1, c3-1)
     & -      F675*coarse( c1  , c2-2, c3+1)
     & +      F525*coarse( c1+2, c2  , c3+1)
     & -      F450*coarse( c1+2, c2-2, c3  )
     & +      F405*coarse( c1  , c2-2, c3-1)
     & +      F350*coarse( c1+2, c2-1, c3+1)
     & -      F315*coarse( c1+2, c2  , c3-1)
     & +      F270*coarse( c1+1, c2-2, c3+1)
     & -      F210*coarse( c1+2, c2-1, c3-1)
     & -      F162*coarse( c1+1, c2-2, c3-1)
     & -       F75*coarse( c1+2, c2-2, c3+1)
     & +       F45*coarse( c1+2, c2-2, c3-1)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     & +    F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1+1, c2  , c3  )
     & +     F8820*coarse( c1  , c2-1, c3  )
     & +     F5880*coarse( c1+1, c2-1, c3  )
     & +     F2205*coarse( c1  , c2  , c3+1)
     & -     F1890*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3  )
     & +     F1470*coarse( c1+1, c2  , c3+1)
     & +     F1470*coarse( c1  , c2-1, c3+1)
     & -     F1323*coarse( c1  , c2  , c3-1)
     & -     F1260*coarse( c1+2, c2-1, c3  )
     & -     F1260*coarse( c1+1, c2-2, c3  )
     & +      F980*coarse( c1+1, c2-1, c3+1)
     & -      F882*coarse( c1+1, c2  , c3-1)
     & -      F882*coarse( c1  , c2-1, c3-1)
     & -      F588*coarse( c1+1, c2-1, c3-1)
     & -      F315*coarse( c1+2, c2  , c3+1)
     & -      F315*coarse( c1  , c2-2, c3+1)
     & +      F270*coarse( c1+2, c2-2, c3  )
     & -      F210*coarse( c1+2, c2-1, c3+1)
     & -      F210*coarse( c1+1, c2-2, c3+1)
     & +      F189*coarse( c1+2, c2  , c3-1)
     & +      F189*coarse( c1  , c2-2, c3-1)
     & +      F126*coarse( c1+2, c2-1, c3-1)
     & +      F126*coarse( c1+1, c2-2, c3-1)
     & +       F45*coarse( c1+2, c2-2, c3+1)
     & -       F27*coarse( c1+2, c2-2, c3-1)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     & +    F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1+1, c2  , c3  )
     & -    F24300*coarse( c1  , c2-1, c3  )
     & +    F10125*coarse( c1  , c2  , c3+1)
     & +     F9720*coarse( c1+1, c2-1, c3  )
     & +     F6750*coarse( c1+2, c2  , c3  )
     & +     F6750*coarse( c1  , c2-2, c3  )
     & -     F6075*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1+1, c2  , c3+1)
     & -     F4050*coarse( c1  , c2-1, c3+1)
     & -     F2700*coarse( c1+2, c2-1, c3  )
     & -     F2700*coarse( c1+1, c2-2, c3  )
     & +     F2430*coarse( c1+1, c2  , c3-1)
     & +     F2430*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1+1, c2-1, c3+1)
     & +     F1125*coarse( c1+2, c2  , c3+1)
     & +     F1125*coarse( c1  , c2-2, c3+1)
     & -      F972*coarse( c1+1, c2-1, c3-1)
     & +      F750*coarse( c1+2, c2-2, c3  )
     & -      F675*coarse( c1+2, c2  , c3-1)
     & -      F675*coarse( c1  , c2-2, c3-1)
     & -      F450*coarse( c1+2, c2-1, c3+1)
     & -      F450*coarse( c1+1, c2-2, c3+1)
     & +      F270*coarse( c1+2, c2-1, c3-1)
     & +      F270*coarse( c1+1, c2-2, c3-1)
     & +      F125*coarse( c1+2, c2-2, c3+1)
     & -       F75*coarse( c1+2, c2-2, c3-1)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1+1, c2  , c3  )
     & -    F11340*coarse( c1  , c2-1, c3  )
     & -     F7560*coarse( c1+1, c2-1, c3  )
     & +     F4725*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3+1)
     & -     F2835*coarse( c1  , c2  , c3-1)
     & +     F2100*coarse( c1+1, c2-2, c3  )
     & -     F1890*coarse( c1+1, c2  , c3-1)
     & -     F1890*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1+2, c2-1, c3  )
     & -     F1260*coarse( c1+1, c2-1, c3+1)
     & +     F1134*coarse( c1  , c2-1, c3-1)
     & +      F756*coarse( c1+1, c2-1, c3-1)
     & -      F675*coarse( c1+2, c2  , c3+1)
     & +      F525*coarse( c1  , c2-2, c3+1)
     & -      F450*coarse( c1+2, c2-2, c3  )
     & +      F405*coarse( c1+2, c2  , c3-1)
     & +      F350*coarse( c1+1, c2-2, c3+1)
     & -      F315*coarse( c1  , c2-2, c3-1)
     & +      F270*coarse( c1+2, c2-1, c3+1)
     & -      F210*coarse( c1+1, c2-2, c3-1)
     & -      F162*coarse( c1+2, c2-1, c3-1)
     & -       F75*coarse( c1+2, c2-2, c3+1)
     & +       F45*coarse( c1+2, c2-2, c3-1)
     & )
            f31 = f31 + 2
          end do
c
c Set cint(1)-cint(5)-cint(6) corner
          if(c3f.lt.cint(6)) then
            c3 = cint(6)
            f31 = 2*c3
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1  , c2-1, c3  )
     & +    F13230*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1  , c2-1, c3-1)
     & -     F7938*coarse( c1+1, c2  , c3  )
     & -     F5292*coarse( c1+1, c2-1, c3  )
     & -     F5292*coarse( c1+1, c2  , c3-1)
     & -     F3528*coarse( c1+1, c2-1, c3-1)
     & -     F2835*coarse( c1  , c2-2, c3  )
     & -     F2835*coarse( c1  , c2  , c3-2)
     & +     F2205*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3-1)
     & -     F1890*coarse( c1  , c2-1, c3-2)
     & +     F1470*coarse( c1+2, c2-1, c3  )
     & +     F1470*coarse( c1+2, c2  , c3-1)
     & +     F1134*coarse( c1+1, c2-2, c3  )
     & +     F1134*coarse( c1+1, c2  , c3-2)
     & +      F980*coarse( c1+2, c2-1, c3-1)
     & +      F756*coarse( c1+1, c2-2, c3-1)
     & +      F756*coarse( c1+1, c2-1, c3-2)
     & +      F405*coarse( c1  , c2-2, c3-2)
     & -      F315*coarse( c1+2, c2-2, c3  )
     & -      F315*coarse( c1+2, c2  , c3-2)
     & -      F210*coarse( c1+2, c2-2, c3-1)
     & -      F210*coarse( c1+2, c2-1, c3-2)
     & -      F162*coarse( c1+1, c2-2, c3-2)
     & +       F45*coarse( c1+2, c2-2, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &       F9261*coarse( c1  , c2  , c3  )
     & +     F6174*coarse( c1+1, c2  , c3  )
     & +     F6174*coarse( c1  , c2-1, c3  )
     & +     F6174*coarse( c1  , c2  , c3-1)
     & +     F4116*coarse( c1+1, c2-1, c3  )
     & +     F4116*coarse( c1+1, c2  , c3-1)
     & +     F4116*coarse( c1  , c2-1, c3-1)
     & +     F2744*coarse( c1+1, c2-1, c3-1)
     & -     F1323*coarse( c1+2, c2  , c3  )
     & -     F1323*coarse( c1  , c2-2, c3  )
     & -     F1323*coarse( c1  , c2  , c3-2)
     & -      F882*coarse( c1+2, c2-1, c3  )
     & -      F882*coarse( c1+1, c2-2, c3  )
     & -      F882*coarse( c1+2, c2  , c3-1)
     & -      F882*coarse( c1  , c2-2, c3-1)
     & -      F882*coarse( c1+1, c2  , c3-2)
     & -      F882*coarse( c1  , c2-1, c3-2)
     & -      F588*coarse( c1+2, c2-1, c3-1)
     & -      F588*coarse( c1+1, c2-2, c3-1)
     & -      F588*coarse( c1+1, c2-1, c3-2)
     & +      F189*coarse( c1+2, c2-2, c3  )
     & +      F189*coarse( c1+2, c2  , c3-2)
     & +      F189*coarse( c1  , c2-2, c3-2)
     & +      F126*coarse( c1+2, c2-2, c3-1)
     & +      F126*coarse( c1+2, c2-1, c3-2)
     & +      F126*coarse( c1+1, c2-2, c3-2)
     & -       F27*coarse( c1+2, c2-2, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2  , c3-1)
     & -    F17010*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3-1)
     & -    F11340*coarse( c1  , c2-1, c3-1)
     & +     F6804*coarse( c1+1, c2-1, c3  )
     & -     F6075*coarse( c1  , c2  , c3-2)
     & +     F4725*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2-2, c3  )
     & +     F4536*coarse( c1+1, c2-1, c3-1)
     & +     F3150*coarse( c1+2, c2  , c3-1)
     & +     F3150*coarse( c1  , c2-2, c3-1)
     & +     F2430*coarse( c1+1, c2  , c3-2)
     & +     F2430*coarse( c1  , c2-1, c3-2)
     & -     F1890*coarse( c1+2, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2-2, c3  )
     & -     F1260*coarse( c1+2, c2-1, c3-1)
     & -     F1260*coarse( c1+1, c2-2, c3-1)
     & -      F972*coarse( c1+1, c2-1, c3-2)
     & -      F675*coarse( c1+2, c2  , c3-2)
     & -      F675*coarse( c1  , c2-2, c3-2)
     & +      F525*coarse( c1+2, c2-2, c3  )
     & +      F350*coarse( c1+2, c2-2, c3-1)
     & +      F270*coarse( c1+2, c2-1, c3-2)
     & +      F270*coarse( c1+1, c2-2, c3-2)
     & -       F75*coarse( c1+2, c2-2, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1+1, c2  , c3  )
     & +    F13230*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1+1, c2  , c3-1)
     & -     F7938*coarse( c1  , c2-1, c3  )
     & -     F5292*coarse( c1+1, c2-1, c3  )
     & -     F5292*coarse( c1  , c2-1, c3-1)
     & -     F3528*coarse( c1+1, c2-1, c3-1)
     & -     F2835*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3-2)
     & +     F2205*coarse( c1  , c2-2, c3  )
     & -     F1890*coarse( c1+2, c2  , c3-1)
     & -     F1890*coarse( c1+1, c2  , c3-2)
     & +     F1470*coarse( c1+1, c2-2, c3  )
     & +     F1470*coarse( c1  , c2-2, c3-1)
     & +     F1134*coarse( c1+2, c2-1, c3  )
     & +     F1134*coarse( c1  , c2-1, c3-2)
     & +      F980*coarse( c1+1, c2-2, c3-1)
     & +      F756*coarse( c1+2, c2-1, c3-1)
     & +      F756*coarse( c1+1, c2-1, c3-2)
     & +      F405*coarse( c1+2, c2  , c3-2)
     & -      F315*coarse( c1+2, c2-2, c3  )
     & -      F315*coarse( c1  , c2-2, c3-2)
     & -      F210*coarse( c1+2, c2-2, c3-1)
     & -      F210*coarse( c1+1, c2-2, c3-2)
     & -      F162*coarse( c1+2, c2-1, c3-2)
     & +       F45*coarse( c1+2, c2-2, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2-1, c3  )
     & -    F17010*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1+1, c2-1, c3  )
     & -    F11340*coarse( c1  , c2-1, c3-1)
     & +     F6804*coarse( c1+1, c2  , c3-1)
     & -     F6075*coarse( c1  , c2-2, c3  )
     & +     F4725*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2  , c3-2)
     & +     F4536*coarse( c1+1, c2-1, c3-1)
     & +     F3150*coarse( c1+2, c2-1, c3  )
     & +     F3150*coarse( c1  , c2-1, c3-2)
     & +     F2430*coarse( c1+1, c2-2, c3  )
     & +     F2430*coarse( c1  , c2-2, c3-1)
     & -     F1890*coarse( c1+2, c2  , c3-1)
     & -     F1890*coarse( c1+1, c2  , c3-2)
     & -     F1260*coarse( c1+2, c2-1, c3-1)
     & -     F1260*coarse( c1+1, c2-1, c3-2)
     & -      F972*coarse( c1+1, c2-2, c3-1)
     & -      F675*coarse( c1+2, c2-2, c3  )
     & -      F675*coarse( c1  , c2-2, c3-2)
     & +      F525*coarse( c1+2, c2  , c3-2)
     & +      F350*coarse( c1+2, c2-1, c3-2)
     & +      F270*coarse( c1+2, c2-2, c3-1)
     & +      F270*coarse( c1+1, c2-2, c3-2)
     & -       F75*coarse( c1+2, c2-2, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1+1, c2  , c3  )
     & +    F13230*coarse( c1  , c2-1, c3  )
     & +     F8820*coarse( c1+1, c2-1, c3  )
     & -     F7938*coarse( c1  , c2  , c3-1)
     & -     F5292*coarse( c1+1, c2  , c3-1)
     & -     F5292*coarse( c1  , c2-1, c3-1)
     & -     F3528*coarse( c1+1, c2-1, c3-1)
     & -     F2835*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2-2, c3  )
     & +     F2205*coarse( c1  , c2  , c3-2)
     & -     F1890*coarse( c1+2, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2-2, c3  )
     & +     F1470*coarse( c1+1, c2  , c3-2)
     & +     F1470*coarse( c1  , c2-1, c3-2)
     & +     F1134*coarse( c1+2, c2  , c3-1)
     & +     F1134*coarse( c1  , c2-2, c3-1)
     & +      F980*coarse( c1+1, c2-1, c3-2)
     & +      F756*coarse( c1+2, c2-1, c3-1)
     & +      F756*coarse( c1+1, c2-2, c3-1)
     & +      F405*coarse( c1+2, c2-2, c3  )
     & -      F315*coarse( c1+2, c2  , c3-2)
     & -      F315*coarse( c1  , c2-2, c3-2)
     & -      F210*coarse( c1+2, c2-1, c3-2)
     & -      F210*coarse( c1+1, c2-2, c3-2)
     & -      F162*coarse( c1+2, c2-2, c3-1)
     & +       F45*coarse( c1+2, c2-2, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F91125*coarse( c1  , c2  , c3  )
     & -    F36450*coarse( c1+1, c2  , c3  )
     & -    F36450*coarse( c1  , c2-1, c3  )
     & -    F36450*coarse( c1  , c2  , c3-1)
     & +    F14580*coarse( c1+1, c2-1, c3  )
     & +    F14580*coarse( c1+1, c2  , c3-1)
     & +    F14580*coarse( c1  , c2-1, c3-1)
     & +    F10125*coarse( c1+2, c2  , c3  )
     & +    F10125*coarse( c1  , c2-2, c3  )
     & +    F10125*coarse( c1  , c2  , c3-2)
     & -     F5832*coarse( c1+1, c2-1, c3-1)
     & -     F4050*coarse( c1+2, c2-1, c3  )
     & -     F4050*coarse( c1+1, c2-2, c3  )
     & -     F4050*coarse( c1+2, c2  , c3-1)
     & -     F4050*coarse( c1  , c2-2, c3-1)
     & -     F4050*coarse( c1+1, c2  , c3-2)
     & -     F4050*coarse( c1  , c2-1, c3-2)
     & +     F1620*coarse( c1+2, c2-1, c3-1)
     & +     F1620*coarse( c1+1, c2-2, c3-1)
     & +     F1620*coarse( c1+1, c2-1, c3-2)
     & +     F1125*coarse( c1+2, c2-2, c3  )
     & +     F1125*coarse( c1+2, c2  , c3-2)
     & +     F1125*coarse( c1  , c2-2, c3-2)
     & -      F450*coarse( c1+2, c2-2, c3-1)
     & -      F450*coarse( c1+2, c2-1, c3-2)
     & -      F450*coarse( c1+1, c2-2, c3-2)
     & +      F125*coarse( c1+2, c2-2, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2-1, c3  )
     & -    F17010*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1+1, c2-1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3-1)
     & +     F6804*coarse( c1  , c2-1, c3-1)
     & -     F6075*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2-2, c3  )
     & +     F4725*coarse( c1  , c2  , c3-2)
     & +     F4536*coarse( c1+1, c2-1, c3-1)
     & +     F3150*coarse( c1+1, c2-2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3-2)
     & +     F2430*coarse( c1+2, c2-1, c3  )
     & +     F2430*coarse( c1+2, c2  , c3-1)
     & -     F1890*coarse( c1  , c2-2, c3-1)
     & -     F1890*coarse( c1  , c2-1, c3-2)
     & -     F1260*coarse( c1+1, c2-2, c3-1)
     & -     F1260*coarse( c1+1, c2-1, c3-2)
     & -      F972*coarse( c1+2, c2-1, c3-1)
     & -      F675*coarse( c1+2, c2-2, c3  )
     & -      F675*coarse( c1+2, c2  , c3-2)
     & +      F525*coarse( c1  , c2-2, c3-2)
     & +      F350*coarse( c1+1, c2-2, c3-2)
     & +      F270*coarse( c1+2, c2-2, c3-1)
     & +      F270*coarse( c1+2, c2-1, c3-2)
     & -       F75*coarse( c1+2, c2-2, c3-2)
     & )
          end if
        end if
      end if
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Set cint(2) boundary
      if(c2i.gt.cint(2)) then
        c2 = cint(2)
        f21 = 2*c2
c
c Set cint(2)--cint(1) edge
        if(c1i.gt.cint(1)) then
          c1 = cint(1)
          f11 = 2*c1
c
c Set cint(2)-cint(1)-cint(3) corner
          if(c3i.gt.cint(3)) then
            c3 = cint(3)
            f31 = 2*c3
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F91125*coarse( c1  , c2  , c3  )
     & -    F36450*coarse( c1+1, c2  , c3  )
     & -    F36450*coarse( c1  , c2+1, c3  )
     & -    F36450*coarse( c1  , c2  , c3+1)
     & +    F14580*coarse( c1+1, c2+1, c3  )
     & +    F14580*coarse( c1+1, c2  , c3+1)
     & +    F14580*coarse( c1  , c2+1, c3+1)
     & +    F10125*coarse( c1+2, c2  , c3  )
     & +    F10125*coarse( c1  , c2+2, c3  )
     & +    F10125*coarse( c1  , c2  , c3+2)
     & -     F5832*coarse( c1+1, c2+1, c3+1)
     & -     F4050*coarse( c1+2, c2+1, c3  )
     & -     F4050*coarse( c1+1, c2+2, c3  )
     & -     F4050*coarse( c1+2, c2  , c3+1)
     & -     F4050*coarse( c1  , c2+2, c3+1)
     & -     F4050*coarse( c1+1, c2  , c3+2)
     & -     F4050*coarse( c1  , c2+1, c3+2)
     & +     F1620*coarse( c1+2, c2+1, c3+1)
     & +     F1620*coarse( c1+1, c2+2, c3+1)
     & +     F1620*coarse( c1+1, c2+1, c3+2)
     & +     F1125*coarse( c1+2, c2+2, c3  )
     & +     F1125*coarse( c1+2, c2  , c3+2)
     & +     F1125*coarse( c1  , c2+2, c3+2)
     & -      F450*coarse( c1+2, c2+2, c3+1)
     & -      F450*coarse( c1+2, c2+1, c3+2)
     & -      F450*coarse( c1+1, c2+2, c3+2)
     & +      F125*coarse( c1+2, c2+2, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2+1, c3  )
     & -    F17010*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1+1, c2+1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3+1)
     & +     F6804*coarse( c1  , c2+1, c3+1)
     & -     F6075*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2+2, c3  )
     & +     F4725*coarse( c1  , c2  , c3+2)
     & +     F4536*coarse( c1+1, c2+1, c3+1)
     & +     F3150*coarse( c1+1, c2+2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3+2)
     & +     F2430*coarse( c1+2, c2+1, c3  )
     & +     F2430*coarse( c1+2, c2  , c3+1)
     & -     F1890*coarse( c1  , c2+2, c3+1)
     & -     F1890*coarse( c1  , c2+1, c3+2)
     & -     F1260*coarse( c1+1, c2+2, c3+1)
     & -     F1260*coarse( c1+1, c2+1, c3+2)
     & -      F972*coarse( c1+2, c2+1, c3+1)
     & -      F675*coarse( c1+2, c2+2, c3  )
     & -      F675*coarse( c1+2, c2  , c3+2)
     & +      F525*coarse( c1  , c2+2, c3+2)
     & +      F350*coarse( c1+1, c2+2, c3+2)
     & +      F270*coarse( c1+2, c2+2, c3+1)
     & +      F270*coarse( c1+2, c2+1, c3+2)
     & -       F75*coarse( c1+2, c2+2, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2+1, c3  )
     & -    F17010*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1+1, c2+1, c3  )
     & -    F11340*coarse( c1  , c2+1, c3+1)
     & +     F6804*coarse( c1+1, c2  , c3+1)
     & -     F6075*coarse( c1  , c2+2, c3  )
     & +     F4725*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2  , c3+2)
     & +     F4536*coarse( c1+1, c2+1, c3+1)
     & +     F3150*coarse( c1+2, c2+1, c3  )
     & +     F3150*coarse( c1  , c2+1, c3+2)
     & +     F2430*coarse( c1+1, c2+2, c3  )
     & +     F2430*coarse( c1  , c2+2, c3+1)
     & -     F1890*coarse( c1+2, c2  , c3+1)
     & -     F1890*coarse( c1+1, c2  , c3+2)
     & -     F1260*coarse( c1+2, c2+1, c3+1)
     & -     F1260*coarse( c1+1, c2+1, c3+2)
     & -      F972*coarse( c1+1, c2+2, c3+1)
     & -      F675*coarse( c1+2, c2+2, c3  )
     & -      F675*coarse( c1  , c2+2, c3+2)
     & +      F525*coarse( c1+2, c2  , c3+2)
     & +      F350*coarse( c1+2, c2+1, c3+2)
     & +      F270*coarse( c1+2, c2+2, c3+1)
     & +      F270*coarse( c1+1, c2+2, c3+2)
     & -       F75*coarse( c1+2, c2+2, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1+1, c2  , c3  )
     & +    F13230*coarse( c1  , c2+1, c3  )
     & +     F8820*coarse( c1+1, c2+1, c3  )
     & -     F7938*coarse( c1  , c2  , c3+1)
     & -     F5292*coarse( c1+1, c2  , c3+1)
     & -     F5292*coarse( c1  , c2+1, c3+1)
     & -     F3528*coarse( c1+1, c2+1, c3+1)
     & -     F2835*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2+2, c3  )
     & +     F2205*coarse( c1  , c2  , c3+2)
     & -     F1890*coarse( c1+2, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2+2, c3  )
     & +     F1470*coarse( c1+1, c2  , c3+2)
     & +     F1470*coarse( c1  , c2+1, c3+2)
     & +     F1134*coarse( c1+2, c2  , c3+1)
     & +     F1134*coarse( c1  , c2+2, c3+1)
     & +      F980*coarse( c1+1, c2+1, c3+2)
     & +      F756*coarse( c1+2, c2+1, c3+1)
     & +      F756*coarse( c1+1, c2+2, c3+1)
     & +      F405*coarse( c1+2, c2+2, c3  )
     & -      F315*coarse( c1+2, c2  , c3+2)
     & -      F315*coarse( c1  , c2+2, c3+2)
     & -      F210*coarse( c1+2, c2+1, c3+2)
     & -      F210*coarse( c1+1, c2+2, c3+2)
     & -      F162*coarse( c1+2, c2+2, c3+1)
     & +       F45*coarse( c1+2, c2+2, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2  , c3+1)
     & -    F17010*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3+1)
     & -    F11340*coarse( c1  , c2+1, c3+1)
     & +     F6804*coarse( c1+1, c2+1, c3  )
     & -     F6075*coarse( c1  , c2  , c3+2)
     & +     F4725*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2+2, c3  )
     & +     F4536*coarse( c1+1, c2+1, c3+1)
     & +     F3150*coarse( c1+2, c2  , c3+1)
     & +     F3150*coarse( c1  , c2+2, c3+1)
     & +     F2430*coarse( c1+1, c2  , c3+2)
     & +     F2430*coarse( c1  , c2+1, c3+2)
     & -     F1890*coarse( c1+2, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2+2, c3  )
     & -     F1260*coarse( c1+2, c2+1, c3+1)
     & -     F1260*coarse( c1+1, c2+2, c3+1)
     & -      F972*coarse( c1+1, c2+1, c3+2)
     & -      F675*coarse( c1+2, c2  , c3+2)
     & -      F675*coarse( c1  , c2+2, c3+2)
     & +      F525*coarse( c1+2, c2+2, c3  )
     & +      F350*coarse( c1+2, c2+2, c3+1)
     & +      F270*coarse( c1+2, c2+1, c3+2)
     & +      F270*coarse( c1+1, c2+2, c3+2)
     & -       F75*coarse( c1+2, c2+2, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1+1, c2  , c3  )
     & +    F13230*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1+1, c2  , c3+1)
     & -     F7938*coarse( c1  , c2+1, c3  )
     & -     F5292*coarse( c1+1, c2+1, c3  )
     & -     F5292*coarse( c1  , c2+1, c3+1)
     & -     F3528*coarse( c1+1, c2+1, c3+1)
     & -     F2835*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3+2)
     & +     F2205*coarse( c1  , c2+2, c3  )
     & -     F1890*coarse( c1+2, c2  , c3+1)
     & -     F1890*coarse( c1+1, c2  , c3+2)
     & +     F1470*coarse( c1+1, c2+2, c3  )
     & +     F1470*coarse( c1  , c2+2, c3+1)
     & +     F1134*coarse( c1+2, c2+1, c3  )
     & +     F1134*coarse( c1  , c2+1, c3+2)
     & +      F980*coarse( c1+1, c2+2, c3+1)
     & +      F756*coarse( c1+2, c2+1, c3+1)
     & +      F756*coarse( c1+1, c2+1, c3+2)
     & +      F405*coarse( c1+2, c2  , c3+2)
     & -      F315*coarse( c1+2, c2+2, c3  )
     & -      F315*coarse( c1  , c2+2, c3+2)
     & -      F210*coarse( c1+2, c2+2, c3+1)
     & -      F210*coarse( c1+1, c2+2, c3+2)
     & -      F162*coarse( c1+2, c2+1, c3+2)
     & +       F45*coarse( c1+2, c2+2, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1  , c2+1, c3  )
     & +    F13230*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1  , c2+1, c3+1)
     & -     F7938*coarse( c1+1, c2  , c3  )
     & -     F5292*coarse( c1+1, c2+1, c3  )
     & -     F5292*coarse( c1+1, c2  , c3+1)
     & -     F3528*coarse( c1+1, c2+1, c3+1)
     & -     F2835*coarse( c1  , c2+2, c3  )
     & -     F2835*coarse( c1  , c2  , c3+2)
     & +     F2205*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3+1)
     & -     F1890*coarse( c1  , c2+1, c3+2)
     & +     F1470*coarse( c1+2, c2+1, c3  )
     & +     F1470*coarse( c1+2, c2  , c3+1)
     & +     F1134*coarse( c1+1, c2+2, c3  )
     & +     F1134*coarse( c1+1, c2  , c3+2)
     & +      F980*coarse( c1+2, c2+1, c3+1)
     & +      F756*coarse( c1+1, c2+2, c3+1)
     & +      F756*coarse( c1+1, c2+1, c3+2)
     & +      F405*coarse( c1  , c2+2, c3+2)
     & -      F315*coarse( c1+2, c2+2, c3  )
     & -      F315*coarse( c1+2, c2  , c3+2)
     & -      F210*coarse( c1+2, c2+2, c3+1)
     & -      F210*coarse( c1+2, c2+1, c3+2)
     & -      F162*coarse( c1+1, c2+2, c3+2)
     & +       F45*coarse( c1+2, c2+2, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &       F9261*coarse( c1  , c2  , c3  )
     & +     F6174*coarse( c1+1, c2  , c3  )
     & +     F6174*coarse( c1  , c2+1, c3  )
     & +     F6174*coarse( c1  , c2  , c3+1)
     & +     F4116*coarse( c1+1, c2+1, c3  )
     & +     F4116*coarse( c1+1, c2  , c3+1)
     & +     F4116*coarse( c1  , c2+1, c3+1)
     & +     F2744*coarse( c1+1, c2+1, c3+1)
     & -     F1323*coarse( c1+2, c2  , c3  )
     & -     F1323*coarse( c1  , c2+2, c3  )
     & -     F1323*coarse( c1  , c2  , c3+2)
     & -      F882*coarse( c1+2, c2+1, c3  )
     & -      F882*coarse( c1+1, c2+2, c3  )
     & -      F882*coarse( c1+2, c2  , c3+1)
     & -      F882*coarse( c1  , c2+2, c3+1)
     & -      F882*coarse( c1+1, c2  , c3+2)
     & -      F882*coarse( c1  , c2+1, c3+2)
     & -      F588*coarse( c1+2, c2+1, c3+1)
     & -      F588*coarse( c1+1, c2+2, c3+1)
     & -      F588*coarse( c1+1, c2+1, c3+2)
     & +      F189*coarse( c1+2, c2+2, c3  )
     & +      F189*coarse( c1+2, c2  , c3+2)
     & +      F189*coarse( c1  , c2+2, c3+2)
     & +      F126*coarse( c1+2, c2+2, c3+1)
     & +      F126*coarse( c1+2, c2+1, c3+2)
     & +      F126*coarse( c1+1, c2+2, c3+2)
     & -       F27*coarse( c1+2, c2+2, c3+2)
     & )
          end if
c
c Set cint(2)-cint(1) edge
          f31 = f3i
          do c3 = c3i,c3f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     & +    F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1+1, c2  , c3  )
     & -    F24300*coarse( c1  , c2+1, c3  )
     & +    F10125*coarse( c1  , c2  , c3-1)
     & +     F9720*coarse( c1+1, c2+1, c3  )
     & +     F6750*coarse( c1+2, c2  , c3  )
     & +     F6750*coarse( c1  , c2+2, c3  )
     & -     F6075*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1+1, c2  , c3-1)
     & -     F4050*coarse( c1  , c2+1, c3-1)
     & -     F2700*coarse( c1+2, c2+1, c3  )
     & -     F2700*coarse( c1+1, c2+2, c3  )
     & +     F2430*coarse( c1+1, c2  , c3+1)
     & +     F2430*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1+1, c2+1, c3-1)
     & +     F1125*coarse( c1+2, c2  , c3-1)
     & +     F1125*coarse( c1  , c2+2, c3-1)
     & -      F972*coarse( c1+1, c2+1, c3+1)
     & +      F750*coarse( c1+2, c2+2, c3  )
     & -      F675*coarse( c1+2, c2  , c3+1)
     & -      F675*coarse( c1  , c2+2, c3+1)
     & -      F450*coarse( c1+2, c2+1, c3-1)
     & -      F450*coarse( c1+1, c2+2, c3-1)
     & +      F270*coarse( c1+2, c2+1, c3+1)
     & +      F270*coarse( c1+1, c2+2, c3+1)
     & +      F125*coarse( c1+2, c2+2, c3-1)
     & -       F75*coarse( c1+2, c2+2, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1+1, c2  , c3  )
     & -    F11340*coarse( c1  , c2+1, c3  )
     & -     F7560*coarse( c1+1, c2+1, c3  )
     & +     F4725*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1+1, c2  , c3-1)
     & +     F3150*coarse( c1  , c2+2, c3  )
     & -     F2835*coarse( c1  , c2  , c3+1)
     & +     F2100*coarse( c1+1, c2+2, c3  )
     & -     F1890*coarse( c1  , c2+1, c3-1)
     & -     F1890*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1+2, c2+1, c3  )
     & -     F1260*coarse( c1+1, c2+1, c3-1)
     & +     F1134*coarse( c1  , c2+1, c3+1)
     & +      F756*coarse( c1+1, c2+1, c3+1)
     & -      F675*coarse( c1+2, c2  , c3-1)
     & +      F525*coarse( c1  , c2+2, c3-1)
     & -      F450*coarse( c1+2, c2+2, c3  )
     & +      F405*coarse( c1+2, c2  , c3+1)
     & +      F350*coarse( c1+1, c2+2, c3-1)
     & -      F315*coarse( c1  , c2+2, c3+1)
     & +      F270*coarse( c1+2, c2+1, c3-1)
     & -      F210*coarse( c1+1, c2+2, c3+1)
     & -      F162*coarse( c1+2, c2+1, c3+1)
     & -       F75*coarse( c1+2, c2+2, c3-1)
     & +       F45*coarse( c1+2, c2+2, c3+1)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3  )
     & -     F7560*coarse( c1+1, c2+1, c3  )
     & +     F4725*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1  , c2+1, c3-1)
     & +     F3150*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3+1)
     & +     F2100*coarse( c1+2, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3-1)
     & -     F1890*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1+1, c2+2, c3  )
     & -     F1260*coarse( c1+1, c2+1, c3-1)
     & +     F1134*coarse( c1+1, c2  , c3+1)
     & +      F756*coarse( c1+1, c2+1, c3+1)
     & -      F675*coarse( c1  , c2+2, c3-1)
     & +      F525*coarse( c1+2, c2  , c3-1)
     & -      F450*coarse( c1+2, c2+2, c3  )
     & +      F405*coarse( c1  , c2+2, c3+1)
     & +      F350*coarse( c1+2, c2+1, c3-1)
     & -      F315*coarse( c1+2, c2  , c3+1)
     & +      F270*coarse( c1+1, c2+2, c3-1)
     & -      F210*coarse( c1+2, c2+1, c3+1)
     & -      F162*coarse( c1+1, c2+2, c3+1)
     & -       F75*coarse( c1+2, c2+2, c3-1)
     & +       F45*coarse( c1+2, c2+2, c3+1)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     & +    F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1+1, c2  , c3  )
     & +     F8820*coarse( c1  , c2+1, c3  )
     & +     F5880*coarse( c1+1, c2+1, c3  )
     & +     F2205*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3  )
     & +     F1470*coarse( c1+1, c2  , c3-1)
     & +     F1470*coarse( c1  , c2+1, c3-1)
     & -     F1323*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1+2, c2+1, c3  )
     & -     F1260*coarse( c1+1, c2+2, c3  )
     & +      F980*coarse( c1+1, c2+1, c3-1)
     & -      F882*coarse( c1+1, c2  , c3+1)
     & -      F882*coarse( c1  , c2+1, c3+1)
     & -      F588*coarse( c1+1, c2+1, c3+1)
     & -      F315*coarse( c1+2, c2  , c3-1)
     & -      F315*coarse( c1  , c2+2, c3-1)
     & +      F270*coarse( c1+2, c2+2, c3  )
     & -      F210*coarse( c1+2, c2+1, c3-1)
     & -      F210*coarse( c1+1, c2+2, c3-1)
     & +      F189*coarse( c1+2, c2  , c3+1)
     & +      F189*coarse( c1  , c2+2, c3+1)
     & +      F126*coarse( c1+2, c2+1, c3+1)
     & +      F126*coarse( c1+1, c2+2, c3+1)
     & +       F45*coarse( c1+2, c2+2, c3-1)
     & -       F27*coarse( c1+2, c2+2, c3+1)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     & +    F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1+1, c2  , c3  )
     & -    F24300*coarse( c1  , c2+1, c3  )
     & +    F10125*coarse( c1  , c2  , c3+1)
     & +     F9720*coarse( c1+1, c2+1, c3  )
     & +     F6750*coarse( c1+2, c2  , c3  )
     & +     F6750*coarse( c1  , c2+2, c3  )
     & -     F6075*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1+1, c2  , c3+1)
     & -     F4050*coarse( c1  , c2+1, c3+1)
     & -     F2700*coarse( c1+2, c2+1, c3  )
     & -     F2700*coarse( c1+1, c2+2, c3  )
     & +     F2430*coarse( c1+1, c2  , c3-1)
     & +     F2430*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1+1, c2+1, c3+1)
     & +     F1125*coarse( c1+2, c2  , c3+1)
     & +     F1125*coarse( c1  , c2+2, c3+1)
     & -      F972*coarse( c1+1, c2+1, c3-1)
     & +      F750*coarse( c1+2, c2+2, c3  )
     & -      F675*coarse( c1+2, c2  , c3-1)
     & -      F675*coarse( c1  , c2+2, c3-1)
     & -      F450*coarse( c1+2, c2+1, c3+1)
     & -      F450*coarse( c1+1, c2+2, c3+1)
     & +      F270*coarse( c1+2, c2+1, c3-1)
     & +      F270*coarse( c1+1, c2+2, c3-1)
     & +      F125*coarse( c1+2, c2+2, c3+1)
     & -       F75*coarse( c1+2, c2+2, c3-1)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1+1, c2  , c3  )
     & -    F11340*coarse( c1  , c2+1, c3  )
     & -     F7560*coarse( c1+1, c2+1, c3  )
     & +     F4725*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3+1)
     & -     F2835*coarse( c1  , c2  , c3-1)
     & +     F2100*coarse( c1+1, c2+2, c3  )
     & -     F1890*coarse( c1+1, c2  , c3-1)
     & -     F1890*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1+2, c2+1, c3  )
     & -     F1260*coarse( c1+1, c2+1, c3+1)
     & +     F1134*coarse( c1  , c2+1, c3-1)
     & +      F756*coarse( c1+1, c2+1, c3-1)
     & -      F675*coarse( c1+2, c2  , c3+1)
     & +      F525*coarse( c1  , c2+2, c3+1)
     & -      F450*coarse( c1+2, c2+2, c3  )
     & +      F405*coarse( c1+2, c2  , c3-1)
     & +      F350*coarse( c1+1, c2+2, c3+1)
     & -      F315*coarse( c1  , c2+2, c3-1)
     & +      F270*coarse( c1+2, c2+1, c3+1)
     & -      F210*coarse( c1+1, c2+2, c3-1)
     & -      F162*coarse( c1+2, c2+1, c3-1)
     & -       F75*coarse( c1+2, c2+2, c3+1)
     & +       F45*coarse( c1+2, c2+2, c3-1)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3  )
     & -     F7560*coarse( c1+1, c2+1, c3  )
     & +     F4725*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1+2, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3+1)
     & -     F2835*coarse( c1  , c2  , c3-1)
     & +     F2100*coarse( c1+2, c2+1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3-1)
     & -     F1890*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1+1, c2+2, c3  )
     & -     F1260*coarse( c1+1, c2+1, c3+1)
     & +     F1134*coarse( c1+1, c2  , c3-1)
     & +      F756*coarse( c1+1, c2+1, c3-1)
     & -      F675*coarse( c1  , c2+2, c3+1)
     & +      F525*coarse( c1+2, c2  , c3+1)
     & -      F450*coarse( c1+2, c2+2, c3  )
     & +      F405*coarse( c1  , c2+2, c3-1)
     & +      F350*coarse( c1+2, c2+1, c3+1)
     & -      F315*coarse( c1+2, c2  , c3-1)
     & +      F270*coarse( c1+1, c2+2, c3+1)
     & -      F210*coarse( c1+2, c2+1, c3-1)
     & -      F162*coarse( c1+1, c2+2, c3-1)
     & -       F75*coarse( c1+2, c2+2, c3+1)
     & +       F45*coarse( c1+2, c2+2, c3-1)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     & +    F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1+1, c2  , c3  )
     & +     F8820*coarse( c1  , c2+1, c3  )
     & +     F5880*coarse( c1+1, c2+1, c3  )
     & +     F2205*coarse( c1  , c2  , c3+1)
     & -     F1890*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3  )
     & +     F1470*coarse( c1+1, c2  , c3+1)
     & +     F1470*coarse( c1  , c2+1, c3+1)
     & -     F1323*coarse( c1  , c2  , c3-1)
     & -     F1260*coarse( c1+2, c2+1, c3  )
     & -     F1260*coarse( c1+1, c2+2, c3  )
     & +      F980*coarse( c1+1, c2+1, c3+1)
     & -      F882*coarse( c1+1, c2  , c3-1)
     & -      F882*coarse( c1  , c2+1, c3-1)
     & -      F588*coarse( c1+1, c2+1, c3-1)
     & -      F315*coarse( c1+2, c2  , c3+1)
     & -      F315*coarse( c1  , c2+2, c3+1)
     & +      F270*coarse( c1+2, c2+2, c3  )
     & -      F210*coarse( c1+2, c2+1, c3+1)
     & -      F210*coarse( c1+1, c2+2, c3+1)
     & +      F189*coarse( c1+2, c2  , c3-1)
     & +      F189*coarse( c1  , c2+2, c3-1)
     & +      F126*coarse( c1+2, c2+1, c3-1)
     & +      F126*coarse( c1+1, c2+2, c3-1)
     & +       F45*coarse( c1+2, c2+2, c3+1)
     & -       F27*coarse( c1+2, c2+2, c3-1)
     & )
            f31 = f31 + 2
          end do
        end if
c
c Set cint(2)-cint(1)-cint(6) corner
          if(c3f.lt.cint(6)) then
            c3 = cint(6)
            f31 = 2*c3
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2  , c3-1)
     & -    F17010*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3-1)
     & -    F11340*coarse( c1  , c2+1, c3-1)
     & +     F6804*coarse( c1+1, c2+1, c3  )
     & -     F6075*coarse( c1  , c2  , c3-2)
     & +     F4725*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2+2, c3  )
     & +     F4536*coarse( c1+1, c2+1, c3-1)
     & +     F3150*coarse( c1+2, c2  , c3-1)
     & +     F3150*coarse( c1  , c2+2, c3-1)
     & +     F2430*coarse( c1+1, c2  , c3-2)
     & +     F2430*coarse( c1  , c2+1, c3-2)
     & -     F1890*coarse( c1+2, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2+2, c3  )
     & -     F1260*coarse( c1+2, c2+1, c3-1)
     & -     F1260*coarse( c1+1, c2+2, c3-1)
     & -      F972*coarse( c1+1, c2+1, c3-2)
     & -      F675*coarse( c1+2, c2  , c3-2)
     & -      F675*coarse( c1  , c2+2, c3-2)
     & +      F525*coarse( c1+2, c2+2, c3  )
     & +      F350*coarse( c1+2, c2+2, c3-1)
     & +      F270*coarse( c1+2, c2+1, c3-2)
     & +      F270*coarse( c1+1, c2+2, c3-2)
     & -       F75*coarse( c1+2, c2+2, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1+1, c2  , c3  )
     & +    F13230*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1+1, c2  , c3-1)
     & -     F7938*coarse( c1  , c2+1, c3  )
     & -     F5292*coarse( c1+1, c2+1, c3  )
     & -     F5292*coarse( c1  , c2+1, c3-1)
     & -     F3528*coarse( c1+1, c2+1, c3-1)
     & -     F2835*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3-2)
     & +     F2205*coarse( c1  , c2+2, c3  )
     & -     F1890*coarse( c1+2, c2  , c3-1)
     & -     F1890*coarse( c1+1, c2  , c3-2)
     & +     F1470*coarse( c1+1, c2+2, c3  )
     & +     F1470*coarse( c1  , c2+2, c3-1)
     & +     F1134*coarse( c1+2, c2+1, c3  )
     & +     F1134*coarse( c1  , c2+1, c3-2)
     & +      F980*coarse( c1+1, c2+2, c3-1)
     & +      F756*coarse( c1+2, c2+1, c3-1)
     & +      F756*coarse( c1+1, c2+1, c3-2)
     & +      F405*coarse( c1+2, c2  , c3-2)
     & -      F315*coarse( c1+2, c2+2, c3  )
     & -      F315*coarse( c1  , c2+2, c3-2)
     & -      F210*coarse( c1+2, c2+2, c3-1)
     & -      F210*coarse( c1+1, c2+2, c3-2)
     & -      F162*coarse( c1+2, c2+1, c3-2)
     & +       F45*coarse( c1+2, c2+2, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1  , c2+1, c3  )
     & +    F13230*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1  , c2+1, c3-1)
     & -     F7938*coarse( c1+1, c2  , c3  )
     & -     F5292*coarse( c1+1, c2+1, c3  )
     & -     F5292*coarse( c1+1, c2  , c3-1)
     & -     F3528*coarse( c1+1, c2+1, c3-1)
     & -     F2835*coarse( c1  , c2+2, c3  )
     & -     F2835*coarse( c1  , c2  , c3-2)
     & +     F2205*coarse( c1+2, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3-1)
     & -     F1890*coarse( c1  , c2+1, c3-2)
     & +     F1470*coarse( c1+2, c2+1, c3  )
     & +     F1470*coarse( c1+2, c2  , c3-1)
     & +     F1134*coarse( c1+1, c2+2, c3  )
     & +     F1134*coarse( c1+1, c2  , c3-2)
     & +      F980*coarse( c1+2, c2+1, c3-1)
     & +      F756*coarse( c1+1, c2+2, c3-1)
     & +      F756*coarse( c1+1, c2+1, c3-2)
     & +      F405*coarse( c1  , c2+2, c3-2)
     & -      F315*coarse( c1+2, c2+2, c3  )
     & -      F315*coarse( c1+2, c2  , c3-2)
     & -      F210*coarse( c1+2, c2+2, c3-1)
     & -      F210*coarse( c1+2, c2+1, c3-2)
     & -      F162*coarse( c1+1, c2+2, c3-2)
     & +       F45*coarse( c1+2, c2+2, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &       F9261*coarse( c1  , c2  , c3  )
     & +     F6174*coarse( c1+1, c2  , c3  )
     & +     F6174*coarse( c1  , c2+1, c3  )
     & +     F6174*coarse( c1  , c2  , c3-1)
     & +     F4116*coarse( c1+1, c2+1, c3  )
     & +     F4116*coarse( c1+1, c2  , c3-1)
     & +     F4116*coarse( c1  , c2+1, c3-1)
     & +     F2744*coarse( c1+1, c2+1, c3-1)
     & -     F1323*coarse( c1+2, c2  , c3  )
     & -     F1323*coarse( c1  , c2+2, c3  )
     & -     F1323*coarse( c1  , c2  , c3-2)
     & -      F882*coarse( c1+2, c2+1, c3  )
     & -      F882*coarse( c1+1, c2+2, c3  )
     & -      F882*coarse( c1+2, c2  , c3-1)
     & -      F882*coarse( c1  , c2+2, c3-1)
     & -      F882*coarse( c1+1, c2  , c3-2)
     & -      F882*coarse( c1  , c2+1, c3-2)
     & -      F588*coarse( c1+2, c2+1, c3-1)
     & -      F588*coarse( c1+1, c2+2, c3-1)
     & -      F588*coarse( c1+1, c2+1, c3-2)
     & +      F189*coarse( c1+2, c2+2, c3  )
     & +      F189*coarse( c1+2, c2  , c3-2)
     & +      F189*coarse( c1  , c2+2, c3-2)
     & +      F126*coarse( c1+2, c2+2, c3-1)
     & +      F126*coarse( c1+2, c2+1, c3-2)
     & +      F126*coarse( c1+1, c2+2, c3-2)
     & -       F27*coarse( c1+2, c2+2, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F91125*coarse( c1  , c2  , c3  )
     & -    F36450*coarse( c1+1, c2  , c3  )
     & -    F36450*coarse( c1  , c2+1, c3  )
     & -    F36450*coarse( c1  , c2  , c3-1)
     & +    F14580*coarse( c1+1, c2+1, c3  )
     & +    F14580*coarse( c1+1, c2  , c3-1)
     & +    F14580*coarse( c1  , c2+1, c3-1)
     & +    F10125*coarse( c1+2, c2  , c3  )
     & +    F10125*coarse( c1  , c2+2, c3  )
     & +    F10125*coarse( c1  , c2  , c3-2)
     & -     F5832*coarse( c1+1, c2+1, c3-1)
     & -     F4050*coarse( c1+2, c2+1, c3  )
     & -     F4050*coarse( c1+1, c2+2, c3  )
     & -     F4050*coarse( c1+2, c2  , c3-1)
     & -     F4050*coarse( c1  , c2+2, c3-1)
     & -     F4050*coarse( c1+1, c2  , c3-2)
     & -     F4050*coarse( c1  , c2+1, c3-2)
     & +     F1620*coarse( c1+2, c2+1, c3-1)
     & +     F1620*coarse( c1+1, c2+2, c3-1)
     & +     F1620*coarse( c1+1, c2+1, c3-2)
     & +     F1125*coarse( c1+2, c2+2, c3  )
     & +     F1125*coarse( c1+2, c2  , c3-2)
     & +     F1125*coarse( c1  , c2+2, c3-2)
     & -      F450*coarse( c1+2, c2+2, c3-1)
     & -      F450*coarse( c1+2, c2+1, c3-2)
     & -      F450*coarse( c1+1, c2+2, c3-2)
     & +      F125*coarse( c1+2, c2+2, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2+1, c3  )
     & -    F17010*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1+1, c2+1, c3  )
     & -    F11340*coarse( c1+1, c2  , c3-1)
     & +     F6804*coarse( c1  , c2+1, c3-1)
     & -     F6075*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2+2, c3  )
     & +     F4725*coarse( c1  , c2  , c3-2)
     & +     F4536*coarse( c1+1, c2+1, c3-1)
     & +     F3150*coarse( c1+1, c2+2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3-2)
     & +     F2430*coarse( c1+2, c2+1, c3  )
     & +     F2430*coarse( c1+2, c2  , c3-1)
     & -     F1890*coarse( c1  , c2+2, c3-1)
     & -     F1890*coarse( c1  , c2+1, c3-2)
     & -     F1260*coarse( c1+1, c2+2, c3-1)
     & -     F1260*coarse( c1+1, c2+1, c3-2)
     & -      F972*coarse( c1+2, c2+1, c3-1)
     & -      F675*coarse( c1+2, c2+2, c3  )
     & -      F675*coarse( c1+2, c2  , c3-2)
     & +      F525*coarse( c1  , c2+2, c3-2)
     & +      F350*coarse( c1+1, c2+2, c3-2)
     & +      F270*coarse( c1+2, c2+2, c3-1)
     & +      F270*coarse( c1+2, c2+1, c3-2)
     & -       F75*coarse( c1+2, c2+2, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2+1, c3  )
     & -    F17010*coarse( c1+1, c2  , c3  )
     & -    F17010*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1+1, c2+1, c3  )
     & -    F11340*coarse( c1  , c2+1, c3-1)
     & +     F6804*coarse( c1+1, c2  , c3-1)
     & -     F6075*coarse( c1  , c2+2, c3  )
     & +     F4725*coarse( c1+2, c2  , c3  )
     & +     F4725*coarse( c1  , c2  , c3-2)
     & +     F4536*coarse( c1+1, c2+1, c3-1)
     & +     F3150*coarse( c1+2, c2+1, c3  )
     & +     F3150*coarse( c1  , c2+1, c3-2)
     & +     F2430*coarse( c1+1, c2+2, c3  )
     & +     F2430*coarse( c1  , c2+2, c3-1)
     & -     F1890*coarse( c1+2, c2  , c3-1)
     & -     F1890*coarse( c1+1, c2  , c3-2)
     & -     F1260*coarse( c1+2, c2+1, c3-1)
     & -     F1260*coarse( c1+1, c2+1, c3-2)
     & -      F972*coarse( c1+1, c2+2, c3-1)
     & -      F675*coarse( c1+2, c2+2, c3  )
     & -      F675*coarse( c1  , c2+2, c3-2)
     & +      F525*coarse( c1+2, c2  , c3-2)
     & +      F350*coarse( c1+2, c2+1, c3-2)
     & +      F270*coarse( c1+2, c2+2, c3-1)
     & +      F270*coarse( c1+1, c2+2, c3-2)
     & -       F75*coarse( c1+2, c2+2, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1+1, c2  , c3  )
     & +    F13230*coarse( c1  , c2+1, c3  )
     & +     F8820*coarse( c1+1, c2+1, c3  )
     & -     F7938*coarse( c1  , c2  , c3-1)
     & -     F5292*coarse( c1+1, c2  , c3-1)
     & -     F5292*coarse( c1  , c2+1, c3-1)
     & -     F3528*coarse( c1+1, c2+1, c3-1)
     & -     F2835*coarse( c1+2, c2  , c3  )
     & -     F2835*coarse( c1  , c2+2, c3  )
     & +     F2205*coarse( c1  , c2  , c3-2)
     & -     F1890*coarse( c1+2, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2+2, c3  )
     & +     F1470*coarse( c1+1, c2  , c3-2)
     & +     F1470*coarse( c1  , c2+1, c3-2)
     & +     F1134*coarse( c1+2, c2  , c3-1)
     & +     F1134*coarse( c1  , c2+2, c3-1)
     & +      F980*coarse( c1+1, c2+1, c3-2)
     & +      F756*coarse( c1+2, c2+1, c3-1)
     & +      F756*coarse( c1+1, c2+2, c3-1)
     & +      F405*coarse( c1+2, c2+2, c3  )
     & -      F315*coarse( c1+2, c2  , c3-2)
     & -      F315*coarse( c1  , c2+2, c3-2)
     & -      F210*coarse( c1+2, c2+1, c3-2)
     & -      F210*coarse( c1+1, c2+2, c3-2)
     & -      F162*coarse( c1+2, c2+2, c3-1)
     & +       F45*coarse( c1+2, c2+2, c3-2)
     & )
          end if
c
c Set cint(2)-cint(3) edge
        if(c3i.gt.cint(3)) then
          c3 = cint(3)
          f31 = 2*c3
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2+1, c3  )
     & -    F24300*coarse( c1  , c2  , c3+1)
     & +    F10125*coarse( c1-1, c2  , c3  )
     & +     F9720*coarse( c1  , c2+1, c3+1)
     & -     F6075*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+2, c3  )
     & +     F6750*coarse( c1  , c2  , c3+2)
     & -     F4050*coarse( c1-1, c2+1, c3  )
     & -     F4050*coarse( c1-1, c2  , c3+1)
     & -     F2700*coarse( c1  , c2+2, c3+1)
     & -     F2700*coarse( c1  , c2+1, c3+2)
     & +     F2430*coarse( c1+1, c2+1, c3  )
     & +     F2430*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1-1, c2+1, c3+1)
     & +     F1125*coarse( c1-1, c2+2, c3  )
     & +     F1125*coarse( c1-1, c2  , c3+2)
     & -      F972*coarse( c1+1, c2+1, c3+1)
     & +      F750*coarse( c1  , c2+2, c3+2)
     & -      F675*coarse( c1+1, c2+2, c3  )
     & -      F675*coarse( c1+1, c2  , c3+2)
     & -      F450*coarse( c1-1, c2+2, c3+1)
     & -      F450*coarse( c1-1, c2+1, c3+2)
     & +      F270*coarse( c1+1, c2+2, c3+1)
     & +      F270*coarse( c1+1, c2+1, c3+2)
     & +      F125*coarse( c1-1, c2+2, c3+2)
     & -       F75*coarse( c1+1, c2+2, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2+1, c3  )
     & -    F24300*coarse( c1  , c2  , c3+1)
     & +    F10125*coarse( c1+1, c2  , c3  )
     & +     F9720*coarse( c1  , c2+1, c3+1)
     & -     F6075*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+2, c3  )
     & +     F6750*coarse( c1  , c2  , c3+2)
     & -     F4050*coarse( c1+1, c2+1, c3  )
     & -     F4050*coarse( c1+1, c2  , c3+1)
     & -     F2700*coarse( c1  , c2+2, c3+1)
     & -     F2700*coarse( c1  , c2+1, c3+2)
     & +     F2430*coarse( c1-1, c2+1, c3  )
     & +     F2430*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1+1, c2+1, c3+1)
     & +     F1125*coarse( c1+1, c2+2, c3  )
     & +     F1125*coarse( c1+1, c2  , c3+2)
     & -      F972*coarse( c1-1, c2+1, c3+1)
     & +      F750*coarse( c1  , c2+2, c3+2)
     & -      F675*coarse( c1-1, c2+2, c3  )
     & -      F675*coarse( c1-1, c2  , c3+2)
     & -      F450*coarse( c1+1, c2+2, c3+1)
     & -      F450*coarse( c1+1, c2+1, c3+2)
     & +      F270*coarse( c1-1, c2+2, c3+1)
     & +      F270*coarse( c1-1, c2+1, c3+2)
     & +      F125*coarse( c1+1, c2+2, c3+2)
     & -       F75*coarse( c1-1, c2+2, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1  , c2  , c3+1)
     & -     F7560*coarse( c1  , c2+1, c3+1)
     & +     F4725*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1-1, c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+2)
     & -     F2835*coarse( c1+1, c2  , c3  )
     & +     F2100*coarse( c1  , c2+1, c3+2)
     & -     F1890*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2+2, c3+1)
     & -     F1260*coarse( c1-1, c2+1, c3+1)
     & +     F1134*coarse( c1+1, c2  , c3+1)
     & +      F756*coarse( c1+1, c2+1, c3+1)
     & -      F675*coarse( c1-1, c2+2, c3  )
     & +      F525*coarse( c1-1, c2  , c3+2)
     & -      F450*coarse( c1  , c2+2, c3+2)
     & +      F405*coarse( c1+1, c2+2, c3  )
     & +      F350*coarse( c1-1, c2+1, c3+2)
     & -      F315*coarse( c1+1, c2  , c3+2)
     & +      F270*coarse( c1-1, c2+2, c3+1)
     & -      F210*coarse( c1+1, c2+1, c3+2)
     & -      F162*coarse( c1+1, c2+2, c3+1)
     & -       F75*coarse( c1-1, c2+2, c3+2)
     & +       F45*coarse( c1+1, c2+2, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1  , c2  , c3+1)
     & -     F7560*coarse( c1  , c2+1, c3+1)
     & +     F4725*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1+1, c2+1, c3  )
     & -     F2835*coarse( c1-1, c2  , c3  )
     & +     F2100*coarse( c1  , c2+1, c3+2)
     & -     F1890*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2+2, c3+1)
     & -     F1260*coarse( c1+1, c2+1, c3+1)
     & +     F1134*coarse( c1-1, c2  , c3+1)
     & +      F756*coarse( c1-1, c2+1, c3+1)
     & -      F675*coarse( c1+1, c2+2, c3  )
     & +      F525*coarse( c1+1, c2  , c3+2)
     & -      F450*coarse( c1  , c2+2, c3+2)
     & +      F405*coarse( c1-1, c2+2, c3  )
     & +      F350*coarse( c1+1, c2+1, c3+2)
     & -      F315*coarse( c1-1, c2  , c3+2)
     & +      F270*coarse( c1+1, c2+2, c3+1)
     & -      F210*coarse( c1-1, c2+1, c3+2)
     & -      F162*coarse( c1-1, c2+2, c3+1)
     & -       F75*coarse( c1+1, c2+2, c3+2)
     & +       F45*coarse( c1-1, c2+2, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1  , c2+1, c3  )
     & -     F7560*coarse( c1  , c2+1, c3+1)
     & +     F4725*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3+1)
     & -     F2835*coarse( c1+1, c2  , c3  )
     & +     F2100*coarse( c1  , c2+2, c3+1)
     & -     F1890*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2+1, c3+2)
     & -     F1260*coarse( c1-1, c2+1, c3+1)
     & +     F1134*coarse( c1+1, c2+1, c3  )
     & +      F756*coarse( c1+1, c2+1, c3+1)
     & -      F675*coarse( c1-1, c2  , c3+2)
     & +      F525*coarse( c1-1, c2+2, c3  )
     & -      F450*coarse( c1  , c2+2, c3+2)
     & +      F405*coarse( c1+1, c2  , c3+2)
     & +      F350*coarse( c1-1, c2+2, c3+1)
     & -      F315*coarse( c1+1, c2+2, c3  )
     & +      F270*coarse( c1-1, c2+1, c3+2)
     & -      F210*coarse( c1+1, c2+2, c3+1)
     & -      F162*coarse( c1+1, c2+1, c3+2)
     & -       F75*coarse( c1-1, c2+2, c3+2)
     & +       F45*coarse( c1+1, c2+2, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1  , c2+1, c3  )
     & -     F7560*coarse( c1  , c2+1, c3+1)
     & +     F4725*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3+1)
     & -     F2835*coarse( c1-1, c2  , c3  )
     & +     F2100*coarse( c1  , c2+2, c3+1)
     & -     F1890*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2+1, c3+2)
     & -     F1260*coarse( c1+1, c2+1, c3+1)
     & +     F1134*coarse( c1-1, c2+1, c3  )
     & +      F756*coarse( c1-1, c2+1, c3+1)
     & -      F675*coarse( c1+1, c2  , c3+2)
     & +      F525*coarse( c1+1, c2+2, c3  )
     & -      F450*coarse( c1  , c2+2, c3+2)
     & +      F405*coarse( c1-1, c2  , c3+2)
     & +      F350*coarse( c1+1, c2+2, c3+1)
     & -      F315*coarse( c1-1, c2+2, c3  )
     & +      F270*coarse( c1+1, c2+1, c3+2)
     & -      F210*coarse( c1-1, c2+2, c3+1)
     & -      F162*coarse( c1-1, c2+1, c3+2)
     & -       F75*coarse( c1+1, c2+2, c3+2)
     & +       F45*coarse( c1-1, c2+2, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1  , c2+1, c3  )
     & +     F8820*coarse( c1  , c2  , c3+1)
     & +     F5880*coarse( c1  , c2+1, c3+1)
     & +     F2205*coarse( c1-1, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3  )
     & -     F1890*coarse( c1  , c2  , c3+2)
     & +     F1470*coarse( c1-1, c2+1, c3  )
     & +     F1470*coarse( c1-1, c2  , c3+1)
     & -     F1323*coarse( c1+1, c2  , c3  )
     & -     F1260*coarse( c1  , c2+2, c3+1)
     & -     F1260*coarse( c1  , c2+1, c3+2)
     & +      F980*coarse( c1-1, c2+1, c3+1)
     & -      F882*coarse( c1+1, c2+1, c3  )
     & -      F882*coarse( c1+1, c2  , c3+1)
     & -      F588*coarse( c1+1, c2+1, c3+1)
     & -      F315*coarse( c1-1, c2+2, c3  )
     & -      F315*coarse( c1-1, c2  , c3+2)
     & +      F270*coarse( c1  , c2+2, c3+2)
     & -      F210*coarse( c1-1, c2+2, c3+1)
     & -      F210*coarse( c1-1, c2+1, c3+2)
     & +      F189*coarse( c1+1, c2+2, c3  )
     & +      F189*coarse( c1+1, c2  , c3+2)
     & +      F126*coarse( c1+1, c2+2, c3+1)
     & +      F126*coarse( c1+1, c2+1, c3+2)
     & +       F45*coarse( c1-1, c2+2, c3+2)
     & -       F27*coarse( c1+1, c2+2, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1  , c2+1, c3  )
     & +     F5880*coarse( c1  , c2+1, c3+1)
     & +     F2205*coarse( c1+1, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3  )
     & -     F1890*coarse( c1  , c2  , c3+2)
     & +     F1470*coarse( c1+1, c2+1, c3  )
     & +     F1470*coarse( c1+1, c2  , c3+1)
     & -     F1323*coarse( c1-1, c2  , c3  )
     & -     F1260*coarse( c1  , c2+2, c3+1)
     & -     F1260*coarse( c1  , c2+1, c3+2)
     & +      F980*coarse( c1+1, c2+1, c3+1)
     & -      F882*coarse( c1-1, c2+1, c3  )
     & -      F882*coarse( c1-1, c2  , c3+1)
     & -      F588*coarse( c1-1, c2+1, c3+1)
     & -      F315*coarse( c1+1, c2+2, c3  )
     & -      F315*coarse( c1+1, c2  , c3+2)
     & +      F270*coarse( c1  , c2+2, c3+2)
     & -      F210*coarse( c1+1, c2+2, c3+1)
     & -      F210*coarse( c1+1, c2+1, c3+2)
     & +      F189*coarse( c1-1, c2+2, c3  )
     & +      F189*coarse( c1-1, c2  , c3+2)
     & +      F126*coarse( c1-1, c2+2, c3+1)
     & +      F126*coarse( c1-1, c2+1, c3+2)
     & +       F45*coarse( c1+1, c2+2, c3+2)
     & -       F27*coarse( c1-1, c2+2, c3+2)
     & )
            f11 = f11 + 2
          end do
        end if
c
c Set cint(2) face
        f31 = f3i
        do c3 = c3i,c3f
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2+1, c3  )
     & +     F6750*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1-1, c2  , c3  )
     & +     F4500*coarse( c1  , c2+2, c3  )
     & -     F4050*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1  , c2+1, c3-1)
     & -     F2700*coarse( c1-1, c2+1, c3  )
     & +     F1620*coarse( c1+1, c2+1, c3  )
     & +     F1620*coarse( c1  , c2+1, c3+1)
     & +     F1125*coarse( c1-1, c2  , c3-1)
     & +      F750*coarse( c1  , c2+2, c3-1)
     & +      F750*coarse( c1-1, c2+2, c3  )
     & -      F675*coarse( c1+1, c2  , c3-1)
     & -      F675*coarse( c1-1, c2  , c3+1)
     & -      F450*coarse( c1-1, c2+1, c3-1)
     & -      F450*coarse( c1+1, c2+2, c3  )
     & -      F450*coarse( c1  , c2+2, c3+1)
     & +      F405*coarse( c1+1, c2  , c3+1)
     & +      F270*coarse( c1+1, c2+1, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3+1)
     & -      F162*coarse( c1+1, c2+1, c3+1)
     & +      F125*coarse( c1-1, c2+2, c3-1)
     & -       F75*coarse( c1+1, c2+2, c3-1)
     & -       F75*coarse( c1-1, c2+2, c3+1)
     & +       F45*coarse( c1+1, c2+2, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2+1, c3  )
     & +     F6750*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1+1, c2  , c3  )
     & +     F4500*coarse( c1  , c2+2, c3  )
     & -     F4050*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1  , c2+1, c3-1)
     & -     F2700*coarse( c1+1, c2+1, c3  )
     & +     F1620*coarse( c1-1, c2+1, c3  )
     & +     F1620*coarse( c1  , c2+1, c3+1)
     & +     F1125*coarse( c1+1, c2  , c3-1)
     & +      F750*coarse( c1  , c2+2, c3-1)
     & +      F750*coarse( c1+1, c2+2, c3  )
     & -      F675*coarse( c1-1, c2  , c3-1)
     & -      F675*coarse( c1+1, c2  , c3+1)
     & -      F450*coarse( c1+1, c2+1, c3-1)
     & -      F450*coarse( c1-1, c2+2, c3  )
     & -      F450*coarse( c1  , c2+2, c3+1)
     & +      F405*coarse( c1-1, c2  , c3+1)
     & +      F270*coarse( c1-1, c2+1, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3+1)
     & -      F162*coarse( c1-1, c2+1, c3+1)
     & +      F125*coarse( c1+1, c2+2, c3-1)
     & -       F75*coarse( c1-1, c2+2, c3-1)
     & -       F75*coarse( c1+1, c2+2, c3+1)
     & +       F45*coarse( c1-1, c2+2, c3+1)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1  , c2+2, c3  )
     & +     F2100*coarse( c1  , c2+1, c3-1)
     & +     F2100*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1+1, c2+1, c3  )
     & -     F1260*coarse( c1  , c2+1, c3+1)
     & +      F525*coarse( c1-1, c2  , c3-1)
     & -      F450*coarse( c1  , c2+2, c3-1)
     & -      F450*coarse( c1-1, c2+2, c3  )
     & +      F350*coarse( c1-1, c2+1, c3-1)
     & -      F315*coarse( c1+1, c2  , c3-1)
     & -      F315*coarse( c1-1, c2  , c3+1)
     & +      F270*coarse( c1+1, c2+2, c3  )
     & +      F270*coarse( c1  , c2+2, c3+1)
     & -      F210*coarse( c1+1, c2+1, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3+1)
     & +      F189*coarse( c1+1, c2  , c3+1)
     & +      F126*coarse( c1+1, c2+1, c3+1)
     & -       F75*coarse( c1-1, c2+2, c3-1)
     & +       F45*coarse( c1+1, c2+2, c3-1)
     & +       F45*coarse( c1-1, c2+2, c3+1)
     & -       F27*coarse( c1+1, c2+2, c3+1)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1+1, c2  , c3  )
     & -     F2700*coarse( c1  , c2+2, c3  )
     & +     F2100*coarse( c1  , c2+1, c3-1)
     & +     F2100*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1-1, c2+1, c3  )
     & -     F1260*coarse( c1  , c2+1, c3+1)
     & +      F525*coarse( c1+1, c2  , c3-1)
     & -      F450*coarse( c1  , c2+2, c3-1)
     & -      F450*coarse( c1+1, c2+2, c3  )
     & +      F350*coarse( c1+1, c2+1, c3-1)
     & -      F315*coarse( c1-1, c2  , c3-1)
     & -      F315*coarse( c1+1, c2  , c3+1)
     & +      F270*coarse( c1-1, c2+2, c3  )
     & +      F270*coarse( c1  , c2+2, c3+1)
     & -      F210*coarse( c1-1, c2+1, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3+1)
     & +      F189*coarse( c1-1, c2  , c3+1)
     & +      F126*coarse( c1-1, c2+1, c3+1)
     & -       F75*coarse( c1+1, c2+2, c3-1)
     & +       F45*coarse( c1-1, c2+2, c3-1)
     & +       F45*coarse( c1+1, c2+2, c3+1)
     & -       F27*coarse( c1-1, c2+2, c3+1)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2+1, c3  )
     & +     F6750*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3+1)
     & +     F4500*coarse( c1  , c2+2, c3  )
     & -     F4050*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1+1, c2  , c3  )
     & -     F2700*coarse( c1-1, c2+1, c3  )
     & -     F2700*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1+1, c2+1, c3  )
     & +     F1125*coarse( c1-1, c2  , c3+1)
     & +      F750*coarse( c1-1, c2+2, c3  )
     & +      F750*coarse( c1  , c2+2, c3+1)
     & -      F675*coarse( c1-1, c2  , c3-1)
     & -      F675*coarse( c1+1, c2  , c3+1)
     & -      F450*coarse( c1  , c2+2, c3-1)
     & -      F450*coarse( c1+1, c2+2, c3  )
     & -      F450*coarse( c1-1, c2+1, c3+1)
     & +      F405*coarse( c1+1, c2  , c3-1)
     & +      F270*coarse( c1-1, c2+1, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3+1)
     & -      F162*coarse( c1+1, c2+1, c3-1)
     & +      F125*coarse( c1-1, c2+2, c3+1)
     & -       F75*coarse( c1-1, c2+2, c3-1)
     & -       F75*coarse( c1+1, c2+2, c3+1)
     & +       F45*coarse( c1+1, c2+2, c3-1)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2+1, c3  )
     & +     F6750*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3+1)
     & +     F4500*coarse( c1  , c2+2, c3  )
     & -     F4050*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1+1, c2+1, c3  )
     & -     F2700*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1-1, c2+1, c3  )
     & +     F1125*coarse( c1+1, c2  , c3+1)
     & +      F750*coarse( c1+1, c2+2, c3  )
     & +      F750*coarse( c1  , c2+2, c3+1)
     & -      F675*coarse( c1+1, c2  , c3-1)
     & -      F675*coarse( c1-1, c2  , c3+1)
     & -      F450*coarse( c1  , c2+2, c3-1)
     & -      F450*coarse( c1-1, c2+2, c3  )
     & -      F450*coarse( c1+1, c2+1, c3+1)
     & +      F405*coarse( c1-1, c2  , c3-1)
     & +      F270*coarse( c1+1, c2+1, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3+1)
     & -      F162*coarse( c1-1, c2+1, c3-1)
     & +      F125*coarse( c1+1, c2+2, c3+1)
     & -       F75*coarse( c1+1, c2+2, c3-1)
     & -       F75*coarse( c1-1, c2+2, c3+1)
     & +       F45*coarse( c1-1, c2+2, c3-1)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2+1, c3  )
     & +     F3150*coarse( c1-1, c2  , c3  )
     & +     F3150*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1  , c2+2, c3  )
     & +     F2100*coarse( c1-1, c2+1, c3  )
     & +     F2100*coarse( c1  , c2+1, c3+1)
     & -     F1890*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1+1, c2  , c3  )
     & -     F1260*coarse( c1  , c2+1, c3-1)
     & -     F1260*coarse( c1+1, c2+1, c3  )
     & +      F525*coarse( c1-1, c2  , c3+1)
     & -      F450*coarse( c1-1, c2+2, c3  )
     & -      F450*coarse( c1  , c2+2, c3+1)
     & +      F350*coarse( c1-1, c2+1, c3+1)
     & -      F315*coarse( c1-1, c2  , c3-1)
     & -      F315*coarse( c1+1, c2  , c3+1)
     & +      F270*coarse( c1  , c2+2, c3-1)
     & +      F270*coarse( c1+1, c2+2, c3  )
     & -      F210*coarse( c1-1, c2+1, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3+1)
     & +      F189*coarse( c1+1, c2  , c3-1)
     & +      F126*coarse( c1+1, c2+1, c3-1)
     & -       F75*coarse( c1-1, c2+2, c3+1)
     & +       F45*coarse( c1-1, c2+2, c3-1)
     & +       F45*coarse( c1+1, c2+2, c3+1)
     & -       F27*coarse( c1+1, c2+2, c3-1)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2+1, c3  )
     & +     F3150*coarse( c1+1, c2  , c3  )
     & +     F3150*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1  , c2+2, c3  )
     & +     F2100*coarse( c1+1, c2+1, c3  )
     & +     F2100*coarse( c1  , c2+1, c3+1)
     & -     F1890*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1-1, c2  , c3  )
     & -     F1260*coarse( c1  , c2+1, c3-1)
     & -     F1260*coarse( c1-1, c2+1, c3  )
     & +      F525*coarse( c1+1, c2  , c3+1)
     & -      F450*coarse( c1  , c2+2, c3+1)
     & -      F450*coarse( c1+1, c2+2, c3  )
     & +      F350*coarse( c1+1, c2+1, c3+1)
     & -      F315*coarse( c1+1, c2  , c3-1)
     & -      F315*coarse( c1-1, c2  , c3+1)
     & +      F270*coarse( c1  , c2+2, c3-1)
     & +      F270*coarse( c1-1, c2+2, c3  )
     & -      F210*coarse( c1+1, c2+1, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3+1)
     & +      F189*coarse( c1-1, c2  , c3-1)
     & +      F126*coarse( c1-1, c2+1, c3-1)
     & -       F75*coarse( c1+1, c2+2, c3+1)
     & +       F45*coarse( c1+1, c2+2, c3-1)
     & +       F45*coarse( c1-1, c2+2, c3+1)
     & -       F27*coarse( c1-1, c2+2, c3-1)
     & )
            f11 = f11 + 2
          end do
          f31 = f31 + 2
        end do
c
c Set cint(2)-cint(6) edge
        if(c3f.lt.cint(6)) then
          c3 = cint(6)
          f31 = 2*c3
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1  , c2+1, c3  )
     & -     F7560*coarse( c1  , c2+1, c3-1)
     & +     F4725*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3-1)
     & -     F2835*coarse( c1+1, c2  , c3  )
     & +     F2100*coarse( c1  , c2+2, c3-1)
     & -     F1890*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2+1, c3-2)
     & -     F1260*coarse( c1-1, c2+1, c3-1)
     & +     F1134*coarse( c1+1, c2+1, c3  )
     & +      F756*coarse( c1+1, c2+1, c3-1)
     & -      F675*coarse( c1-1, c2  , c3-2)
     & +      F525*coarse( c1-1, c2+2, c3  )
     & -      F450*coarse( c1  , c2+2, c3-2)
     & +      F405*coarse( c1+1, c2  , c3-2)
     & +      F350*coarse( c1-1, c2+2, c3-1)
     & -      F315*coarse( c1+1, c2+2, c3  )
     & +      F270*coarse( c1-1, c2+1, c3-2)
     & -      F210*coarse( c1+1, c2+2, c3-1)
     & -      F162*coarse( c1+1, c2+1, c3-2)
     & -       F75*coarse( c1-1, c2+2, c3-2)
     & +       F45*coarse( c1+1, c2+2, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1  , c2+1, c3  )
     & -     F7560*coarse( c1  , c2+1, c3-1)
     & +     F4725*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3-1)
     & -     F2835*coarse( c1-1, c2  , c3  )
     & +     F2100*coarse( c1  , c2+2, c3-1)
     & -     F1890*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2+1, c3-2)
     & -     F1260*coarse( c1+1, c2+1, c3-1)
     & +     F1134*coarse( c1-1, c2+1, c3  )
     & +      F756*coarse( c1-1, c2+1, c3-1)
     & -      F675*coarse( c1+1, c2  , c3-2)
     & +      F525*coarse( c1+1, c2+2, c3  )
     & -      F450*coarse( c1  , c2+2, c3-2)
     & +      F405*coarse( c1-1, c2  , c3-2)
     & +      F350*coarse( c1+1, c2+2, c3-1)
     & -      F315*coarse( c1-1, c2+2, c3  )
     & +      F270*coarse( c1+1, c2+1, c3-2)
     & -      F210*coarse( c1-1, c2+2, c3-1)
     & -      F162*coarse( c1-1, c2+1, c3-2)
     & -       F75*coarse( c1+1, c2+2, c3-2)
     & +       F45*coarse( c1-1, c2+2, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1  , c2+1, c3  )
     & +     F8820*coarse( c1  , c2  , c3-1)
     & +     F5880*coarse( c1  , c2+1, c3-1)
     & +     F2205*coarse( c1-1, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3  )
     & -     F1890*coarse( c1  , c2  , c3-2)
     & +     F1470*coarse( c1-1, c2+1, c3  )
     & +     F1470*coarse( c1-1, c2  , c3-1)
     & -     F1323*coarse( c1+1, c2  , c3  )
     & -     F1260*coarse( c1  , c2+2, c3-1)
     & -     F1260*coarse( c1  , c2+1, c3-2)
     & +      F980*coarse( c1-1, c2+1, c3-1)
     & -      F882*coarse( c1+1, c2+1, c3  )
     & -      F882*coarse( c1+1, c2  , c3-1)
     & -      F588*coarse( c1+1, c2+1, c3-1)
     & -      F315*coarse( c1-1, c2+2, c3  )
     & -      F315*coarse( c1-1, c2  , c3-2)
     & +      F270*coarse( c1  , c2+2, c3-2)
     & -      F210*coarse( c1-1, c2+2, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3-2)
     & +      F189*coarse( c1+1, c2+2, c3  )
     & +      F189*coarse( c1+1, c2  , c3-2)
     & +      F126*coarse( c1+1, c2+2, c3-1)
     & +      F126*coarse( c1+1, c2+1, c3-2)
     & +       F45*coarse( c1-1, c2+2, c3-2)
     & -       F27*coarse( c1+1, c2+2, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1  , c2+1, c3  )
     & +     F5880*coarse( c1  , c2+1, c3-1)
     & +     F2205*coarse( c1+1, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3  )
     & -     F1890*coarse( c1  , c2  , c3-2)
     & +     F1470*coarse( c1+1, c2+1, c3  )
     & +     F1470*coarse( c1+1, c2  , c3-1)
     & -     F1323*coarse( c1-1, c2  , c3  )
     & -     F1260*coarse( c1  , c2+2, c3-1)
     & -     F1260*coarse( c1  , c2+1, c3-2)
     & +      F980*coarse( c1+1, c2+1, c3-1)
     & -      F882*coarse( c1-1, c2+1, c3  )
     & -      F882*coarse( c1-1, c2  , c3-1)
     & -      F588*coarse( c1-1, c2+1, c3-1)
     & -      F315*coarse( c1+1, c2+2, c3  )
     & -      F315*coarse( c1+1, c2  , c3-2)
     & +      F270*coarse( c1  , c2+2, c3-2)
     & -      F210*coarse( c1+1, c2+2, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3-2)
     & +      F189*coarse( c1-1, c2+2, c3  )
     & +      F189*coarse( c1-1, c2  , c3-2)
     & +      F126*coarse( c1-1, c2+2, c3-1)
     & +      F126*coarse( c1-1, c2+1, c3-2)
     & +       F45*coarse( c1+1, c2+2, c3-2)
     & -       F27*coarse( c1-1, c2+2, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2+1, c3  )
     & -    F24300*coarse( c1  , c2  , c3-1)
     & +    F10125*coarse( c1-1, c2  , c3  )
     & +     F9720*coarse( c1  , c2+1, c3-1)
     & -     F6075*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+2, c3  )
     & +     F6750*coarse( c1  , c2  , c3-2)
     & -     F4050*coarse( c1-1, c2+1, c3  )
     & -     F4050*coarse( c1-1, c2  , c3-1)
     & -     F2700*coarse( c1  , c2+2, c3-1)
     & -     F2700*coarse( c1  , c2+1, c3-2)
     & +     F2430*coarse( c1+1, c2+1, c3  )
     & +     F2430*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1-1, c2+1, c3-1)
     & +     F1125*coarse( c1-1, c2+2, c3  )
     & +     F1125*coarse( c1-1, c2  , c3-2)
     & -      F972*coarse( c1+1, c2+1, c3-1)
     & +      F750*coarse( c1  , c2+2, c3-2)
     & -      F675*coarse( c1+1, c2+2, c3  )
     & -      F675*coarse( c1+1, c2  , c3-2)
     & -      F450*coarse( c1-1, c2+2, c3-1)
     & -      F450*coarse( c1-1, c2+1, c3-2)
     & +      F270*coarse( c1+1, c2+2, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3-2)
     & +      F125*coarse( c1-1, c2+2, c3-2)
     & -       F75*coarse( c1+1, c2+2, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2+1, c3  )
     & -    F24300*coarse( c1  , c2  , c3-1)
     & +    F10125*coarse( c1+1, c2  , c3  )
     & +     F9720*coarse( c1  , c2+1, c3-1)
     & -     F6075*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+2, c3  )
     & +     F6750*coarse( c1  , c2  , c3-2)
     & -     F4050*coarse( c1+1, c2+1, c3  )
     & -     F4050*coarse( c1+1, c2  , c3-1)
     & -     F2700*coarse( c1  , c2+2, c3-1)
     & -     F2700*coarse( c1  , c2+1, c3-2)
     & +     F2430*coarse( c1-1, c2+1, c3  )
     & +     F2430*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1+1, c2+1, c3-1)
     & +     F1125*coarse( c1+1, c2+2, c3  )
     & +     F1125*coarse( c1+1, c2  , c3-2)
     & -      F972*coarse( c1-1, c2+1, c3-1)
     & +      F750*coarse( c1  , c2+2, c3-2)
     & -      F675*coarse( c1-1, c2+2, c3  )
     & -      F675*coarse( c1-1, c2  , c3-2)
     & -      F450*coarse( c1+1, c2+2, c3-1)
     & -      F450*coarse( c1+1, c2+1, c3-2)
     & +      F270*coarse( c1-1, c2+2, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3-2)
     & +      F125*coarse( c1+1, c2+2, c3-2)
     & -       F75*coarse( c1-1, c2+2, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1  , c2  , c3-1)
     & -     F7560*coarse( c1  , c2+1, c3-1)
     & +     F4725*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1-1, c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-2)
     & -     F2835*coarse( c1+1, c2  , c3  )
     & +     F2100*coarse( c1  , c2+1, c3-2)
     & -     F1890*coarse( c1+1, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2+2, c3-1)
     & -     F1260*coarse( c1-1, c2+1, c3-1)
     & +     F1134*coarse( c1+1, c2  , c3-1)
     & +      F756*coarse( c1+1, c2+1, c3-1)
     & -      F675*coarse( c1-1, c2+2, c3  )
     & +      F525*coarse( c1-1, c2  , c3-2)
     & -      F450*coarse( c1  , c2+2, c3-2)
     & +      F405*coarse( c1+1, c2+2, c3  )
     & +      F350*coarse( c1-1, c2+1, c3-2)
     & -      F315*coarse( c1+1, c2  , c3-2)
     & +      F270*coarse( c1-1, c2+2, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3-2)
     & -      F162*coarse( c1+1, c2+2, c3-1)
     & -       F75*coarse( c1-1, c2+2, c3-2)
     & +       F45*coarse( c1+1, c2+2, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1  , c2  , c3-1)
     & -     F7560*coarse( c1  , c2+1, c3-1)
     & +     F4725*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1+1, c2+1, c3  )
     & -     F2835*coarse( c1-1, c2  , c3  )
     & +     F2100*coarse( c1  , c2+1, c3-2)
     & -     F1890*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2+2, c3-1)
     & -     F1260*coarse( c1+1, c2+1, c3-1)
     & +     F1134*coarse( c1-1, c2  , c3-1)
     & +      F756*coarse( c1-1, c2+1, c3-1)
     & -      F675*coarse( c1+1, c2+2, c3  )
     & +      F525*coarse( c1+1, c2  , c3-2)
     & -      F450*coarse( c1  , c2+2, c3-2)
     & +      F405*coarse( c1-1, c2+2, c3  )
     & +      F350*coarse( c1+1, c2+1, c3-2)
     & -      F315*coarse( c1-1, c2  , c3-2)
     & +      F270*coarse( c1+1, c2+2, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3-2)
     & -      F162*coarse( c1-1, c2+2, c3-1)
     & -       F75*coarse( c1+1, c2+2, c3-2)
     & +       F45*coarse( c1-1, c2+2, c3-2)
     & )
            f11 = f11 + 2
          end do

        end if
      end if
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Set cint(3) boundary
      if(c3i.gt.cint(3)) then
        c3 = cint(3)
        f31 = 2*c3
        f21 = f2i
        do c2 = c2i,c2f
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2  , c3+1)
     & +     F6750*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1-1, c2  , c3  )
     & +     F4500*coarse( c1  , c2  , c3+2)
     & -     F4050*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2-1, c3+1)
     & -     F2700*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2+1, c3+1)
     & +     F1125*coarse( c1-1, c2-1, c3  )
     & +      F750*coarse( c1  , c2-1, c3+2)
     & +      F750*coarse( c1-1, c2  , c3+2)
     & -      F675*coarse( c1+1, c2-1, c3  )
     & -      F675*coarse( c1-1, c2+1, c3  )
     & -      F450*coarse( c1-1, c2-1, c3+1)
     & -      F450*coarse( c1+1, c2  , c3+2)
     & -      F450*coarse( c1  , c2+1, c3+2)
     & +      F405*coarse( c1+1, c2+1, c3  )
     & +      F270*coarse( c1+1, c2-1, c3+1)
     & +      F270*coarse( c1-1, c2+1, c3+1)
     & -      F162*coarse( c1+1, c2+1, c3+1)
     & +      F125*coarse( c1-1, c2-1, c3+2)
     & -       F75*coarse( c1+1, c2-1, c3+2)
     & -       F75*coarse( c1-1, c2+1, c3+2)
     & +       F45*coarse( c1+1, c2+1, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2  , c3+1)
     & +     F6750*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1+1, c2  , c3  )
     & +     F4500*coarse( c1  , c2  , c3+2)
     & -     F4050*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2-1, c3+1)
     & -     F2700*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2+1, c3+1)
     & +     F1125*coarse( c1+1, c2-1, c3  )
     & +      F750*coarse( c1  , c2-1, c3+2)
     & +      F750*coarse( c1+1, c2  , c3+2)
     & -      F675*coarse( c1-1, c2-1, c3  )
     & -      F675*coarse( c1+1, c2+1, c3  )
     & -      F450*coarse( c1+1, c2-1, c3+1)
     & -      F450*coarse( c1-1, c2  , c3+2)
     & -      F450*coarse( c1  , c2+1, c3+2)
     & +      F405*coarse( c1-1, c2+1, c3  )
     & +      F270*coarse( c1-1, c2-1, c3+1)
     & +      F270*coarse( c1+1, c2+1, c3+1)
     & -      F162*coarse( c1-1, c2+1, c3+1)
     & +      F125*coarse( c1+1, c2-1, c3+2)
     & -       F75*coarse( c1-1, c2-1, c3+2)
     & -       F75*coarse( c1+1, c2+1, c3+2)
     & +       F45*coarse( c1-1, c2+1, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2  , c3+1)
     & +     F6750*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+1, c3  )
     & +     F4500*coarse( c1  , c2  , c3+2)
     & -     F4050*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1+1, c2  , c3  )
     & -     F2700*coarse( c1-1, c2  , c3+1)
     & -     F2700*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1+1, c2  , c3+1)
     & +     F1125*coarse( c1-1, c2+1, c3  )
     & +      F750*coarse( c1-1, c2  , c3+2)
     & +      F750*coarse( c1  , c2+1, c3+2)
     & -      F675*coarse( c1-1, c2-1, c3  )
     & -      F675*coarse( c1+1, c2+1, c3  )
     & -      F450*coarse( c1-1, c2+1, c3+1)
     & -      F450*coarse( c1  , c2-1, c3+2)
     & -      F450*coarse( c1+1, c2  , c3+2)
     & +      F405*coarse( c1+1, c2-1, c3  )
     & +      F270*coarse( c1-1, c2-1, c3+1)
     & +      F270*coarse( c1+1, c2+1, c3+1)
     & -      F162*coarse( c1+1, c2-1, c3+1)
     & +      F125*coarse( c1-1, c2+1, c3+2)
     & -       F75*coarse( c1-1, c2-1, c3+2)
     & -       F75*coarse( c1+1, c2+1, c3+2)
     & +       F45*coarse( c1+1, c2-1, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2  , c3+1)
     & +     F6750*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+1, c3  )
     & +     F4500*coarse( c1  , c2  , c3+2)
     & -     F4050*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1+1, c2  , c3+1)
     & -     F2700*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1-1, c2  , c3+1)
     & +     F1125*coarse( c1+1, c2+1, c3  )
     & +      F750*coarse( c1+1, c2  , c3+2)
     & +      F750*coarse( c1  , c2+1, c3+2)
     & -      F675*coarse( c1+1, c2-1, c3  )
     & -      F675*coarse( c1-1, c2+1, c3  )
     & -      F450*coarse( c1+1, c2+1, c3+1)
     & -      F450*coarse( c1  , c2-1, c3+2)
     & -      F450*coarse( c1-1, c2  , c3+2)
     & +      F405*coarse( c1-1, c2-1, c3  )
     & +      F270*coarse( c1+1, c2-1, c3+1)
     & +      F270*coarse( c1-1, c2+1, c3+1)
     & -      F162*coarse( c1-1, c2-1, c3+1)
     & +      F125*coarse( c1+1, c2+1, c3+2)
     & -       F75*coarse( c1+1, c2-1, c3+2)
     & -       F75*coarse( c1-1, c2+1, c3+2)
     & +       F45*coarse( c1-1, c2-1, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2  , c3+1)
     & +     F3150*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1  , c2  , c3+2)
     & +     F2100*coarse( c1  , c2-1, c3+1)
     & +     F2100*coarse( c1-1, c2  , c3+1)
     & -     F1890*coarse( c1+1, c2  , c3  )
     & -     F1890*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1+1, c2  , c3+1)
     & -     F1260*coarse( c1  , c2+1, c3+1)
     & +      F525*coarse( c1-1, c2-1, c3  )
     & -      F450*coarse( c1  , c2-1, c3+2)
     & -      F450*coarse( c1-1, c2  , c3+2)
     & +      F350*coarse( c1-1, c2-1, c3+1)
     & -      F315*coarse( c1+1, c2-1, c3  )
     & -      F315*coarse( c1-1, c2+1, c3  )
     & +      F270*coarse( c1+1, c2  , c3+2)
     & +      F270*coarse( c1  , c2+1, c3+2)
     & -      F210*coarse( c1+1, c2-1, c3+1)
     & -      F210*coarse( c1-1, c2+1, c3+1)
     & +      F189*coarse( c1+1, c2+1, c3  )
     & +      F126*coarse( c1+1, c2+1, c3+1)
     & -       F75*coarse( c1-1, c2-1, c3+2)
     & +       F45*coarse( c1-1, c2+1, c3+2)
     & +       F45*coarse( c1+1, c2-1, c3+2)
     & -       F27*coarse( c1+1, c2+1, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2  , c3+1)
     & +     F3150*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1+1, c2  , c3  )
     & -     F2700*coarse( c1  , c2  , c3+2)
     & +     F2100*coarse( c1  , c2-1, c3+1)
     & +     F2100*coarse( c1+1, c2  , c3+1)
     & -     F1890*coarse( c1-1, c2  , c3  )
     & -     F1890*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1-1, c2  , c3+1)
     & -     F1260*coarse( c1  , c2+1, c3+1)
     & +      F525*coarse( c1+1, c2-1, c3  )
     & -      F450*coarse( c1  , c2-1, c3+2)
     & -      F450*coarse( c1+1, c2  , c3+2)
     & +      F350*coarse( c1+1, c2-1, c3+1)
     & -      F315*coarse( c1-1, c2-1, c3  )
     & -      F315*coarse( c1+1, c2+1, c3  )
     & +      F270*coarse( c1-1, c2  , c3+2)
     & +      F270*coarse( c1  , c2+1, c3+2)
     & -      F210*coarse( c1-1, c2-1, c3+1)
     & -      F210*coarse( c1+1, c2+1, c3+1)
     & +      F189*coarse( c1-1, c2+1, c3  )
     & +      F126*coarse( c1-1, c2+1, c3+1)
     & -       F75*coarse( c1+1, c2-1, c3+2)
     & +       F45*coarse( c1-1, c2-1, c3+2)
     & +       F45*coarse( c1+1, c2+1, c3+2)
     & -       F27*coarse( c1-1, c2+1, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2  , c3+1)
     & +     F3150*coarse( c1-1, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2  , c3+2)
     & +     F2100*coarse( c1-1, c2  , c3+1)
     & +     F2100*coarse( c1  , c2+1, c3+1)
     & -     F1890*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-1, c3+1)
     & -     F1260*coarse( c1+1, c2  , c3+1)
     & +      F525*coarse( c1-1, c2+1, c3  )
     & -      F450*coarse( c1-1, c2  , c3+2)
     & -      F450*coarse( c1  , c2+1, c3+2)
     & +      F350*coarse( c1-1, c2+1, c3+1)
     & -      F315*coarse( c1-1, c2-1, c3  )
     & -      F315*coarse( c1+1, c2+1, c3  )
     & +      F270*coarse( c1  , c2-1, c3+2)
     & +      F270*coarse( c1+1, c2  , c3+2)
     & -      F210*coarse( c1-1, c2-1, c3+1)
     & -      F210*coarse( c1+1, c2+1, c3+1)
     & +      F189*coarse( c1+1, c2-1, c3  )
     & +      F126*coarse( c1+1, c2-1, c3+1)
     & -       F75*coarse( c1-1, c2+1, c3+2)
     & +       F45*coarse( c1-1, c2-1, c3+2)
     & +       F45*coarse( c1+1, c2+1, c3+2)
     & -       F27*coarse( c1+1, c2-1, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2  , c3+1)
     & +     F3150*coarse( c1+1, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2  , c3+2)
     & +     F2100*coarse( c1+1, c2  , c3+1)
     & +     F2100*coarse( c1  , c2+1, c3+1)
     & -     F1890*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-1, c3+1)
     & -     F1260*coarse( c1-1, c2  , c3+1)
     & +      F525*coarse( c1+1, c2+1, c3  )
     & -      F450*coarse( c1+1, c2  , c3+2)
     & -      F450*coarse( c1  , c2+1, c3+2)
     & +      F350*coarse( c1+1, c2+1, c3+1)
     & -      F315*coarse( c1+1, c2-1, c3  )
     & -      F315*coarse( c1-1, c2+1, c3  )
     & +      F270*coarse( c1  , c2-1, c3+2)
     & +      F270*coarse( c1-1, c2  , c3+2)
     & -      F210*coarse( c1+1, c2-1, c3+1)
     & -      F210*coarse( c1-1, c2+1, c3+1)
     & +      F189*coarse( c1-1, c2-1, c3  )
     & +      F126*coarse( c1-1, c2-1, c3+1)
     & -       F75*coarse( c1+1, c2+1, c3+2)
     & +       F45*coarse( c1+1, c2-1, c3+2)
     & +       F45*coarse( c1-1, c2+1, c3+2)
     & -       F27*coarse( c1-1, c2-1, c3+2)
     & )
            f11 = f11 + 2
          end do
          f21 = f21 + 2
        end do
      end if
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Set cint(4) boundary
      if(c1f.lt.cint(4)) then
        c1 = cint(4)
        f11 = 2*c1
c
c Set cint(4)--cint(2) corner
        if(c2i.gt.cint(2)) then
          c2 = cint(2)
          f21 = 2*c2
c
c Set cint(4)-cint(2)-cint(3) corner
          if(c3i.gt.cint(3)) then
            c3 = cint(3)
            f31 = 2*c3
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2+1, c3  )
     & -    F17010*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1-1, c2+1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3+1)
     & +     F6804*coarse( c1  , c2+1, c3+1)
     & -     F6075*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2+2, c3  )
     & +     F4725*coarse( c1  , c2  , c3+2)
     & +     F4536*coarse( c1-1, c2+1, c3+1)
     & +     F3150*coarse( c1-1, c2+2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3+2)
     & +     F2430*coarse( c1-2, c2+1, c3  )
     & +     F2430*coarse( c1-2, c2  , c3+1)
     & -     F1890*coarse( c1  , c2+2, c3+1)
     & -     F1890*coarse( c1  , c2+1, c3+2)
     & -     F1260*coarse( c1-1, c2+2, c3+1)
     & -     F1260*coarse( c1-1, c2+1, c3+2)
     & -      F972*coarse( c1-2, c2+1, c3+1)
     & -      F675*coarse( c1-2, c2+2, c3  )
     & -      F675*coarse( c1-2, c2  , c3+2)
     & +      F525*coarse( c1  , c2+2, c3+2)
     & +      F350*coarse( c1-1, c2+2, c3+2)
     & +      F270*coarse( c1-2, c2+2, c3+1)
     & +      F270*coarse( c1-2, c2+1, c3+2)
     & -       F75*coarse( c1-2, c2+2, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F91125*coarse( c1  , c2  , c3  )
     & -    F36450*coarse( c1-1, c2  , c3  )
     & -    F36450*coarse( c1  , c2+1, c3  )
     & -    F36450*coarse( c1  , c2  , c3+1)
     & +    F14580*coarse( c1-1, c2+1, c3  )
     & +    F14580*coarse( c1-1, c2  , c3+1)
     & +    F14580*coarse( c1  , c2+1, c3+1)
     & +    F10125*coarse( c1-2, c2  , c3  )
     & +    F10125*coarse( c1  , c2+2, c3  )
     & +    F10125*coarse( c1  , c2  , c3+2)
     & -     F5832*coarse( c1-1, c2+1, c3+1)
     & -     F4050*coarse( c1-2, c2+1, c3  )
     & -     F4050*coarse( c1-1, c2+2, c3  )
     & -     F4050*coarse( c1-2, c2  , c3+1)
     & -     F4050*coarse( c1  , c2+2, c3+1)
     & -     F4050*coarse( c1-1, c2  , c3+2)
     & -     F4050*coarse( c1  , c2+1, c3+2)
     & +     F1620*coarse( c1-2, c2+1, c3+1)
     & +     F1620*coarse( c1-1, c2+2, c3+1)
     & +     F1620*coarse( c1-1, c2+1, c3+2)
     & +     F1125*coarse( c1-2, c2+2, c3  )
     & +     F1125*coarse( c1-2, c2  , c3+2)
     & +     F1125*coarse( c1  , c2+2, c3+2)
     & -      F450*coarse( c1-2, c2+2, c3+1)
     & -      F450*coarse( c1-2, c2+1, c3+2)
     & -      F450*coarse( c1-1, c2+2, c3+2)
     & +      F125*coarse( c1-2, c2+2, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1-1, c2  , c3  )
     & +    F13230*coarse( c1  , c2+1, c3  )
     & +     F8820*coarse( c1-1, c2+1, c3  )
     & -     F7938*coarse( c1  , c2  , c3+1)
     & -     F5292*coarse( c1-1, c2  , c3+1)
     & -     F5292*coarse( c1  , c2+1, c3+1)
     & -     F3528*coarse( c1-1, c2+1, c3+1)
     & -     F2835*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2+2, c3  )
     & +     F2205*coarse( c1  , c2  , c3+2)
     & -     F1890*coarse( c1-2, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2+2, c3  )
     & +     F1470*coarse( c1-1, c2  , c3+2)
     & +     F1470*coarse( c1  , c2+1, c3+2)
     & +     F1134*coarse( c1-2, c2  , c3+1)
     & +     F1134*coarse( c1  , c2+2, c3+1)
     & +      F980*coarse( c1-1, c2+1, c3+2)
     & +      F756*coarse( c1-2, c2+1, c3+1)
     & +      F756*coarse( c1-1, c2+2, c3+1)
     & +      F405*coarse( c1-2, c2+2, c3  )
     & -      F315*coarse( c1-2, c2  , c3+2)
     & -      F315*coarse( c1  , c2+2, c3+2)
     & -      F210*coarse( c1-2, c2+1, c3+2)
     & -      F210*coarse( c1-1, c2+2, c3+2)
     & -      F162*coarse( c1-2, c2+2, c3+1)
     & +       F45*coarse( c1-2, c2+2, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2+1, c3  )
     & -    F17010*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1-1, c2+1, c3  )
     & -    F11340*coarse( c1  , c2+1, c3+1)
     & +     F6804*coarse( c1-1, c2  , c3+1)
     & -     F6075*coarse( c1  , c2+2, c3  )
     & +     F4725*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2  , c3+2)
     & +     F4536*coarse( c1-1, c2+1, c3+1)
     & +     F3150*coarse( c1-2, c2+1, c3  )
     & +     F3150*coarse( c1  , c2+1, c3+2)
     & +     F2430*coarse( c1-1, c2+2, c3  )
     & +     F2430*coarse( c1  , c2+2, c3+1)
     & -     F1890*coarse( c1-2, c2  , c3+1)
     & -     F1890*coarse( c1-1, c2  , c3+2)
     & -     F1260*coarse( c1-2, c2+1, c3+1)
     & -     F1260*coarse( c1-1, c2+1, c3+2)
     & -      F972*coarse( c1-1, c2+2, c3+1)
     & -      F675*coarse( c1-2, c2+2, c3  )
     & -      F675*coarse( c1  , c2+2, c3+2)
     & +      F525*coarse( c1-2, c2  , c3+2)
     & +      F350*coarse( c1-2, c2+1, c3+2)
     & +      F270*coarse( c1-2, c2+2, c3+1)
     & +      F270*coarse( c1-1, c2+2, c3+2)
     & -       F75*coarse( c1-2, c2+2, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1-1, c2  , c3  )
     & +    F13230*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1-1, c2  , c3+1)
     & -     F7938*coarse( c1  , c2+1, c3  )
     & -     F5292*coarse( c1-1, c2+1, c3  )
     & -     F5292*coarse( c1  , c2+1, c3+1)
     & -     F3528*coarse( c1-1, c2+1, c3+1)
     & -     F2835*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3+2)
     & +     F2205*coarse( c1  , c2+2, c3  )
     & -     F1890*coarse( c1-2, c2  , c3+1)
     & -     F1890*coarse( c1-1, c2  , c3+2)
     & +     F1470*coarse( c1-1, c2+2, c3  )
     & +     F1470*coarse( c1  , c2+2, c3+1)
     & +     F1134*coarse( c1-2, c2+1, c3  )
     & +     F1134*coarse( c1  , c2+1, c3+2)
     & +      F980*coarse( c1-1, c2+2, c3+1)
     & +      F756*coarse( c1-2, c2+1, c3+1)
     & +      F756*coarse( c1-1, c2+1, c3+2)
     & +      F405*coarse( c1-2, c2  , c3+2)
     & -      F315*coarse( c1-2, c2+2, c3  )
     & -      F315*coarse( c1  , c2+2, c3+2)
     & -      F210*coarse( c1-2, c2+2, c3+1)
     & -      F210*coarse( c1-1, c2+2, c3+2)
     & -      F162*coarse( c1-2, c2+1, c3+2)
     & +       F45*coarse( c1-2, c2+2, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2  , c3+1)
     & -    F17010*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3+1)
     & -    F11340*coarse( c1  , c2+1, c3+1)
     & +     F6804*coarse( c1-1, c2+1, c3  )
     & -     F6075*coarse( c1  , c2  , c3+2)
     & +     F4725*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2+2, c3  )
     & +     F4536*coarse( c1-1, c2+1, c3+1)
     & +     F3150*coarse( c1-2, c2  , c3+1)
     & +     F3150*coarse( c1  , c2+2, c3+1)
     & +     F2430*coarse( c1-1, c2  , c3+2)
     & +     F2430*coarse( c1  , c2+1, c3+2)
     & -     F1890*coarse( c1-2, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2+2, c3  )
     & -     F1260*coarse( c1-2, c2+1, c3+1)
     & -     F1260*coarse( c1-1, c2+2, c3+1)
     & -      F972*coarse( c1-1, c2+1, c3+2)
     & -      F675*coarse( c1-2, c2  , c3+2)
     & -      F675*coarse( c1  , c2+2, c3+2)
     & +      F525*coarse( c1-2, c2+2, c3  )
     & +      F350*coarse( c1-2, c2+2, c3+1)
     & +      F270*coarse( c1-2, c2+1, c3+2)
     & +      F270*coarse( c1-1, c2+2, c3+2)
     & -       F75*coarse( c1-2, c2+2, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &       F9261*coarse( c1  , c2  , c3  )
     & +     F6174*coarse( c1-1, c2  , c3  )
     & +     F6174*coarse( c1  , c2+1, c3  )
     & +     F6174*coarse( c1  , c2  , c3+1)
     & +     F4116*coarse( c1-1, c2+1, c3  )
     & +     F4116*coarse( c1-1, c2  , c3+1)
     & +     F4116*coarse( c1  , c2+1, c3+1)
     & +     F2744*coarse( c1-1, c2+1, c3+1)
     & -     F1323*coarse( c1-2, c2  , c3  )
     & -     F1323*coarse( c1  , c2+2, c3  )
     & -     F1323*coarse( c1  , c2  , c3+2)
     & -      F882*coarse( c1-2, c2+1, c3  )
     & -      F882*coarse( c1-1, c2+2, c3  )
     & -      F882*coarse( c1-2, c2  , c3+1)
     & -      F882*coarse( c1  , c2+2, c3+1)
     & -      F882*coarse( c1-1, c2  , c3+2)
     & -      F882*coarse( c1  , c2+1, c3+2)
     & -      F588*coarse( c1-2, c2+1, c3+1)
     & -      F588*coarse( c1-1, c2+2, c3+1)
     & -      F588*coarse( c1-1, c2+1, c3+2)
     & +      F189*coarse( c1-2, c2+2, c3  )
     & +      F189*coarse( c1-2, c2  , c3+2)
     & +      F189*coarse( c1  , c2+2, c3+2)
     & +      F126*coarse( c1-2, c2+2, c3+1)
     & +      F126*coarse( c1-2, c2+1, c3+2)
     & +      F126*coarse( c1-1, c2+2, c3+2)
     & -       F27*coarse( c1-2, c2+2, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1  , c2+1, c3  )
     & +    F13230*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1  , c2+1, c3+1)
     & -     F7938*coarse( c1-1, c2  , c3  )
     & -     F5292*coarse( c1-1, c2+1, c3  )
     & -     F5292*coarse( c1-1, c2  , c3+1)
     & -     F3528*coarse( c1-1, c2+1, c3+1)
     & -     F2835*coarse( c1  , c2+2, c3  )
     & -     F2835*coarse( c1  , c2  , c3+2)
     & +     F2205*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3+1)
     & -     F1890*coarse( c1  , c2+1, c3+2)
     & +     F1470*coarse( c1-2, c2+1, c3  )
     & +     F1470*coarse( c1-2, c2  , c3+1)
     & +     F1134*coarse( c1-1, c2+2, c3  )
     & +     F1134*coarse( c1-1, c2  , c3+2)
     & +      F980*coarse( c1-2, c2+1, c3+1)
     & +      F756*coarse( c1-1, c2+2, c3+1)
     & +      F756*coarse( c1-1, c2+1, c3+2)
     & +      F405*coarse( c1  , c2+2, c3+2)
     & -      F315*coarse( c1-2, c2+2, c3  )
     & -      F315*coarse( c1-2, c2  , c3+2)
     & -      F210*coarse( c1-2, c2+2, c3+1)
     & -      F210*coarse( c1-2, c2+1, c3+2)
     & -      F162*coarse( c1-1, c2+2, c3+2)
     & +       F45*coarse( c1-2, c2+2, c3+2)
     & )
          end if
c
c Set cint(4)-cint(2) edge
          f31 = f3i
          do c3 = c3i,c3f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1-1, c2  , c3  )
     & -    F11340*coarse( c1  , c2+1, c3  )
     & -     F7560*coarse( c1-1, c2+1, c3  )
     & +     F4725*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1-1, c2  , c3-1)
     & +     F3150*coarse( c1  , c2+2, c3  )
     & -     F2835*coarse( c1  , c2  , c3+1)
     & +     F2100*coarse( c1-1, c2+2, c3  )
     & -     F1890*coarse( c1  , c2+1, c3-1)
     & -     F1890*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1-2, c2+1, c3  )
     & -     F1260*coarse( c1-1, c2+1, c3-1)
     & +     F1134*coarse( c1  , c2+1, c3+1)
     & +      F756*coarse( c1-1, c2+1, c3+1)
     & -      F675*coarse( c1-2, c2  , c3-1)
     & +      F525*coarse( c1  , c2+2, c3-1)
     & -      F450*coarse( c1-2, c2+2, c3  )
     & +      F405*coarse( c1-2, c2  , c3+1)
     & +      F350*coarse( c1-1, c2+2, c3-1)
     & -      F315*coarse( c1  , c2+2, c3+1)
     & +      F270*coarse( c1-2, c2+1, c3-1)
     & -      F210*coarse( c1-1, c2+2, c3+1)
     & -      F162*coarse( c1-2, c2+1, c3+1)
     & -       F75*coarse( c1-2, c2+2, c3-1)
     & +       F45*coarse( c1-2, c2+2, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     & +    F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1-1, c2  , c3  )
     & -    F24300*coarse( c1  , c2+1, c3  )
     & +    F10125*coarse( c1  , c2  , c3-1)
     & +     F9720*coarse( c1-1, c2+1, c3  )
     & +     F6750*coarse( c1-2, c2  , c3  )
     & +     F6750*coarse( c1  , c2+2, c3  )
     & -     F6075*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1-1, c2  , c3-1)
     & -     F4050*coarse( c1  , c2+1, c3-1)
     & -     F2700*coarse( c1-2, c2+1, c3  )
     & -     F2700*coarse( c1-1, c2+2, c3  )
     & +     F2430*coarse( c1-1, c2  , c3+1)
     & +     F2430*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1-1, c2+1, c3-1)
     & +     F1125*coarse( c1-2, c2  , c3-1)
     & +     F1125*coarse( c1  , c2+2, c3-1)
     & -      F972*coarse( c1-1, c2+1, c3+1)
     & +      F750*coarse( c1-2, c2+2, c3  )
     & -      F675*coarse( c1-2, c2  , c3+1)
     & -      F675*coarse( c1  , c2+2, c3+1)
     & -      F450*coarse( c1-2, c2+1, c3-1)
     & -      F450*coarse( c1-1, c2+2, c3-1)
     & +      F270*coarse( c1-2, c2+1, c3+1)
     & +      F270*coarse( c1-1, c2+2, c3+1)
     & +      F125*coarse( c1-2, c2+2, c3-1)
     & -       F75*coarse( c1-2, c2+2, c3+1)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     & +    F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1-1, c2  , c3  )
     & +     F8820*coarse( c1  , c2+1, c3  )
     & +     F5880*coarse( c1-1, c2+1, c3  )
     & +     F2205*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3  )
     & +     F1470*coarse( c1-1, c2  , c3-1)
     & +     F1470*coarse( c1  , c2+1, c3-1)
     & -     F1323*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1-2, c2+1, c3  )
     & -     F1260*coarse( c1-1, c2+2, c3  )
     & +      F980*coarse( c1-1, c2+1, c3-1)
     & -      F882*coarse( c1-1, c2  , c3+1)
     & -      F882*coarse( c1  , c2+1, c3+1)
     & -      F588*coarse( c1-1, c2+1, c3+1)
     & -      F315*coarse( c1-2, c2  , c3-1)
     & -      F315*coarse( c1  , c2+2, c3-1)
     & +      F270*coarse( c1-2, c2+2, c3  )
     & -      F210*coarse( c1-2, c2+1, c3-1)
     & -      F210*coarse( c1-1, c2+2, c3-1)
     & +      F189*coarse( c1-2, c2  , c3+1)
     & +      F189*coarse( c1  , c2+2, c3+1)
     & +      F126*coarse( c1-2, c2+1, c3+1)
     & +      F126*coarse( c1-1, c2+2, c3+1)
     & +       F45*coarse( c1-2, c2+2, c3-1)
     & -       F27*coarse( c1-2, c2+2, c3+1)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3  )
     & -     F7560*coarse( c1-1, c2+1, c3  )
     & +     F4725*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1  , c2+1, c3-1)
     & +     F3150*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3+1)
     & +     F2100*coarse( c1-2, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3-1)
     & -     F1890*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1-1, c2+2, c3  )
     & -     F1260*coarse( c1-1, c2+1, c3-1)
     & +     F1134*coarse( c1-1, c2  , c3+1)
     & +      F756*coarse( c1-1, c2+1, c3+1)
     & -      F675*coarse( c1  , c2+2, c3-1)
     & +      F525*coarse( c1-2, c2  , c3-1)
     & -      F450*coarse( c1-2, c2+2, c3  )
     & +      F405*coarse( c1  , c2+2, c3+1)
     & +      F350*coarse( c1-2, c2+1, c3-1)
     & -      F315*coarse( c1-2, c2  , c3+1)
     & +      F270*coarse( c1-1, c2+2, c3-1)
     & -      F210*coarse( c1-2, c2+1, c3+1)
     & -      F162*coarse( c1-1, c2+2, c3+1)
     & -       F75*coarse( c1-2, c2+2, c3-1)
     & +       F45*coarse( c1-2, c2+2, c3+1)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1-1, c2  , c3  )
     & -    F11340*coarse( c1  , c2+1, c3  )
     & -     F7560*coarse( c1-1, c2+1, c3  )
     & +     F4725*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3+1)
     & -     F2835*coarse( c1  , c2  , c3-1)
     & +     F2100*coarse( c1-1, c2+2, c3  )
     & -     F1890*coarse( c1-1, c2  , c3-1)
     & -     F1890*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1-2, c2+1, c3  )
     & -     F1260*coarse( c1-1, c2+1, c3+1)
     & +     F1134*coarse( c1  , c2+1, c3-1)
     & +      F756*coarse( c1-1, c2+1, c3-1)
     & -      F675*coarse( c1-2, c2  , c3+1)
     & +      F525*coarse( c1  , c2+2, c3+1)
     & -      F450*coarse( c1-2, c2+2, c3  )
     & +      F405*coarse( c1-2, c2  , c3-1)
     & +      F350*coarse( c1-1, c2+2, c3+1)
     & -      F315*coarse( c1  , c2+2, c3-1)
     & +      F270*coarse( c1-2, c2+1, c3+1)
     & -      F210*coarse( c1-1, c2+2, c3-1)
     & -      F162*coarse( c1-2, c2+1, c3-1)
     & -       F75*coarse( c1-2, c2+2, c3+1)
     & +       F45*coarse( c1-2, c2+2, c3-1)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     & +    F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1-1, c2  , c3  )
     & -    F24300*coarse( c1  , c2+1, c3  )
     & +    F10125*coarse( c1  , c2  , c3+1)
     & +     F9720*coarse( c1-1, c2+1, c3  )
     & +     F6750*coarse( c1-2, c2  , c3  )
     & +     F6750*coarse( c1  , c2+2, c3  )
     & -     F6075*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1-1, c2  , c3+1)
     & -     F4050*coarse( c1  , c2+1, c3+1)
     & -     F2700*coarse( c1-2, c2+1, c3  )
     & -     F2700*coarse( c1-1, c2+2, c3  )
     & +     F2430*coarse( c1-1, c2  , c3-1)
     & +     F2430*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1-1, c2+1, c3+1)
     & +     F1125*coarse( c1-2, c2  , c3+1)
     & +     F1125*coarse( c1  , c2+2, c3+1)
     & -      F972*coarse( c1-1, c2+1, c3-1)
     & +      F750*coarse( c1-2, c2+2, c3  )
     & -      F675*coarse( c1-2, c2  , c3-1)
     & -      F675*coarse( c1  , c2+2, c3-1)
     & -      F450*coarse( c1-2, c2+1, c3+1)
     & -      F450*coarse( c1-1, c2+2, c3+1)
     & +      F270*coarse( c1-2, c2+1, c3-1)
     & +      F270*coarse( c1-1, c2+2, c3-1)
     & +      F125*coarse( c1-2, c2+2, c3+1)
     & -       F75*coarse( c1-2, c2+2, c3-1)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     & +    F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1-1, c2  , c3  )
     & +     F8820*coarse( c1  , c2+1, c3  )
     & +     F5880*coarse( c1-1, c2+1, c3  )
     & +     F2205*coarse( c1  , c2  , c3+1)
     & -     F1890*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3  )
     & +     F1470*coarse( c1-1, c2  , c3+1)
     & +     F1470*coarse( c1  , c2+1, c3+1)
     & -     F1323*coarse( c1  , c2  , c3-1)
     & -     F1260*coarse( c1-2, c2+1, c3  )
     & -     F1260*coarse( c1-1, c2+2, c3  )
     & +      F980*coarse( c1-1, c2+1, c3+1)
     & -      F882*coarse( c1-1, c2  , c3-1)
     & -      F882*coarse( c1  , c2+1, c3-1)
     & -      F588*coarse( c1-1, c2+1, c3-1)
     & -      F315*coarse( c1-2, c2  , c3+1)
     & -      F315*coarse( c1  , c2+2, c3+1)
     & +      F270*coarse( c1-2, c2+2, c3  )
     & -      F210*coarse( c1-2, c2+1, c3+1)
     & -      F210*coarse( c1-1, c2+2, c3+1)
     & +      F189*coarse( c1-2, c2  , c3-1)
     & +      F189*coarse( c1  , c2+2, c3-1)
     & +      F126*coarse( c1-2, c2+1, c3-1)
     & +      F126*coarse( c1-1, c2+2, c3-1)
     & +       F45*coarse( c1-2, c2+2, c3+1)
     & -       F27*coarse( c1-2, c2+2, c3-1)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3  )
     & -     F7560*coarse( c1-1, c2+1, c3  )
     & +     F4725*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1  , c2+2, c3  )
     & +     F3150*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3+1)
     & -     F2835*coarse( c1  , c2  , c3-1)
     & +     F2100*coarse( c1-2, c2+1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3-1)
     & -     F1890*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1-1, c2+2, c3  )
     & -     F1260*coarse( c1-1, c2+1, c3+1)
     & +     F1134*coarse( c1-1, c2  , c3-1)
     & +      F756*coarse( c1-1, c2+1, c3-1)
     & -      F675*coarse( c1  , c2+2, c3+1)
     & +      F525*coarse( c1-2, c2  , c3+1)
     & -      F450*coarse( c1-2, c2+2, c3  )
     & +      F405*coarse( c1  , c2+2, c3-1)
     & +      F350*coarse( c1-2, c2+1, c3+1)
     & -      F315*coarse( c1-2, c2  , c3-1)
     & +      F270*coarse( c1-1, c2+2, c3+1)
     & -      F210*coarse( c1-2, c2+1, c3-1)
     & -      F162*coarse( c1-1, c2+2, c3-1)
     & -       F75*coarse( c1-2, c2+2, c3+1)
     & +       F45*coarse( c1-2, c2+2, c3-1)
     & )
            f31 = f31 + 2
          end do
c
c Set cint(4)-cint(2)-cint(6) corner
          if(c3f.lt.cint(6)) then
            c3 = cint(6)
            f31 = 2*c3
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1-1, c2  , c3  )
     & +    F13230*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1-1, c2  , c3-1)
     & -     F7938*coarse( c1  , c2+1, c3  )
     & -     F5292*coarse( c1-1, c2+1, c3  )
     & -     F5292*coarse( c1  , c2+1, c3-1)
     & -     F3528*coarse( c1-1, c2+1, c3-1)
     & -     F2835*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3-2)
     & +     F2205*coarse( c1  , c2+2, c3  )
     & -     F1890*coarse( c1-2, c2  , c3-1)
     & -     F1890*coarse( c1-1, c2  , c3-2)
     & +     F1470*coarse( c1-1, c2+2, c3  )
     & +     F1470*coarse( c1  , c2+2, c3-1)
     & +     F1134*coarse( c1-2, c2+1, c3  )
     & +     F1134*coarse( c1  , c2+1, c3-2)
     & +      F980*coarse( c1-1, c2+2, c3-1)
     & +      F756*coarse( c1-2, c2+1, c3-1)
     & +      F756*coarse( c1-1, c2+1, c3-2)
     & +      F405*coarse( c1-2, c2  , c3-2)
     & -      F315*coarse( c1-2, c2+2, c3  )
     & -      F315*coarse( c1  , c2+2, c3-2)
     & -      F210*coarse( c1-2, c2+2, c3-1)
     & -      F210*coarse( c1-1, c2+2, c3-2)
     & -      F162*coarse( c1-2, c2+1, c3-2)
     & +       F45*coarse( c1-2, c2+2, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2  , c3-1)
     & -    F17010*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2+1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3-1)
     & -    F11340*coarse( c1  , c2+1, c3-1)
     & +     F6804*coarse( c1-1, c2+1, c3  )
     & -     F6075*coarse( c1  , c2  , c3-2)
     & +     F4725*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2+2, c3  )
     & +     F4536*coarse( c1-1, c2+1, c3-1)
     & +     F3150*coarse( c1-2, c2  , c3-1)
     & +     F3150*coarse( c1  , c2+2, c3-1)
     & +     F2430*coarse( c1-1, c2  , c3-2)
     & +     F2430*coarse( c1  , c2+1, c3-2)
     & -     F1890*coarse( c1-2, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2+2, c3  )
     & -     F1260*coarse( c1-2, c2+1, c3-1)
     & -     F1260*coarse( c1-1, c2+2, c3-1)
     & -      F972*coarse( c1-1, c2+1, c3-2)
     & -      F675*coarse( c1-2, c2  , c3-2)
     & -      F675*coarse( c1  , c2+2, c3-2)
     & +      F525*coarse( c1-2, c2+2, c3  )
     & +      F350*coarse( c1-2, c2+2, c3-1)
     & +      F270*coarse( c1-2, c2+1, c3-2)
     & +      F270*coarse( c1-1, c2+2, c3-2)
     & -       F75*coarse( c1-2, c2+2, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &       F9261*coarse( c1  , c2  , c3  )
     & +     F6174*coarse( c1-1, c2  , c3  )
     & +     F6174*coarse( c1  , c2+1, c3  )
     & +     F6174*coarse( c1  , c2  , c3-1)
     & +     F4116*coarse( c1-1, c2+1, c3  )
     & +     F4116*coarse( c1-1, c2  , c3-1)
     & +     F4116*coarse( c1  , c2+1, c3-1)
     & +     F2744*coarse( c1-1, c2+1, c3-1)
     & -     F1323*coarse( c1-2, c2  , c3  )
     & -     F1323*coarse( c1  , c2+2, c3  )
     & -     F1323*coarse( c1  , c2  , c3-2)
     & -      F882*coarse( c1-2, c2+1, c3  )
     & -      F882*coarse( c1-1, c2+2, c3  )
     & -      F882*coarse( c1-2, c2  , c3-1)
     & -      F882*coarse( c1  , c2+2, c3-1)
     & -      F882*coarse( c1-1, c2  , c3-2)
     & -      F882*coarse( c1  , c2+1, c3-2)
     & -      F588*coarse( c1-2, c2+1, c3-1)
     & -      F588*coarse( c1-1, c2+2, c3-1)
     & -      F588*coarse( c1-1, c2+1, c3-2)
     & +      F189*coarse( c1-2, c2+2, c3  )
     & +      F189*coarse( c1-2, c2  , c3-2)
     & +      F189*coarse( c1  , c2+2, c3-2)
     & +      F126*coarse( c1-2, c2+2, c3-1)
     & +      F126*coarse( c1-2, c2+1, c3-2)
     & +      F126*coarse( c1-1, c2+2, c3-2)
     & -       F27*coarse( c1-2, c2+2, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1  , c2+1, c3  )
     & +    F13230*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1  , c2+1, c3-1)
     & -     F7938*coarse( c1-1, c2  , c3  )
     & -     F5292*coarse( c1-1, c2+1, c3  )
     & -     F5292*coarse( c1-1, c2  , c3-1)
     & -     F3528*coarse( c1-1, c2+1, c3-1)
     & -     F2835*coarse( c1  , c2+2, c3  )
     & -     F2835*coarse( c1  , c2  , c3-2)
     & +     F2205*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2+2, c3-1)
     & -     F1890*coarse( c1  , c2+1, c3-2)
     & +     F1470*coarse( c1-2, c2+1, c3  )
     & +     F1470*coarse( c1-2, c2  , c3-1)
     & +     F1134*coarse( c1-1, c2+2, c3  )
     & +     F1134*coarse( c1-1, c2  , c3-2)
     & +      F980*coarse( c1-2, c2+1, c3-1)
     & +      F756*coarse( c1-1, c2+2, c3-1)
     & +      F756*coarse( c1-1, c2+1, c3-2)
     & +      F405*coarse( c1  , c2+2, c3-2)
     & -      F315*coarse( c1-2, c2+2, c3  )
     & -      F315*coarse( c1-2, c2  , c3-2)
     & -      F210*coarse( c1-2, c2+2, c3-1)
     & -      F210*coarse( c1-2, c2+1, c3-2)
     & -      F162*coarse( c1-1, c2+2, c3-2)
     & +       F45*coarse( c1-2, c2+2, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2+1, c3  )
     & -    F17010*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1-1, c2+1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3-1)
     & +     F6804*coarse( c1  , c2+1, c3-1)
     & -     F6075*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2+2, c3  )
     & +     F4725*coarse( c1  , c2  , c3-2)
     & +     F4536*coarse( c1-1, c2+1, c3-1)
     & +     F3150*coarse( c1-1, c2+2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3-2)
     & +     F2430*coarse( c1-2, c2+1, c3  )
     & +     F2430*coarse( c1-2, c2  , c3-1)
     & -     F1890*coarse( c1  , c2+2, c3-1)
     & -     F1890*coarse( c1  , c2+1, c3-2)
     & -     F1260*coarse( c1-1, c2+2, c3-1)
     & -     F1260*coarse( c1-1, c2+1, c3-2)
     & -      F972*coarse( c1-2, c2+1, c3-1)
     & -      F675*coarse( c1-2, c2+2, c3  )
     & -      F675*coarse( c1-2, c2  , c3-2)
     & +      F525*coarse( c1  , c2+2, c3-2)
     & +      F350*coarse( c1-1, c2+2, c3-2)
     & +      F270*coarse( c1-2, c2+2, c3-1)
     & +      F270*coarse( c1-2, c2+1, c3-2)
     & -       F75*coarse( c1-2, c2+2, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F91125*coarse( c1  , c2  , c3  )
     & -    F36450*coarse( c1-1, c2  , c3  )
     & -    F36450*coarse( c1  , c2+1, c3  )
     & -    F36450*coarse( c1  , c2  , c3-1)
     & +    F14580*coarse( c1-1, c2+1, c3  )
     & +    F14580*coarse( c1-1, c2  , c3-1)
     & +    F14580*coarse( c1  , c2+1, c3-1)
     & +    F10125*coarse( c1-2, c2  , c3  )
     & +    F10125*coarse( c1  , c2+2, c3  )
     & +    F10125*coarse( c1  , c2  , c3-2)
     & -     F5832*coarse( c1-1, c2+1, c3-1)
     & -     F4050*coarse( c1-2, c2+1, c3  )
     & -     F4050*coarse( c1-1, c2+2, c3  )
     & -     F4050*coarse( c1-2, c2  , c3-1)
     & -     F4050*coarse( c1  , c2+2, c3-1)
     & -     F4050*coarse( c1-1, c2  , c3-2)
     & -     F4050*coarse( c1  , c2+1, c3-2)
     & +     F1620*coarse( c1-2, c2+1, c3-1)
     & +     F1620*coarse( c1-1, c2+2, c3-1)
     & +     F1620*coarse( c1-1, c2+1, c3-2)
     & +     F1125*coarse( c1-2, c2+2, c3  )
     & +     F1125*coarse( c1-2, c2  , c3-2)
     & +     F1125*coarse( c1  , c2+2, c3-2)
     & -      F450*coarse( c1-2, c2+2, c3-1)
     & -      F450*coarse( c1-2, c2+1, c3-2)
     & -      F450*coarse( c1-1, c2+2, c3-2)
     & +      F125*coarse( c1-2, c2+2, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1-1, c2  , c3  )
     & +    F13230*coarse( c1  , c2+1, c3  )
     & +     F8820*coarse( c1-1, c2+1, c3  )
     & -     F7938*coarse( c1  , c2  , c3-1)
     & -     F5292*coarse( c1-1, c2  , c3-1)
     & -     F5292*coarse( c1  , c2+1, c3-1)
     & -     F3528*coarse( c1-1, c2+1, c3-1)
     & -     F2835*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2+2, c3  )
     & +     F2205*coarse( c1  , c2  , c3-2)
     & -     F1890*coarse( c1-2, c2+1, c3  )
     & -     F1890*coarse( c1-1, c2+2, c3  )
     & +     F1470*coarse( c1-1, c2  , c3-2)
     & +     F1470*coarse( c1  , c2+1, c3-2)
     & +     F1134*coarse( c1-2, c2  , c3-1)
     & +     F1134*coarse( c1  , c2+2, c3-1)
     & +      F980*coarse( c1-1, c2+1, c3-2)
     & +      F756*coarse( c1-2, c2+1, c3-1)
     & +      F756*coarse( c1-1, c2+2, c3-1)
     & +      F405*coarse( c1-2, c2+2, c3  )
     & -      F315*coarse( c1-2, c2  , c3-2)
     & -      F315*coarse( c1  , c2+2, c3-2)
     & -      F210*coarse( c1-2, c2+1, c3-2)
     & -      F210*coarse( c1-1, c2+2, c3-2)
     & -      F162*coarse( c1-2, c2+2, c3-1)
     & +       F45*coarse( c1-2, c2+2, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2+1, c3  )
     & -    F17010*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1-1, c2+1, c3  )
     & -    F11340*coarse( c1  , c2+1, c3-1)
     & +     F6804*coarse( c1-1, c2  , c3-1)
     & -     F6075*coarse( c1  , c2+2, c3  )
     & +     F4725*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2  , c3-2)
     & +     F4536*coarse( c1-1, c2+1, c3-1)
     & +     F3150*coarse( c1-2, c2+1, c3  )
     & +     F3150*coarse( c1  , c2+1, c3-2)
     & +     F2430*coarse( c1-1, c2+2, c3  )
     & +     F2430*coarse( c1  , c2+2, c3-1)
     & -     F1890*coarse( c1-2, c2  , c3-1)
     & -     F1890*coarse( c1-1, c2  , c3-2)
     & -     F1260*coarse( c1-2, c2+1, c3-1)
     & -     F1260*coarse( c1-1, c2+1, c3-2)
     & -      F972*coarse( c1-1, c2+2, c3-1)
     & -      F675*coarse( c1-2, c2+2, c3  )
     & -      F675*coarse( c1  , c2+2, c3-2)
     & +      F525*coarse( c1-2, c2  , c3-2)
     & +      F350*coarse( c1-2, c2+1, c3-2)
     & +      F270*coarse( c1-2, c2+2, c3-1)
     & +      F270*coarse( c1-1, c2+2, c3-2)
     & -       F75*coarse( c1-2, c2+2, c3-2)
     & )
          end if
        end if
c
c Set cint(4)-cint(3) edge
        if(c3i.gt.cint(3)) then
          c3 = cint(3)
          f31 = 2*c3
          f21 = f2i
          do c2 = c2i,c2f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1-1, c2  , c3  )
     & -    F11340*coarse( c1  , c2  , c3+1)
     & -     F7560*coarse( c1-1, c2  , c3+1)
     & +     F4725*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1-1, c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+2)
     & -     F2835*coarse( c1  , c2+1, c3  )
     & +     F2100*coarse( c1-1, c2  , c3+2)
     & -     F1890*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1-2, c2  , c3+1)
     & -     F1260*coarse( c1-1, c2-1, c3+1)
     & +     F1134*coarse( c1  , c2+1, c3+1)
     & +      F756*coarse( c1-1, c2+1, c3+1)
     & -      F675*coarse( c1-2, c2-1, c3  )
     & +      F525*coarse( c1  , c2-1, c3+2)
     & -      F450*coarse( c1-2, c2  , c3+2)
     & +      F405*coarse( c1-2, c2+1, c3  )
     & +      F350*coarse( c1-1, c2-1, c3+2)
     & -      F315*coarse( c1  , c2+1, c3+2)
     & +      F270*coarse( c1-2, c2-1, c3+1)
     & -      F210*coarse( c1-1, c2+1, c3+2)
     & -      F162*coarse( c1-2, c2+1, c3+1)
     & -       F75*coarse( c1-2, c2-1, c3+2)
     & +       F45*coarse( c1-2, c2+1, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1-1, c2  , c3  )
     & -    F24300*coarse( c1  , c2  , c3+1)
     & +    F10125*coarse( c1  , c2-1, c3  )
     & +     F9720*coarse( c1-1, c2  , c3+1)
     & +     F6750*coarse( c1-2, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3+2)
     & -     F6075*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1-1, c2-1, c3  )
     & -     F4050*coarse( c1  , c2-1, c3+1)
     & -     F2700*coarse( c1-2, c2  , c3+1)
     & -     F2700*coarse( c1-1, c2  , c3+2)
     & +     F2430*coarse( c1-1, c2+1, c3  )
     & +     F2430*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1-1, c2-1, c3+1)
     & +     F1125*coarse( c1-2, c2-1, c3  )
     & +     F1125*coarse( c1  , c2-1, c3+2)
     & -      F972*coarse( c1-1, c2+1, c3+1)
     & +      F750*coarse( c1-2, c2  , c3+2)
     & -      F675*coarse( c1-2, c2+1, c3  )
     & -      F675*coarse( c1  , c2+1, c3+2)
     & -      F450*coarse( c1-2, c2-1, c3+1)
     & -      F450*coarse( c1-1, c2-1, c3+2)
     & +      F270*coarse( c1-2, c2+1, c3+1)
     & +      F270*coarse( c1-1, c2+1, c3+2)
     & +      F125*coarse( c1-2, c2-1, c3+2)
     & -       F75*coarse( c1-2, c2+1, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1-1, c2  , c3  )
     & -    F11340*coarse( c1  , c2  , c3+1)
     & -     F7560*coarse( c1-1, c2  , c3+1)
     & +     F4725*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1-1, c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+2)
     & -     F2835*coarse( c1  , c2-1, c3  )
     & +     F2100*coarse( c1-1, c2  , c3+2)
     & -     F1890*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1-2, c2  , c3+1)
     & -     F1260*coarse( c1-1, c2+1, c3+1)
     & +     F1134*coarse( c1  , c2-1, c3+1)
     & +      F756*coarse( c1-1, c2-1, c3+1)
     & -      F675*coarse( c1-2, c2+1, c3  )
     & +      F525*coarse( c1  , c2+1, c3+2)
     & -      F450*coarse( c1-2, c2  , c3+2)
     & +      F405*coarse( c1-2, c2-1, c3  )
     & +      F350*coarse( c1-1, c2+1, c3+2)
     & -      F315*coarse( c1  , c2-1, c3+2)
     & +      F270*coarse( c1-2, c2+1, c3+1)
     & -      F210*coarse( c1-1, c2-1, c3+2)
     & -      F162*coarse( c1-2, c2-1, c3+1)
     & -       F75*coarse( c1-2, c2+1, c3+2)
     & +       F45*coarse( c1-2, c2-1, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2  , c3+1)
     & -    F24300*coarse( c1-1, c2  , c3  )
     & +    F10125*coarse( c1  , c2+1, c3  )
     & +     F9720*coarse( c1-1, c2  , c3+1)
     & +     F6750*coarse( c1-2, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3+2)
     & -     F6075*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1-1, c2+1, c3  )
     & -     F4050*coarse( c1  , c2+1, c3+1)
     & -     F2700*coarse( c1-2, c2  , c3+1)
     & -     F2700*coarse( c1-1, c2  , c3+2)
     & +     F2430*coarse( c1-1, c2-1, c3  )
     & +     F2430*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1-1, c2+1, c3+1)
     & +     F1125*coarse( c1-2, c2+1, c3  )
     & +     F1125*coarse( c1  , c2+1, c3+2)
     & -      F972*coarse( c1-1, c2-1, c3+1)
     & +      F750*coarse( c1-2, c2  , c3+2)
     & -      F675*coarse( c1-2, c2-1, c3  )
     & -      F675*coarse( c1  , c2-1, c3+2)
     & -      F450*coarse( c1-2, c2+1, c3+1)
     & -      F450*coarse( c1-1, c2+1, c3+2)
     & +      F270*coarse( c1-2, c2-1, c3+1)
     & +      F270*coarse( c1-1, c2-1, c3+2)
     & +      F125*coarse( c1-2, c2+1, c3+2)
     & -       F75*coarse( c1-2, c2-1, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1-1, c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3+1)
     & +     F5880*coarse( c1-1, c2  , c3+1)
     & +     F2205*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3+2)
     & +     F1470*coarse( c1-1, c2-1, c3  )
     & +     F1470*coarse( c1  , c2-1, c3+1)
     & -     F1323*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1-2, c2  , c3+1)
     & -     F1260*coarse( c1-1, c2  , c3+2)
     & +      F980*coarse( c1-1, c2-1, c3+1)
     & -      F882*coarse( c1-1, c2+1, c3  )
     & -      F882*coarse( c1  , c2+1, c3+1)
     & -      F588*coarse( c1-1, c2+1, c3+1)
     & -      F315*coarse( c1-2, c2-1, c3  )
     & -      F315*coarse( c1  , c2-1, c3+2)
     & +      F270*coarse( c1-2, c2  , c3+2)
     & -      F210*coarse( c1-2, c2-1, c3+1)
     & -      F210*coarse( c1-1, c2-1, c3+2)
     & +      F189*coarse( c1-2, c2+1, c3  )
     & +      F189*coarse( c1  , c2+1, c3+2)
     & +      F126*coarse( c1-2, c2+1, c3+1)
     & +      F126*coarse( c1-1, c2+1, c3+2)
     & +       F45*coarse( c1-2, c2-1, c3+2)
     & -       F27*coarse( c1-2, c2+1, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1-1, c2  , c3  )
     & -     F7560*coarse( c1-1, c2  , c3+1)
     & +     F4725*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1  , c2-1, c3+1)
     & +     F3150*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2+1, c3  )
     & +     F2100*coarse( c1-2, c2  , c3+1)
     & -     F1890*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3+1)
     & +     F1620*coarse( c1-1, c2  , c3+2)
     & -     F1260*coarse( c1-1, c2-1, c3+1)
     & +     F1134*coarse( c1-1, c2+1, c3  )
     & +      F756*coarse( c1-1, c2+1, c3+1)
     & -      F675*coarse( c1  , c2-1, c3+2)
     & +      F525*coarse( c1-2, c2-1, c3  )
     & -      F450*coarse( c1-2, c2  , c3+2)
     & +      F405*coarse( c1  , c2+1, c3+2)
     & +      F350*coarse( c1-2, c2-1, c3+1)
     & -      F315*coarse( c1-2, c2+1, c3  )
     & +      F270*coarse( c1-1, c2-1, c3+2)
     & -      F210*coarse( c1-2, c2+1, c3+1)
     & -      F162*coarse( c1-1, c2+1, c3+2)
     & -       F75*coarse( c1-2, c2-1, c3+2)
     & +       F45*coarse( c1-2, c2+1, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1-1, c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3+1)
     & +     F5880*coarse( c1-1, c2  , c3+1)
     & +     F2205*coarse( c1  , c2+1, c3  )
     & -     F1890*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3+2)
     & +     F1470*coarse( c1-1, c2+1, c3  )
     & +     F1470*coarse( c1  , c2+1, c3+1)
     & -     F1323*coarse( c1  , c2-1, c3  )
     & -     F1260*coarse( c1-2, c2  , c3+1)
     & -     F1260*coarse( c1-1, c2  , c3+2)
     & +      F980*coarse( c1-1, c2+1, c3+1)
     & -      F882*coarse( c1-1, c2-1, c3  )
     & -      F882*coarse( c1  , c2-1, c3+1)
     & -      F588*coarse( c1-1, c2-1, c3+1)
     & -      F315*coarse( c1-2, c2+1, c3  )
     & -      F315*coarse( c1  , c2+1, c3+2)
     & +      F270*coarse( c1-2, c2  , c3+2)
     & -      F210*coarse( c1-2, c2+1, c3+1)
     & -      F210*coarse( c1-1, c2+1, c3+2)
     & +      F189*coarse( c1-2, c2-1, c3  )
     & +      F189*coarse( c1  , c2-1, c3+2)
     & +      F126*coarse( c1-2, c2-1, c3+1)
     & +      F126*coarse( c1-1, c2-1, c3+2)
     & +       F45*coarse( c1-2, c2+1, c3+2)
     & -       F27*coarse( c1-2, c2-1, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1-1, c2  , c3  )
     & -     F7560*coarse( c1-1, c2  , c3+1)
     & +     F4725*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3+1)
     & -     F2835*coarse( c1  , c2-1, c3  )
     & +     F2100*coarse( c1-2, c2  , c3+1)
     & -     F1890*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1-1, c2  , c3+2)
     & -     F1260*coarse( c1-1, c2+1, c3+1)
     & +     F1134*coarse( c1-1, c2-1, c3  )
     & +      F756*coarse( c1-1, c2-1, c3+1)
     & -      F675*coarse( c1  , c2+1, c3+2)
     & +      F525*coarse( c1-2, c2+1, c3  )
     & -      F450*coarse( c1-2, c2  , c3+2)
     & +      F405*coarse( c1  , c2-1, c3+2)
     & +      F350*coarse( c1-2, c2+1, c3+1)
     & -      F315*coarse( c1-2, c2-1, c3  )
     & +      F270*coarse( c1-1, c2+1, c3+2)
     & -      F210*coarse( c1-2, c2-1, c3+1)
     & -      F162*coarse( c1-1, c2-1, c3+2)
     & -       F75*coarse( c1-2, c2+1, c3+2)
     & +       F45*coarse( c1-2, c2-1, c3+2)
     & )
            f21 = f21 + 2
          end do
        end if
c
c Set cint(4) face
        f31 = f3i
        do c3 = c3i,c3f
          f21 = f2i
          do c2 = c2i,c2f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1-1, c2  , c3  )
     & +     F3150*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1  , c2-1, c3  )
     & -     F2700*coarse( c1-2, c2  , c3  )
     & +     F2100*coarse( c1-1, c2  , c3-1)
     & +     F2100*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3  )
     & -     F1890*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1-1, c2+1, c3  )
     & -     F1260*coarse( c1-1, c2  , c3+1)
     & +      F525*coarse( c1  , c2-1, c3-1)
     & -      F450*coarse( c1-2, c2  , c3-1)
     & -      F450*coarse( c1-2, c2-1, c3  )
     & +      F350*coarse( c1-1, c2-1, c3-1)
     & -      F315*coarse( c1  , c2+1, c3-1)
     & -      F315*coarse( c1  , c2-1, c3+1)
     & +      F270*coarse( c1-2, c2+1, c3  )
     & +      F270*coarse( c1-2, c2  , c3+1)
     & -      F210*coarse( c1-1, c2+1, c3-1)
     & -      F210*coarse( c1-1, c2-1, c3+1)
     & +      F189*coarse( c1  , c2+1, c3+1)
     & +      F126*coarse( c1-1, c2+1, c3+1)
     & -       F75*coarse( c1-2, c2-1, c3-1)
     & +       F45*coarse( c1-2, c2+1, c3-1)
     & +       F45*coarse( c1-2, c2-1, c3+1)
     & -       F27*coarse( c1-2, c2+1, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1  , c2-1, c3  )
     & +     F4500*coarse( c1-2, c2  , c3  )
     & -     F4050*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1-1, c2  , c3-1)
     & -     F2700*coarse( c1-1, c2-1, c3  )
     & +     F1620*coarse( c1-1, c2+1, c3  )
     & +     F1620*coarse( c1-1, c2  , c3+1)
     & +     F1125*coarse( c1  , c2-1, c3-1)
     & +      F750*coarse( c1-2, c2  , c3-1)
     & +      F750*coarse( c1-2, c2-1, c3  )
     & -      F675*coarse( c1  , c2+1, c3-1)
     & -      F675*coarse( c1  , c2-1, c3+1)
     & -      F450*coarse( c1-1, c2-1, c3-1)
     & -      F450*coarse( c1-2, c2+1, c3  )
     & -      F450*coarse( c1-2, c2  , c3+1)
     & +      F405*coarse( c1  , c2+1, c3+1)
     & +      F270*coarse( c1-1, c2+1, c3-1)
     & +      F270*coarse( c1-1, c2-1, c3+1)
     & -      F162*coarse( c1-1, c2+1, c3+1)
     & +      F125*coarse( c1-2, c2-1, c3-1)
     & -       F75*coarse( c1-2, c2+1, c3-1)
     & -       F75*coarse( c1-2, c2-1, c3+1)
     & +       F45*coarse( c1-2, c2+1, c3+1)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1-1, c2  , c3  )
     & +     F3150*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1-2, c2  , c3  )
     & +     F2100*coarse( c1-1, c2  , c3-1)
     & +     F2100*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1-1, c2-1, c3  )
     & -     F1260*coarse( c1-1, c2  , c3+1)
     & +      F525*coarse( c1  , c2+1, c3-1)
     & -      F450*coarse( c1-2, c2  , c3-1)
     & -      F450*coarse( c1-2, c2+1, c3  )
     & +      F350*coarse( c1-1, c2+1, c3-1)
     & -      F315*coarse( c1  , c2-1, c3-1)
     & -      F315*coarse( c1  , c2+1, c3+1)
     & +      F270*coarse( c1-2, c2-1, c3  )
     & +      F270*coarse( c1-2, c2  , c3+1)
     & -      F210*coarse( c1-1, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3+1)
     & +      F189*coarse( c1  , c2-1, c3+1)
     & +      F126*coarse( c1-1, c2-1, c3+1)
     & -       F75*coarse( c1-2, c2+1, c3-1)
     & +       F45*coarse( c1-2, c2-1, c3-1)
     & +       F45*coarse( c1-2, c2+1, c3+1)
     & -       F27*coarse( c1-2, c2-1, c3+1)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1  , c2+1, c3  )
     & +     F4500*coarse( c1-2, c2  , c3  )
     & -     F4050*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1-1, c2  , c3-1)
     & -     F2700*coarse( c1-1, c2+1, c3  )
     & +     F1620*coarse( c1-1, c2-1, c3  )
     & +     F1620*coarse( c1-1, c2  , c3+1)
     & +     F1125*coarse( c1  , c2+1, c3-1)
     & +      F750*coarse( c1-2, c2  , c3-1)
     & +      F750*coarse( c1-2, c2+1, c3  )
     & -      F675*coarse( c1  , c2-1, c3-1)
     & -      F675*coarse( c1  , c2+1, c3+1)
     & -      F450*coarse( c1-1, c2+1, c3-1)
     & -      F450*coarse( c1-2, c2-1, c3  )
     & -      F450*coarse( c1-2, c2  , c3+1)
     & +      F405*coarse( c1  , c2-1, c3+1)
     & +      F270*coarse( c1-1, c2-1, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3+1)
     & -      F162*coarse( c1-1, c2-1, c3+1)
     & +      F125*coarse( c1-2, c2+1, c3-1)
     & -       F75*coarse( c1-2, c2-1, c3-1)
     & -       F75*coarse( c1-2, c2+1, c3+1)
     & +       F45*coarse( c1-2, c2-1, c3+1)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1-1, c2  , c3  )
     & +     F3150*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1-2, c2  , c3  )
     & +     F2100*coarse( c1-1, c2-1, c3  )
     & +     F2100*coarse( c1-1, c2  , c3+1)
     & -     F1890*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1-1, c2  , c3-1)
     & -     F1260*coarse( c1-1, c2+1, c3  )
     & +      F525*coarse( c1  , c2-1, c3+1)
     & -      F450*coarse( c1-2, c2-1, c3  )
     & -      F450*coarse( c1-2, c2  , c3+1)
     & +      F350*coarse( c1-1, c2-1, c3+1)
     & -      F315*coarse( c1  , c2-1, c3-1)
     & -      F315*coarse( c1  , c2+1, c3+1)
     & +      F270*coarse( c1-2, c2  , c3-1)
     & +      F270*coarse( c1-2, c2+1, c3  )
     & -      F210*coarse( c1-1, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3+1)
     & +      F189*coarse( c1  , c2+1, c3-1)
     & +      F126*coarse( c1-1, c2+1, c3-1)
     & -       F75*coarse( c1-2, c2-1, c3+1)
     & +       F45*coarse( c1-2, c2-1, c3-1)
     & +       F45*coarse( c1-2, c2+1, c3+1)
     & -       F27*coarse( c1-2, c2+1, c3-1)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1  , c2  , c3+1)
     & +     F4500*coarse( c1-2, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1-1, c2-1, c3  )
     & -     F2700*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1-1, c2+1, c3  )
     & +     F1125*coarse( c1  , c2-1, c3+1)
     & +      F750*coarse( c1-2, c2-1, c3  )
     & +      F750*coarse( c1-2, c2  , c3+1)
     & -      F675*coarse( c1  , c2-1, c3-1)
     & -      F675*coarse( c1  , c2+1, c3+1)
     & -      F450*coarse( c1-2, c2  , c3-1)
     & -      F450*coarse( c1-2, c2+1, c3  )
     & -      F450*coarse( c1-1, c2-1, c3+1)
     & +      F405*coarse( c1  , c2+1, c3-1)
     & +      F270*coarse( c1-1, c2-1, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3+1)
     & -      F162*coarse( c1-1, c2+1, c3-1)
     & +      F125*coarse( c1-2, c2-1, c3+1)
     & -       F75*coarse( c1-2, c2-1, c3-1)
     & -       F75*coarse( c1-2, c2+1, c3+1)
     & +       F45*coarse( c1-2, c2+1, c3-1)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1-1, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1-2, c2  , c3  )
     & +     F2100*coarse( c1-1, c2+1, c3  )
     & +     F2100*coarse( c1-1, c2  , c3+1)
     & -     F1890*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1  , c2-1, c3  )
     & -     F1260*coarse( c1-1, c2  , c3-1)
     & -     F1260*coarse( c1-1, c2-1, c3  )
     & +      F525*coarse( c1  , c2+1, c3+1)
     & -      F450*coarse( c1-2, c2+1, c3  )
     & -      F450*coarse( c1-2, c2  , c3+1)
     & +      F350*coarse( c1-1, c2+1, c3+1)
     & -      F315*coarse( c1  , c2+1, c3-1)
     & -      F315*coarse( c1  , c2-1, c3+1)
     & +      F270*coarse( c1-2, c2  , c3-1)
     & +      F270*coarse( c1-2, c2-1, c3  )
     & -      F210*coarse( c1-1, c2+1, c3-1)
     & -      F210*coarse( c1-1, c2-1, c3+1)
     & +      F189*coarse( c1  , c2-1, c3-1)
     & +      F126*coarse( c1-1, c2-1, c3-1)
     & -       F75*coarse( c1-2, c2+1, c3+1)
     & +       F45*coarse( c1-2, c2+1, c3-1)
     & +       F45*coarse( c1-2, c2-1, c3+1)
     & -       F27*coarse( c1-2, c2-1, c3-1)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+1, c3  )
     & +     F6750*coarse( c1  , c2  , c3+1)
     & +     F4500*coarse( c1-2, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1  , c2-1, c3  )
     & -     F2700*coarse( c1-1, c2+1, c3  )
     & -     F2700*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1-1, c2-1, c3  )
     & +     F1125*coarse( c1  , c2+1, c3+1)
     & +      F750*coarse( c1-2, c2+1, c3  )
     & +      F750*coarse( c1-2, c2  , c3+1)
     & -      F675*coarse( c1  , c2+1, c3-1)
     & -      F675*coarse( c1  , c2-1, c3+1)
     & -      F450*coarse( c1-2, c2  , c3-1)
     & -      F450*coarse( c1-2, c2-1, c3  )
     & -      F450*coarse( c1-1, c2+1, c3+1)
     & +      F405*coarse( c1  , c2-1, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3-1)
     & +      F270*coarse( c1-1, c2-1, c3+1)
     & -      F162*coarse( c1-1, c2-1, c3-1)
     & +      F125*coarse( c1-2, c2+1, c3+1)
     & -       F75*coarse( c1-2, c2+1, c3-1)
     & -       F75*coarse( c1-2, c2-1, c3+1)
     & +       F45*coarse( c1-2, c2-1, c3-1)
     & )
            f21 = f21 + 2
          end do
          f31 = f31 + 2
        end do
c
c Set cint(4)-cint(6) edge
        if(c3f.lt.cint(6)) then
          c3 = cint(6)
          f31 = 2*c3
          f21 = f2i
          do c2 = c2i,c2f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1-1, c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3-1)
     & +     F5880*coarse( c1-1, c2  , c3-1)
     & +     F2205*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3-2)
     & +     F1470*coarse( c1-1, c2-1, c3  )
     & +     F1470*coarse( c1  , c2-1, c3-1)
     & -     F1323*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1-2, c2  , c3-1)
     & -     F1260*coarse( c1-1, c2  , c3-2)
     & +      F980*coarse( c1-1, c2-1, c3-1)
     & -      F882*coarse( c1-1, c2+1, c3  )
     & -      F882*coarse( c1  , c2+1, c3-1)
     & -      F588*coarse( c1-1, c2+1, c3-1)
     & -      F315*coarse( c1-2, c2-1, c3  )
     & -      F315*coarse( c1  , c2-1, c3-2)
     & +      F270*coarse( c1-2, c2  , c3-2)
     & -      F210*coarse( c1-2, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2-1, c3-2)
     & +      F189*coarse( c1-2, c2+1, c3  )
     & +      F189*coarse( c1  , c2+1, c3-2)
     & +      F126*coarse( c1-2, c2+1, c3-1)
     & +      F126*coarse( c1-1, c2+1, c3-2)
     & +       F45*coarse( c1-2, c2-1, c3-2)
     & -       F27*coarse( c1-2, c2+1, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1-1, c2  , c3  )
     & -     F7560*coarse( c1-1, c2  , c3-1)
     & +     F4725*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1  , c2-1, c3-1)
     & +     F3150*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2+1, c3  )
     & +     F2100*coarse( c1-2, c2  , c3-1)
     & -     F1890*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1-1, c2  , c3-2)
     & -     F1260*coarse( c1-1, c2-1, c3-1)
     & +     F1134*coarse( c1-1, c2+1, c3  )
     & +      F756*coarse( c1-1, c2+1, c3-1)
     & -      F675*coarse( c1  , c2-1, c3-2)
     & +      F525*coarse( c1-2, c2-1, c3  )
     & -      F450*coarse( c1-2, c2  , c3-2)
     & +      F405*coarse( c1  , c2+1, c3-2)
     & +      F350*coarse( c1-2, c2-1, c3-1)
     & -      F315*coarse( c1-2, c2+1, c3  )
     & +      F270*coarse( c1-1, c2-1, c3-2)
     & -      F210*coarse( c1-2, c2+1, c3-1)
     & -      F162*coarse( c1-1, c2+1, c3-2)
     & -       F75*coarse( c1-2, c2-1, c3-2)
     & +       F45*coarse( c1-2, c2+1, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1-1, c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3-1)
     & +     F5880*coarse( c1-1, c2  , c3-1)
     & +     F2205*coarse( c1  , c2+1, c3  )
     & -     F1890*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3-2)
     & +     F1470*coarse( c1-1, c2+1, c3  )
     & +     F1470*coarse( c1  , c2+1, c3-1)
     & -     F1323*coarse( c1  , c2-1, c3  )
     & -     F1260*coarse( c1-2, c2  , c3-1)
     & -     F1260*coarse( c1-1, c2  , c3-2)
     & +      F980*coarse( c1-1, c2+1, c3-1)
     & -      F882*coarse( c1-1, c2-1, c3  )
     & -      F882*coarse( c1  , c2-1, c3-1)
     & -      F588*coarse( c1-1, c2-1, c3-1)
     & -      F315*coarse( c1-2, c2+1, c3  )
     & -      F315*coarse( c1  , c2+1, c3-2)
     & +      F270*coarse( c1-2, c2  , c3-2)
     & -      F210*coarse( c1-2, c2+1, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3-2)
     & +      F189*coarse( c1-2, c2-1, c3  )
     & +      F189*coarse( c1  , c2-1, c3-2)
     & +      F126*coarse( c1-2, c2-1, c3-1)
     & +      F126*coarse( c1-1, c2-1, c3-2)
     & +       F45*coarse( c1-2, c2+1, c3-2)
     & -       F27*coarse( c1-2, c2-1, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1-1, c2  , c3  )
     & -     F7560*coarse( c1-1, c2  , c3-1)
     & +     F4725*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3-1)
     & -     F2835*coarse( c1  , c2-1, c3  )
     & +     F2100*coarse( c1-2, c2  , c3-1)
     & -     F1890*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1-1, c2  , c3-2)
     & -     F1260*coarse( c1-1, c2+1, c3-1)
     & +     F1134*coarse( c1-1, c2-1, c3  )
     & +      F756*coarse( c1-1, c2-1, c3-1)
     & -      F675*coarse( c1  , c2+1, c3-2)
     & +      F525*coarse( c1-2, c2+1, c3  )
     & -      F450*coarse( c1-2, c2  , c3-2)
     & +      F405*coarse( c1  , c2-1, c3-2)
     & +      F350*coarse( c1-2, c2+1, c3-1)
     & -      F315*coarse( c1-2, c2-1, c3  )
     & +      F270*coarse( c1-1, c2+1, c3-2)
     & -      F210*coarse( c1-2, c2-1, c3-1)
     & -      F162*coarse( c1-1, c2-1, c3-2)
     & -       F75*coarse( c1-2, c2+1, c3-2)
     & +       F45*coarse( c1-2, c2-1, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1-1, c2  , c3  )
     & -    F11340*coarse( c1  , c2  , c3-1)
     & -     F7560*coarse( c1-1, c2  , c3-1)
     & +     F4725*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1-1, c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-2)
     & -     F2835*coarse( c1  , c2+1, c3  )
     & +     F2100*coarse( c1-1, c2  , c3-2)
     & -     F1890*coarse( c1-1, c2+1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1-2, c2  , c3-1)
     & -     F1260*coarse( c1-1, c2-1, c3-1)
     & +     F1134*coarse( c1  , c2+1, c3-1)
     & +      F756*coarse( c1-1, c2+1, c3-1)
     & -      F675*coarse( c1-2, c2-1, c3  )
     & +      F525*coarse( c1  , c2-1, c3-2)
     & -      F450*coarse( c1-2, c2  , c3-2)
     & +      F405*coarse( c1-2, c2+1, c3  )
     & +      F350*coarse( c1-1, c2-1, c3-2)
     & -      F315*coarse( c1  , c2+1, c3-2)
     & +      F270*coarse( c1-2, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3-2)
     & -      F162*coarse( c1-2, c2+1, c3-1)
     & -       F75*coarse( c1-2, c2-1, c3-2)
     & +       F45*coarse( c1-2, c2+1, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1-1, c2  , c3  )
     & -    F24300*coarse( c1  , c2  , c3-1)
     & +    F10125*coarse( c1  , c2-1, c3  )
     & +     F9720*coarse( c1-1, c2  , c3-1)
     & +     F6750*coarse( c1-2, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3-2)
     & -     F6075*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1-1, c2-1, c3  )
     & -     F4050*coarse( c1  , c2-1, c3-1)
     & -     F2700*coarse( c1-2, c2  , c3-1)
     & -     F2700*coarse( c1-1, c2  , c3-2)
     & +     F2430*coarse( c1-1, c2+1, c3  )
     & +     F2430*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1-1, c2-1, c3-1)
     & +     F1125*coarse( c1-2, c2-1, c3  )
     & +     F1125*coarse( c1  , c2-1, c3-2)
     & -      F972*coarse( c1-1, c2+1, c3-1)
     & +      F750*coarse( c1-2, c2  , c3-2)
     & -      F675*coarse( c1-2, c2+1, c3  )
     & -      F675*coarse( c1  , c2+1, c3-2)
     & -      F450*coarse( c1-2, c2-1, c3-1)
     & -      F450*coarse( c1-1, c2-1, c3-2)
     & +      F270*coarse( c1-2, c2+1, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3-2)
     & +      F125*coarse( c1-2, c2-1, c3-2)
     & -       F75*coarse( c1-2, c2+1, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1-1, c2  , c3  )
     & -    F11340*coarse( c1  , c2  , c3-1)
     & -     F7560*coarse( c1-1, c2  , c3-1)
     & +     F4725*coarse( c1  , c2+1, c3  )
     & -     F4050*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1-1, c2+1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-2)
     & -     F2835*coarse( c1  , c2-1, c3  )
     & +     F2100*coarse( c1-1, c2  , c3-2)
     & -     F1890*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1-2, c2  , c3-1)
     & -     F1260*coarse( c1-1, c2+1, c3-1)
     & +     F1134*coarse( c1  , c2-1, c3-1)
     & +      F756*coarse( c1-1, c2-1, c3-1)
     & -      F675*coarse( c1-2, c2+1, c3  )
     & +      F525*coarse( c1  , c2+1, c3-2)
     & -      F450*coarse( c1-2, c2  , c3-2)
     & +      F405*coarse( c1-2, c2-1, c3  )
     & +      F350*coarse( c1-1, c2+1, c3-2)
     & -      F315*coarse( c1  , c2-1, c3-2)
     & +      F270*coarse( c1-2, c2+1, c3-1)
     & -      F210*coarse( c1-1, c2-1, c3-2)
     & -      F162*coarse( c1-2, c2-1, c3-1)
     & -       F75*coarse( c1-2, c2+1, c3-2)
     & +       F45*coarse( c1-2, c2-1, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2  , c3-1)
     & -    F24300*coarse( c1-1, c2  , c3  )
     & +    F10125*coarse( c1  , c2+1, c3  )
     & +     F9720*coarse( c1-1, c2  , c3-1)
     & +     F6750*coarse( c1-2, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3-2)
     & -     F6075*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1-1, c2+1, c3  )
     & -     F4050*coarse( c1  , c2+1, c3-1)
     & -     F2700*coarse( c1-2, c2  , c3-1)
     & -     F2700*coarse( c1-1, c2  , c3-2)
     & +     F2430*coarse( c1-1, c2-1, c3  )
     & +     F2430*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1-1, c2+1, c3-1)
     & +     F1125*coarse( c1-2, c2+1, c3  )
     & +     F1125*coarse( c1  , c2+1, c3-2)
     & -      F972*coarse( c1-1, c2-1, c3-1)
     & +      F750*coarse( c1-2, c2  , c3-2)
     & -      F675*coarse( c1-2, c2-1, c3  )
     & -      F675*coarse( c1  , c2-1, c3-2)
     & -      F450*coarse( c1-2, c2+1, c3-1)
     & -      F450*coarse( c1-1, c2+1, c3-2)
     & +      F270*coarse( c1-2, c2-1, c3-1)
     & +      F270*coarse( c1-1, c2-1, c3-2)
     & +      F125*coarse( c1-2, c2+1, c3-2)
     & -       F75*coarse( c1-2, c2-1, c3-2)
     & )
            f21 = f21 + 2
          end do
        end if
      end if
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Set cint(5) boundary
      if(c2f.lt.cint(5)) then
        c2 = cint(5)
        f21 = 2*c2
c
c Set cint(5)-cint(3) edge
        if(c3i.gt.cint(3)) then
          c3 = cint(3)
          f31 = 2*c3
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1  , c2  , c3+1)
     & -     F7560*coarse( c1  , c2-1, c3+1)
     & +     F4725*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1-1, c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3+2)
     & -     F2835*coarse( c1+1, c2  , c3  )
     & +     F2100*coarse( c1  , c2-1, c3+2)
     & -     F1890*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2-2, c3+1)
     & -     F1260*coarse( c1-1, c2-1, c3+1)
     & +     F1134*coarse( c1+1, c2  , c3+1)
     & +      F756*coarse( c1+1, c2-1, c3+1)
     & -      F675*coarse( c1-1, c2-2, c3  )
     & +      F525*coarse( c1-1, c2  , c3+2)
     & -      F450*coarse( c1  , c2-2, c3+2)
     & +      F405*coarse( c1+1, c2-2, c3  )
     & +      F350*coarse( c1-1, c2-1, c3+2)
     & -      F315*coarse( c1+1, c2  , c3+2)
     & +      F270*coarse( c1-1, c2-2, c3+1)
     & -      F210*coarse( c1+1, c2-1, c3+2)
     & -      F162*coarse( c1+1, c2-2, c3+1)
     & -       F75*coarse( c1-1, c2-2, c3+2)
     & +       F45*coarse( c1+1, c2-2, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1  , c2  , c3+1)
     & -     F7560*coarse( c1  , c2-1, c3+1)
     & +     F4725*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1+1, c2-1, c3  )
     & -     F2835*coarse( c1-1, c2  , c3  )
     & +     F2100*coarse( c1  , c2-1, c3+2)
     & -     F1890*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2-2, c3+1)
     & -     F1260*coarse( c1+1, c2-1, c3+1)
     & +     F1134*coarse( c1-1, c2  , c3+1)
     & +      F756*coarse( c1-1, c2-1, c3+1)
     & -      F675*coarse( c1+1, c2-2, c3  )
     & +      F525*coarse( c1+1, c2  , c3+2)
     & -      F450*coarse( c1  , c2-2, c3+2)
     & +      F405*coarse( c1-1, c2-2, c3  )
     & +      F350*coarse( c1+1, c2-1, c3+2)
     & -      F315*coarse( c1-1, c2  , c3+2)
     & +      F270*coarse( c1+1, c2-2, c3+1)
     & -      F210*coarse( c1-1, c2-1, c3+2)
     & -      F162*coarse( c1-1, c2-2, c3+1)
     & -       F75*coarse( c1+1, c2-2, c3+2)
     & +       F45*coarse( c1-1, c2-2, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2-1, c3  )
     & -    F24300*coarse( c1  , c2  , c3+1)
     & +    F10125*coarse( c1-1, c2  , c3  )
     & +     F9720*coarse( c1  , c2-1, c3+1)
     & -     F6075*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2-2, c3  )
     & +     F6750*coarse( c1  , c2  , c3+2)
     & -     F4050*coarse( c1-1, c2-1, c3  )
     & -     F4050*coarse( c1-1, c2  , c3+1)
     & -     F2700*coarse( c1  , c2-2, c3+1)
     & -     F2700*coarse( c1  , c2-1, c3+2)
     & +     F2430*coarse( c1+1, c2-1, c3  )
     & +     F2430*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1-1, c2-1, c3+1)
     & +     F1125*coarse( c1-1, c2-2, c3  )
     & +     F1125*coarse( c1-1, c2  , c3+2)
     & -      F972*coarse( c1+1, c2-1, c3+1)
     & +      F750*coarse( c1  , c2-2, c3+2)
     & -      F675*coarse( c1+1, c2-2, c3  )
     & -      F675*coarse( c1+1, c2  , c3+2)
     & -      F450*coarse( c1-1, c2-2, c3+1)
     & -      F450*coarse( c1-1, c2-1, c3+2)
     & +      F270*coarse( c1+1, c2-2, c3+1)
     & +      F270*coarse( c1+1, c2-1, c3+2)
     & +      F125*coarse( c1-1, c2-2, c3+2)
     & -       F75*coarse( c1+1, c2-2, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2-1, c3  )
     & -    F24300*coarse( c1  , c2  , c3+1)
     & +    F10125*coarse( c1+1, c2  , c3  )
     & +     F9720*coarse( c1  , c2-1, c3+1)
     & -     F6075*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2-2, c3  )
     & +     F6750*coarse( c1  , c2  , c3+2)
     & -     F4050*coarse( c1+1, c2-1, c3  )
     & -     F4050*coarse( c1+1, c2  , c3+1)
     & -     F2700*coarse( c1  , c2-2, c3+1)
     & -     F2700*coarse( c1  , c2-1, c3+2)
     & +     F2430*coarse( c1-1, c2-1, c3  )
     & +     F2430*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1+1, c2-1, c3+1)
     & +     F1125*coarse( c1+1, c2-2, c3  )
     & +     F1125*coarse( c1+1, c2  , c3+2)
     & -      F972*coarse( c1-1, c2-1, c3+1)
     & +      F750*coarse( c1  , c2-2, c3+2)
     & -      F675*coarse( c1-1, c2-2, c3  )
     & -      F675*coarse( c1-1, c2  , c3+2)
     & -      F450*coarse( c1+1, c2-2, c3+1)
     & -      F450*coarse( c1+1, c2-1, c3+2)
     & +      F270*coarse( c1-1, c2-2, c3+1)
     & +      F270*coarse( c1-1, c2-1, c3+2)
     & +      F125*coarse( c1+1, c2-2, c3+2)
     & -       F75*coarse( c1-1, c2-2, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1  , c2-1, c3  )
     & +     F8820*coarse( c1  , c2  , c3+1)
     & +     F5880*coarse( c1  , c2-1, c3+1)
     & +     F2205*coarse( c1-1, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3  )
     & -     F1890*coarse( c1  , c2  , c3+2)
     & +     F1470*coarse( c1-1, c2-1, c3  )
     & +     F1470*coarse( c1-1, c2  , c3+1)
     & -     F1323*coarse( c1+1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-2, c3+1)
     & -     F1260*coarse( c1  , c2-1, c3+2)
     & +      F980*coarse( c1-1, c2-1, c3+1)
     & -      F882*coarse( c1+1, c2-1, c3  )
     & -      F882*coarse( c1+1, c2  , c3+1)
     & -      F588*coarse( c1+1, c2-1, c3+1)
     & -      F315*coarse( c1-1, c2-2, c3  )
     & -      F315*coarse( c1-1, c2  , c3+2)
     & +      F270*coarse( c1  , c2-2, c3+2)
     & -      F210*coarse( c1-1, c2-2, c3+1)
     & -      F210*coarse( c1-1, c2-1, c3+2)
     & +      F189*coarse( c1+1, c2-2, c3  )
     & +      F189*coarse( c1+1, c2  , c3+2)
     & +      F126*coarse( c1+1, c2-2, c3+1)
     & +      F126*coarse( c1+1, c2-1, c3+2)
     & +       F45*coarse( c1-1, c2-2, c3+2)
     & -       F27*coarse( c1+1, c2-2, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1  , c2-1, c3  )
     & +     F5880*coarse( c1  , c2-1, c3+1)
     & +     F2205*coarse( c1+1, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3  )
     & -     F1890*coarse( c1  , c2  , c3+2)
     & +     F1470*coarse( c1+1, c2-1, c3  )
     & +     F1470*coarse( c1+1, c2  , c3+1)
     & -     F1323*coarse( c1-1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-2, c3+1)
     & -     F1260*coarse( c1  , c2-1, c3+2)
     & +      F980*coarse( c1+1, c2-1, c3+1)
     & -      F882*coarse( c1-1, c2-1, c3  )
     & -      F882*coarse( c1-1, c2  , c3+1)
     & -      F588*coarse( c1-1, c2-1, c3+1)
     & -      F315*coarse( c1+1, c2-2, c3  )
     & -      F315*coarse( c1+1, c2  , c3+2)
     & +      F270*coarse( c1  , c2-2, c3+2)
     & -      F210*coarse( c1+1, c2-2, c3+1)
     & -      F210*coarse( c1+1, c2-1, c3+2)
     & +      F189*coarse( c1-1, c2-2, c3  )
     & +      F189*coarse( c1-1, c2  , c3+2)
     & +      F126*coarse( c1-1, c2-2, c3+1)
     & +      F126*coarse( c1-1, c2-1, c3+2)
     & +       F45*coarse( c1+1, c2-2, c3+2)
     & -       F27*coarse( c1-1, c2-2, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1  , c2-1, c3  )
     & -     F7560*coarse( c1  , c2-1, c3+1)
     & +     F4725*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3+1)
     & -     F2835*coarse( c1+1, c2  , c3  )
     & +     F2100*coarse( c1  , c2-2, c3+1)
     & -     F1890*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2-1, c3+2)
     & -     F1260*coarse( c1-1, c2-1, c3+1)
     & +     F1134*coarse( c1+1, c2-1, c3  )
     & +      F756*coarse( c1+1, c2-1, c3+1)
     & -      F675*coarse( c1-1, c2  , c3+2)
     & +      F525*coarse( c1-1, c2-2, c3  )
     & -      F450*coarse( c1  , c2-2, c3+2)
     & +      F405*coarse( c1+1, c2  , c3+2)
     & +      F350*coarse( c1-1, c2-2, c3+1)
     & -      F315*coarse( c1+1, c2-2, c3  )
     & +      F270*coarse( c1-1, c2-1, c3+2)
     & -      F210*coarse( c1+1, c2-2, c3+1)
     & -      F162*coarse( c1+1, c2-1, c3+2)
     & -       F75*coarse( c1-1, c2-2, c3+2)
     & +       F45*coarse( c1+1, c2-2, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1  , c2-1, c3  )
     & -     F7560*coarse( c1  , c2-1, c3+1)
     & +     F4725*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3+2)
     & +     F3150*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3+1)
     & -     F2835*coarse( c1-1, c2  , c3  )
     & +     F2100*coarse( c1  , c2-2, c3+1)
     & -     F1890*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1  , c2-1, c3+2)
     & -     F1260*coarse( c1+1, c2-1, c3+1)
     & +     F1134*coarse( c1-1, c2-1, c3  )
     & +      F756*coarse( c1-1, c2-1, c3+1)
     & -      F675*coarse( c1+1, c2  , c3+2)
     & +      F525*coarse( c1+1, c2-2, c3  )
     & -      F450*coarse( c1  , c2-2, c3+2)
     & +      F405*coarse( c1-1, c2  , c3+2)
     & +      F350*coarse( c1+1, c2-2, c3+1)
     & -      F315*coarse( c1-1, c2-2, c3  )
     & +      F270*coarse( c1+1, c2-1, c3+2)
     & -      F210*coarse( c1-1, c2-2, c3+1)
     & -      F162*coarse( c1-1, c2-1, c3+2)
     & -       F75*coarse( c1+1, c2-2, c3+2)
     & +       F45*coarse( c1-1, c2-2, c3+2)
     & )
            f11 = f11 + 2
          end do
        end if
c
c Set cint(5) face
        f31 = f3i
        do c3 = c3i,c3f
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1  , c2-2, c3  )
     & +     F2100*coarse( c1  , c2-1, c3-1)
     & +     F2100*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1+1, c2-1, c3  )
     & -     F1260*coarse( c1  , c2-1, c3+1)
     & +      F525*coarse( c1-1, c2  , c3-1)
     & -      F450*coarse( c1  , c2-2, c3-1)
     & -      F450*coarse( c1-1, c2-2, c3  )
     & +      F350*coarse( c1-1, c2-1, c3-1)
     & -      F315*coarse( c1+1, c2  , c3-1)
     & -      F315*coarse( c1-1, c2  , c3+1)
     & +      F270*coarse( c1+1, c2-2, c3  )
     & +      F270*coarse( c1  , c2-2, c3+1)
     & -      F210*coarse( c1+1, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2-1, c3+1)
     & +      F189*coarse( c1+1, c2  , c3+1)
     & +      F126*coarse( c1+1, c2-1, c3+1)
     & -       F75*coarse( c1-1, c2-2, c3-1)
     & +       F45*coarse( c1+1, c2-2, c3-1)
     & +       F45*coarse( c1-1, c2-2, c3+1)
     & -       F27*coarse( c1+1, c2-2, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1+1, c2  , c3  )
     & -     F2700*coarse( c1  , c2-2, c3  )
     & +     F2100*coarse( c1  , c2-1, c3-1)
     & +     F2100*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3  )
     & -     F1890*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1-1, c2-1, c3  )
     & -     F1260*coarse( c1  , c2-1, c3+1)
     & +      F525*coarse( c1+1, c2  , c3-1)
     & -      F450*coarse( c1  , c2-2, c3-1)
     & -      F450*coarse( c1+1, c2-2, c3  )
     & +      F350*coarse( c1+1, c2-1, c3-1)
     & -      F315*coarse( c1-1, c2  , c3-1)
     & -      F315*coarse( c1+1, c2  , c3+1)
     & +      F270*coarse( c1-1, c2-2, c3  )
     & +      F270*coarse( c1  , c2-2, c3+1)
     & -      F210*coarse( c1-1, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2-1, c3+1)
     & +      F189*coarse( c1-1, c2  , c3+1)
     & +      F126*coarse( c1-1, c2-1, c3+1)
     & -       F75*coarse( c1+1, c2-2, c3-1)
     & +       F45*coarse( c1-1, c2-2, c3-1)
     & +       F45*coarse( c1+1, c2-2, c3+1)
     & -       F27*coarse( c1-1, c2-2, c3+1)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1-1, c2  , c3  )
     & +     F4500*coarse( c1  , c2-2, c3  )
     & -     F4050*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1  , c2-1, c3-1)
     & -     F2700*coarse( c1-1, c2-1, c3  )
     & +     F1620*coarse( c1+1, c2-1, c3  )
     & +     F1620*coarse( c1  , c2-1, c3+1)
     & +     F1125*coarse( c1-1, c2  , c3-1)
     & +      F750*coarse( c1  , c2-2, c3-1)
     & +      F750*coarse( c1-1, c2-2, c3  )
     & -      F675*coarse( c1+1, c2  , c3-1)
     & -      F675*coarse( c1-1, c2  , c3+1)
     & -      F450*coarse( c1-1, c2-1, c3-1)
     & -      F450*coarse( c1+1, c2-2, c3  )
     & -      F450*coarse( c1  , c2-2, c3+1)
     & +      F405*coarse( c1+1, c2  , c3+1)
     & +      F270*coarse( c1+1, c2-1, c3-1)
     & +      F270*coarse( c1-1, c2-1, c3+1)
     & -      F162*coarse( c1+1, c2-1, c3+1)
     & +      F125*coarse( c1-1, c2-2, c3-1)
     & -       F75*coarse( c1+1, c2-2, c3-1)
     & -       F75*coarse( c1-1, c2-2, c3+1)
     & +       F45*coarse( c1+1, c2-2, c3+1)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1+1, c2  , c3  )
     & +     F4500*coarse( c1  , c2-2, c3  )
     & -     F4050*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1  , c2-1, c3-1)
     & -     F2700*coarse( c1+1, c2-1, c3  )
     & +     F1620*coarse( c1-1, c2-1, c3  )
     & +     F1620*coarse( c1  , c2-1, c3+1)
     & +     F1125*coarse( c1+1, c2  , c3-1)
     & +      F750*coarse( c1  , c2-2, c3-1)
     & +      F750*coarse( c1+1, c2-2, c3  )
     & -      F675*coarse( c1-1, c2  , c3-1)
     & -      F675*coarse( c1+1, c2  , c3+1)
     & -      F450*coarse( c1+1, c2-1, c3-1)
     & -      F450*coarse( c1-1, c2-2, c3  )
     & -      F450*coarse( c1  , c2-2, c3+1)
     & +      F405*coarse( c1-1, c2  , c3+1)
     & +      F270*coarse( c1-1, c2-1, c3-1)
     & +      F270*coarse( c1+1, c2-1, c3+1)
     & -      F162*coarse( c1-1, c2-1, c3+1)
     & +      F125*coarse( c1+1, c2-2, c3-1)
     & -       F75*coarse( c1-1, c2-2, c3-1)
     & -       F75*coarse( c1+1, c2-2, c3+1)
     & +       F45*coarse( c1-1, c2-2, c3+1)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1-1, c2  , c3  )
     & +     F3150*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1  , c2-2, c3  )
     & +     F2100*coarse( c1-1, c2-1, c3  )
     & +     F2100*coarse( c1  , c2-1, c3+1)
     & -     F1890*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1+1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-1, c3-1)
     & -     F1260*coarse( c1+1, c2-1, c3  )
     & +      F525*coarse( c1-1, c2  , c3+1)
     & -      F450*coarse( c1-1, c2-2, c3  )
     & -      F450*coarse( c1  , c2-2, c3+1)
     & +      F350*coarse( c1-1, c2-1, c3+1)
     & -      F315*coarse( c1-1, c2  , c3-1)
     & -      F315*coarse( c1+1, c2  , c3+1)
     & +      F270*coarse( c1  , c2-2, c3-1)
     & +      F270*coarse( c1+1, c2-2, c3  )
     & -      F210*coarse( c1-1, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2-1, c3+1)
     & +      F189*coarse( c1+1, c2  , c3-1)
     & +      F126*coarse( c1+1, c2-1, c3-1)
     & -       F75*coarse( c1-1, c2-2, c3+1)
     & +       F45*coarse( c1-1, c2-2, c3-1)
     & +       F45*coarse( c1+1, c2-2, c3+1)
     & -       F27*coarse( c1+1, c2-2, c3-1)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1+1, c2  , c3  )
     & +     F3150*coarse( c1  , c2  , c3+1)
     & -     F2700*coarse( c1  , c2-2, c3  )
     & +     F2100*coarse( c1+1, c2-1, c3  )
     & +     F2100*coarse( c1  , c2-1, c3+1)
     & -     F1890*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1-1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-1, c3-1)
     & -     F1260*coarse( c1-1, c2-1, c3  )
     & +      F525*coarse( c1+1, c2  , c3+1)
     & -      F450*coarse( c1  , c2-2, c3+1)
     & -      F450*coarse( c1+1, c2-2, c3  )
     & +      F350*coarse( c1+1, c2-1, c3+1)
     & -      F315*coarse( c1+1, c2  , c3-1)
     & -      F315*coarse( c1-1, c2  , c3+1)
     & +      F270*coarse( c1  , c2-2, c3-1)
     & +      F270*coarse( c1-1, c2-2, c3  )
     & -      F210*coarse( c1+1, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2-1, c3+1)
     & +      F189*coarse( c1-1, c2  , c3-1)
     & +      F126*coarse( c1-1, c2-1, c3-1)
     & -       F75*coarse( c1+1, c2-2, c3+1)
     & +       F45*coarse( c1+1, c2-2, c3-1)
     & +       F45*coarse( c1-1, c2-2, c3+1)
     & -       F27*coarse( c1-1, c2-2, c3-1)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3+1)
     & +     F4500*coarse( c1  , c2-2, c3  )
     & -     F4050*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1+1, c2  , c3  )
     & -     F2700*coarse( c1-1, c2-1, c3  )
     & -     F2700*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1+1, c2-1, c3  )
     & +     F1125*coarse( c1-1, c2  , c3+1)
     & +      F750*coarse( c1-1, c2-2, c3  )
     & +      F750*coarse( c1  , c2-2, c3+1)
     & -      F675*coarse( c1-1, c2  , c3-1)
     & -      F675*coarse( c1+1, c2  , c3+1)
     & -      F450*coarse( c1  , c2-2, c3-1)
     & -      F450*coarse( c1+1, c2-2, c3  )
     & -      F450*coarse( c1-1, c2-1, c3+1)
     & +      F405*coarse( c1+1, c2  , c3-1)
     & +      F270*coarse( c1-1, c2-1, c3-1)
     & +      F270*coarse( c1+1, c2-1, c3+1)
     & -      F162*coarse( c1+1, c2-1, c3-1)
     & +      F125*coarse( c1-1, c2-2, c3+1)
     & -       F75*coarse( c1-1, c2-2, c3-1)
     & -       F75*coarse( c1+1, c2-2, c3+1)
     & +       F45*coarse( c1+1, c2-2, c3-1)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2  , c3+1)
     & +     F4500*coarse( c1  , c2-2, c3  )
     & -     F4050*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1+1, c2-1, c3  )
     & -     F2700*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1-1, c2-1, c3  )
     & +     F1125*coarse( c1+1, c2  , c3+1)
     & +      F750*coarse( c1+1, c2-2, c3  )
     & +      F750*coarse( c1  , c2-2, c3+1)
     & -      F675*coarse( c1+1, c2  , c3-1)
     & -      F675*coarse( c1-1, c2  , c3+1)
     & -      F450*coarse( c1  , c2-2, c3-1)
     & -      F450*coarse( c1-1, c2-2, c3  )
     & -      F450*coarse( c1+1, c2-1, c3+1)
     & +      F405*coarse( c1-1, c2  , c3-1)
     & +      F270*coarse( c1+1, c2-1, c3-1)
     & +      F270*coarse( c1-1, c2-1, c3+1)
     & -      F162*coarse( c1-1, c2-1, c3-1)
     & +      F125*coarse( c1+1, c2-2, c3+1)
     & -       F75*coarse( c1+1, c2-2, c3-1)
     & -       F75*coarse( c1-1, c2-2, c3+1)
     & +       F45*coarse( c1-1, c2-2, c3-1)
     & )
            f11 = f11 + 2
          end do
          f31 = f31 + 2
        end do
c
c Set cint(5)-cint(6) edge
        if(c3f.lt.cint(6)) then
          c3 = cint(6)
          f31 = 2*c3
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1  , c2-1, c3  )
     & +     F8820*coarse( c1  , c2  , c3-1)
     & +     F5880*coarse( c1  , c2-1, c3-1)
     & +     F2205*coarse( c1-1, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3  )
     & -     F1890*coarse( c1  , c2  , c3-2)
     & +     F1470*coarse( c1-1, c2-1, c3  )
     & +     F1470*coarse( c1-1, c2  , c3-1)
     & -     F1323*coarse( c1+1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-2, c3-1)
     & -     F1260*coarse( c1  , c2-1, c3-2)
     & +      F980*coarse( c1-1, c2-1, c3-1)
     & -      F882*coarse( c1+1, c2-1, c3  )
     & -      F882*coarse( c1+1, c2  , c3-1)
     & -      F588*coarse( c1+1, c2-1, c3-1)
     & -      F315*coarse( c1-1, c2-2, c3  )
     & -      F315*coarse( c1-1, c2  , c3-2)
     & +      F270*coarse( c1  , c2-2, c3-2)
     & -      F210*coarse( c1-1, c2-2, c3-1)
     & -      F210*coarse( c1-1, c2-1, c3-2)
     & +      F189*coarse( c1+1, c2-2, c3  )
     & +      F189*coarse( c1+1, c2  , c3-2)
     & +      F126*coarse( c1+1, c2-2, c3-1)
     & +      F126*coarse( c1+1, c2-1, c3-2)
     & +       F45*coarse( c1-1, c2-2, c3-2)
     & -       F27*coarse( c1+1, c2-2, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1  , c2-1, c3  )
     & +     F5880*coarse( c1  , c2-1, c3-1)
     & +     F2205*coarse( c1+1, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3  )
     & -     F1890*coarse( c1  , c2  , c3-2)
     & +     F1470*coarse( c1+1, c2-1, c3  )
     & +     F1470*coarse( c1+1, c2  , c3-1)
     & -     F1323*coarse( c1-1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-2, c3-1)
     & -     F1260*coarse( c1  , c2-1, c3-2)
     & +      F980*coarse( c1+1, c2-1, c3-1)
     & -      F882*coarse( c1-1, c2-1, c3  )
     & -      F882*coarse( c1-1, c2  , c3-1)
     & -      F588*coarse( c1-1, c2-1, c3-1)
     & -      F315*coarse( c1+1, c2-2, c3  )
     & -      F315*coarse( c1+1, c2  , c3-2)
     & +      F270*coarse( c1  , c2-2, c3-2)
     & -      F210*coarse( c1+1, c2-2, c3-1)
     & -      F210*coarse( c1+1, c2-1, c3-2)
     & +      F189*coarse( c1-1, c2-2, c3  )
     & +      F189*coarse( c1-1, c2  , c3-2)
     & +      F126*coarse( c1-1, c2-2, c3-1)
     & +      F126*coarse( c1-1, c2-1, c3-2)
     & +       F45*coarse( c1+1, c2-2, c3-2)
     & -       F27*coarse( c1-1, c2-2, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1  , c2-1, c3  )
     & -     F7560*coarse( c1  , c2-1, c3-1)
     & +     F4725*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3-1)
     & -     F2835*coarse( c1+1, c2  , c3  )
     & +     F2100*coarse( c1  , c2-2, c3-1)
     & -     F1890*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2-1, c3-2)
     & -     F1260*coarse( c1-1, c2-1, c3-1)
     & +     F1134*coarse( c1+1, c2-1, c3  )
     & +      F756*coarse( c1+1, c2-1, c3-1)
     & -      F675*coarse( c1-1, c2  , c3-2)
     & +      F525*coarse( c1-1, c2-2, c3  )
     & -      F450*coarse( c1  , c2-2, c3-2)
     & +      F405*coarse( c1+1, c2  , c3-2)
     & +      F350*coarse( c1-1, c2-2, c3-1)
     & -      F315*coarse( c1+1, c2-2, c3  )
     & +      F270*coarse( c1-1, c2-1, c3-2)
     & -      F210*coarse( c1+1, c2-2, c3-1)
     & -      F162*coarse( c1+1, c2-1, c3-2)
     & -       F75*coarse( c1-1, c2-2, c3-2)
     & +       F45*coarse( c1+1, c2-2, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1  , c2-1, c3  )
     & -     F7560*coarse( c1  , c2-1, c3-1)
     & +     F4725*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1+1, c2  , c3-1)
     & -     F2835*coarse( c1-1, c2  , c3  )
     & +     F2100*coarse( c1  , c2-2, c3-1)
     & -     F1890*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2-1, c3-2)
     & -     F1260*coarse( c1+1, c2-1, c3-1)
     & +     F1134*coarse( c1-1, c2-1, c3  )
     & +      F756*coarse( c1-1, c2-1, c3-1)
     & -      F675*coarse( c1+1, c2  , c3-2)
     & +      F525*coarse( c1+1, c2-2, c3  )
     & -      F450*coarse( c1  , c2-2, c3-2)
     & +      F405*coarse( c1-1, c2  , c3-2)
     & +      F350*coarse( c1+1, c2-2, c3-1)
     & -      F315*coarse( c1-1, c2-2, c3  )
     & +      F270*coarse( c1+1, c2-1, c3-2)
     & -      F210*coarse( c1-1, c2-2, c3-1)
     & -      F162*coarse( c1-1, c2-1, c3-2)
     & -       F75*coarse( c1+1, c2-2, c3-2)
     & +       F45*coarse( c1-1, c2-2, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1  , c2  , c3-1)
     & -     F7560*coarse( c1  , c2-1, c3-1)
     & +     F4725*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1-1, c2-1, c3  )
     & +     F3150*coarse( c1  , c2  , c3-2)
     & -     F2835*coarse( c1+1, c2  , c3  )
     & +     F2100*coarse( c1  , c2-1, c3-2)
     & -     F1890*coarse( c1+1, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2-2, c3-1)
     & -     F1260*coarse( c1-1, c2-1, c3-1)
     & +     F1134*coarse( c1+1, c2  , c3-1)
     & +      F756*coarse( c1+1, c2-1, c3-1)
     & -      F675*coarse( c1-1, c2-2, c3  )
     & +      F525*coarse( c1-1, c2  , c3-2)
     & -      F450*coarse( c1  , c2-2, c3-2)
     & +      F405*coarse( c1+1, c2-2, c3  )
     & +      F350*coarse( c1-1, c2-1, c3-2)
     & -      F315*coarse( c1+1, c2  , c3-2)
     & +      F270*coarse( c1-1, c2-2, c3-1)
     & -      F210*coarse( c1+1, c2-1, c3-2)
     & -      F162*coarse( c1+1, c2-2, c3-1)
     & -       F75*coarse( c1-1, c2-2, c3-2)
     & +       F45*coarse( c1+1, c2-2, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1  , c2  , c3-1)
     & -     F7560*coarse( c1  , c2-1, c3-1)
     & +     F4725*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1  , c2  , c3-2)
     & +     F3150*coarse( c1+1, c2-1, c3  )
     & -     F2835*coarse( c1-1, c2  , c3  )
     & +     F2100*coarse( c1  , c2-1, c3-2)
     & -     F1890*coarse( c1-1, c2-1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2-2, c3-1)
     & -     F1260*coarse( c1+1, c2-1, c3-1)
     & +     F1134*coarse( c1-1, c2  , c3-1)
     & +      F756*coarse( c1-1, c2-1, c3-1)
     & -      F675*coarse( c1+1, c2-2, c3  )
     & +      F525*coarse( c1+1, c2  , c3-2)
     & -      F450*coarse( c1  , c2-2, c3-2)
     & +      F405*coarse( c1-1, c2-2, c3  )
     & +      F350*coarse( c1+1, c2-1, c3-2)
     & -      F315*coarse( c1-1, c2  , c3-2)
     & +      F270*coarse( c1+1, c2-2, c3-1)
     & -      F210*coarse( c1-1, c2-1, c3-2)
     & -      F162*coarse( c1-1, c2-2, c3-1)
     & -       F75*coarse( c1+1, c2-2, c3-2)
     & +       F45*coarse( c1-1, c2-2, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2-1, c3  )
     & -    F24300*coarse( c1  , c2  , c3-1)
     & +    F10125*coarse( c1-1, c2  , c3  )
     & +     F9720*coarse( c1  , c2-1, c3-1)
     & -     F6075*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2-2, c3  )
     & +     F6750*coarse( c1  , c2  , c3-2)
     & -     F4050*coarse( c1-1, c2-1, c3  )
     & -     F4050*coarse( c1-1, c2  , c3-1)
     & -     F2700*coarse( c1  , c2-2, c3-1)
     & -     F2700*coarse( c1  , c2-1, c3-2)
     & +     F2430*coarse( c1+1, c2-1, c3  )
     & +     F2430*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1-1, c2-1, c3-1)
     & +     F1125*coarse( c1-1, c2-2, c3  )
     & +     F1125*coarse( c1-1, c2  , c3-2)
     & -      F972*coarse( c1+1, c2-1, c3-1)
     & +      F750*coarse( c1  , c2-2, c3-2)
     & -      F675*coarse( c1+1, c2-2, c3  )
     & -      F675*coarse( c1+1, c2  , c3-2)
     & -      F450*coarse( c1-1, c2-2, c3-1)
     & -      F450*coarse( c1-1, c2-1, c3-2)
     & +      F270*coarse( c1+1, c2-2, c3-1)
     & +      F270*coarse( c1+1, c2-1, c3-2)
     & +      F125*coarse( c1-1, c2-2, c3-2)
     & -       F75*coarse( c1+1, c2-2, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1  , c2-1, c3  )
     & -    F24300*coarse( c1  , c2  , c3-1)
     & +    F10125*coarse( c1+1, c2  , c3  )
     & +     F9720*coarse( c1  , c2-1, c3-1)
     & -     F6075*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2-2, c3  )
     & +     F6750*coarse( c1  , c2  , c3-2)
     & -     F4050*coarse( c1+1, c2-1, c3  )
     & -     F4050*coarse( c1+1, c2  , c3-1)
     & -     F2700*coarse( c1  , c2-2, c3-1)
     & -     F2700*coarse( c1  , c2-1, c3-2)
     & +     F2430*coarse( c1-1, c2-1, c3  )
     & +     F2430*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1+1, c2-1, c3-1)
     & +     F1125*coarse( c1+1, c2-2, c3  )
     & +     F1125*coarse( c1+1, c2  , c3-2)
     & -      F972*coarse( c1-1, c2-1, c3-1)
     & +      F750*coarse( c1  , c2-2, c3-2)
     & -      F675*coarse( c1-1, c2-2, c3  )
     & -      F675*coarse( c1-1, c2  , c3-2)
     & -      F450*coarse( c1+1, c2-2, c3-1)
     & -      F450*coarse( c1+1, c2-1, c3-2)
     & +      F270*coarse( c1-1, c2-2, c3-1)
     & +      F270*coarse( c1-1, c2-1, c3-2)
     & +      F125*coarse( c1+1, c2-2, c3-2)
     & -       F75*coarse( c1-1, c2-2, c3-2)
     & )
            f11 = f11 + 2
          end do
        end if
c
c Set cint(5)-cint(4) corner
        if(c1f.lt.cint(4)) then
          c1 = cint(4)
          f11 = 2*c1
c
c Set cint(5)-cint(4)-cint(3) corner
          if(c3i.gt.cint(3)) then
            c3 = cint(3)
            f31 = 2*c3
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1-1, c2  , c3  )
     & +    F13230*coarse( c1  , c2-1, c3  )
     & +     F8820*coarse( c1-1, c2-1, c3  )
     & -     F7938*coarse( c1  , c2  , c3+1)
     & -     F5292*coarse( c1-1, c2  , c3+1)
     & -     F5292*coarse( c1  , c2-1, c3+1)
     & -     F3528*coarse( c1-1, c2-1, c3+1)
     & -     F2835*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2-2, c3  )
     & +     F2205*coarse( c1  , c2  , c3+2)
     & -     F1890*coarse( c1-2, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2-2, c3  )
     & +     F1470*coarse( c1-1, c2  , c3+2)
     & +     F1470*coarse( c1  , c2-1, c3+2)
     & +     F1134*coarse( c1-2, c2  , c3+1)
     & +     F1134*coarse( c1  , c2-2, c3+1)
     & +      F980*coarse( c1-1, c2-1, c3+2)
     & +      F756*coarse( c1-2, c2-1, c3+1)
     & +      F756*coarse( c1-1, c2-2, c3+1)
     & +      F405*coarse( c1-2, c2-2, c3  )
     & -      F315*coarse( c1-2, c2  , c3+2)
     & -      F315*coarse( c1  , c2-2, c3+2)
     & -      F210*coarse( c1-2, c2-1, c3+2)
     & -      F210*coarse( c1-1, c2-2, c3+2)
     & -      F162*coarse( c1-2, c2-2, c3+1)
     & +       F45*coarse( c1-2, c2-2, c3+2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2-1, c3  )
     & -    F17010*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1-1, c2-1, c3  )
     & -    F11340*coarse( c1  , c2-1, c3+1)
     & +     F6804*coarse( c1-1, c2  , c3+1)
     & -     F6075*coarse( c1  , c2-2, c3  )
     & +     F4725*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2  , c3+2)
     & +     F4536*coarse( c1-1, c2-1, c3+1)
     & +     F3150*coarse( c1-2, c2-1, c3  )
     & +     F3150*coarse( c1  , c2-1, c3+2)
     & +     F2430*coarse( c1-1, c2-2, c3  )
     & +     F2430*coarse( c1  , c2-2, c3+1)
     & -     F1890*coarse( c1-2, c2  , c3+1)
     & -     F1890*coarse( c1-1, c2  , c3+2)
     & -     F1260*coarse( c1-2, c2-1, c3+1)
     & -     F1260*coarse( c1-1, c2-1, c3+2)
     & -      F972*coarse( c1-1, c2-2, c3+1)
     & -      F675*coarse( c1-2, c2-2, c3  )
     & -      F675*coarse( c1  , c2-2, c3+2)
     & +      F525*coarse( c1-2, c2  , c3+2)
     & +      F350*coarse( c1-2, c2-1, c3+2)
     & +      F270*coarse( c1-2, c2-2, c3+1)
     & +      F270*coarse( c1-1, c2-2, c3+2)
     & -       F75*coarse( c1-2, c2-2, c3+2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2-1, c3  )
     & -    F17010*coarse( c1  , c2  , c3+1)
     & -    F11340*coarse( c1-1, c2-1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3+1)
     & +     F6804*coarse( c1  , c2-1, c3+1)
     & -     F6075*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2-2, c3  )
     & +     F4725*coarse( c1  , c2  , c3+2)
     & +     F4536*coarse( c1-1, c2-1, c3+1)
     & +     F3150*coarse( c1-1, c2-2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3+2)
     & +     F2430*coarse( c1-2, c2-1, c3  )
     & +     F2430*coarse( c1-2, c2  , c3+1)
     & -     F1890*coarse( c1  , c2-2, c3+1)
     & -     F1890*coarse( c1  , c2-1, c3+2)
     & -     F1260*coarse( c1-1, c2-2, c3+1)
     & -     F1260*coarse( c1-1, c2-1, c3+2)
     & -      F972*coarse( c1-2, c2-1, c3+1)
     & -      F675*coarse( c1-2, c2-2, c3  )
     & -      F675*coarse( c1-2, c2  , c3+2)
     & +      F525*coarse( c1  , c2-2, c3+2)
     & +      F350*coarse( c1-1, c2-2, c3+2)
     & +      F270*coarse( c1-2, c2-2, c3+1)
     & +      F270*coarse( c1-2, c2-1, c3+2)
     & -       F75*coarse( c1-2, c2-2, c3+2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F91125*coarse( c1  , c2  , c3  )
     & -    F36450*coarse( c1-1, c2  , c3  )
     & -    F36450*coarse( c1  , c2-1, c3  )
     & -    F36450*coarse( c1  , c2  , c3+1)
     & +    F14580*coarse( c1-1, c2-1, c3  )
     & +    F14580*coarse( c1-1, c2  , c3+1)
     & +    F14580*coarse( c1  , c2-1, c3+1)
     & +    F10125*coarse( c1-2, c2  , c3  )
     & +    F10125*coarse( c1  , c2-2, c3  )
     & +    F10125*coarse( c1  , c2  , c3+2)
     & -     F5832*coarse( c1-1, c2-1, c3+1)
     & -     F4050*coarse( c1-2, c2-1, c3  )
     & -     F4050*coarse( c1-1, c2-2, c3  )
     & -     F4050*coarse( c1-2, c2  , c3+1)
     & -     F4050*coarse( c1  , c2-2, c3+1)
     & -     F4050*coarse( c1-1, c2  , c3+2)
     & -     F4050*coarse( c1  , c2-1, c3+2)
     & +     F1620*coarse( c1-2, c2-1, c3+1)
     & +     F1620*coarse( c1-1, c2-2, c3+1)
     & +     F1620*coarse( c1-1, c2-1, c3+2)
     & +     F1125*coarse( c1-2, c2-2, c3  )
     & +     F1125*coarse( c1-2, c2  , c3+2)
     & +     F1125*coarse( c1  , c2-2, c3+2)
     & -      F450*coarse( c1-2, c2-2, c3+1)
     & -      F450*coarse( c1-2, c2-1, c3+2)
     & -      F450*coarse( c1-1, c2-2, c3+2)
     & +      F125*coarse( c1-2, c2-2, c3+2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &       F9261*coarse( c1  , c2  , c3  )
     & +     F6174*coarse( c1-1, c2  , c3  )
     & +     F6174*coarse( c1  , c2-1, c3  )
     & +     F6174*coarse( c1  , c2  , c3+1)
     & +     F4116*coarse( c1-1, c2-1, c3  )
     & +     F4116*coarse( c1-1, c2  , c3+1)
     & +     F4116*coarse( c1  , c2-1, c3+1)
     & +     F2744*coarse( c1-1, c2-1, c3+1)
     & -     F1323*coarse( c1-2, c2  , c3  )
     & -     F1323*coarse( c1  , c2-2, c3  )
     & -     F1323*coarse( c1  , c2  , c3+2)
     & -      F882*coarse( c1-2, c2-1, c3  )
     & -      F882*coarse( c1-1, c2-2, c3  )
     & -      F882*coarse( c1-2, c2  , c3+1)
     & -      F882*coarse( c1  , c2-2, c3+1)
     & -      F882*coarse( c1-1, c2  , c3+2)
     & -      F882*coarse( c1  , c2-1, c3+2)
     & -      F588*coarse( c1-2, c2-1, c3+1)
     & -      F588*coarse( c1-1, c2-2, c3+1)
     & -      F588*coarse( c1-1, c2-1, c3+2)
     & +      F189*coarse( c1-2, c2-2, c3  )
     & +      F189*coarse( c1-2, c2  , c3+2)
     & +      F189*coarse( c1  , c2-2, c3+2)
     & +      F126*coarse( c1-2, c2-2, c3+1)
     & +      F126*coarse( c1-2, c2-1, c3+2)
     & +      F126*coarse( c1-1, c2-2, c3+2)
     & -       F27*coarse( c1-2, c2-2, c3+2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1  , c2-1, c3  )
     & +    F13230*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1  , c2-1, c3+1)
     & -     F7938*coarse( c1-1, c2  , c3  )
     & -     F5292*coarse( c1-1, c2-1, c3  )
     & -     F5292*coarse( c1-1, c2  , c3+1)
     & -     F3528*coarse( c1-1, c2-1, c3+1)
     & -     F2835*coarse( c1  , c2-2, c3  )
     & -     F2835*coarse( c1  , c2  , c3+2)
     & +     F2205*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3+1)
     & -     F1890*coarse( c1  , c2-1, c3+2)
     & +     F1470*coarse( c1-2, c2-1, c3  )
     & +     F1470*coarse( c1-2, c2  , c3+1)
     & +     F1134*coarse( c1-1, c2-2, c3  )
     & +     F1134*coarse( c1-1, c2  , c3+2)
     & +      F980*coarse( c1-2, c2-1, c3+1)
     & +      F756*coarse( c1-1, c2-2, c3+1)
     & +      F756*coarse( c1-1, c2-1, c3+2)
     & +      F405*coarse( c1  , c2-2, c3+2)
     & -      F315*coarse( c1-2, c2-2, c3  )
     & -      F315*coarse( c1-2, c2  , c3+2)
     & -      F210*coarse( c1-2, c2-2, c3+1)
     & -      F210*coarse( c1-2, c2-1, c3+2)
     & -      F162*coarse( c1-1, c2-2, c3+2)
     & +       F45*coarse( c1-2, c2-2, c3+2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1-1, c2  , c3  )
     & +    F13230*coarse( c1  , c2  , c3+1)
     & +     F8820*coarse( c1-1, c2  , c3+1)
     & -     F7938*coarse( c1  , c2-1, c3  )
     & -     F5292*coarse( c1-1, c2-1, c3  )
     & -     F5292*coarse( c1  , c2-1, c3+1)
     & -     F3528*coarse( c1-1, c2-1, c3+1)
     & -     F2835*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3+2)
     & +     F2205*coarse( c1  , c2-2, c3  )
     & -     F1890*coarse( c1-2, c2  , c3+1)
     & -     F1890*coarse( c1-1, c2  , c3+2)
     & +     F1470*coarse( c1-1, c2-2, c3  )
     & +     F1470*coarse( c1  , c2-2, c3+1)
     & +     F1134*coarse( c1-2, c2-1, c3  )
     & +     F1134*coarse( c1  , c2-1, c3+2)
     & +      F980*coarse( c1-1, c2-2, c3+1)
     & +      F756*coarse( c1-2, c2-1, c3+1)
     & +      F756*coarse( c1-1, c2-1, c3+2)
     & +      F405*coarse( c1-2, c2  , c3+2)
     & -      F315*coarse( c1-2, c2-2, c3  )
     & -      F315*coarse( c1  , c2-2, c3+2)
     & -      F210*coarse( c1-2, c2-2, c3+1)
     & -      F210*coarse( c1-1, c2-2, c3+2)
     & -      F162*coarse( c1-2, c2-1, c3+2)
     & +       F45*coarse( c1-2, c2-2, c3+2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2  , c3+1)
     & -    F17010*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3+1)
     & -    F11340*coarse( c1  , c2-1, c3+1)
     & +     F6804*coarse( c1-1, c2-1, c3  )
     & -     F6075*coarse( c1  , c2  , c3+2)
     & +     F4725*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2-2, c3  )
     & +     F4536*coarse( c1-1, c2-1, c3+1)
     & +     F3150*coarse( c1-2, c2  , c3+1)
     & +     F3150*coarse( c1  , c2-2, c3+1)
     & +     F2430*coarse( c1-1, c2  , c3+2)
     & +     F2430*coarse( c1  , c2-1, c3+2)
     & -     F1890*coarse( c1-2, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2-2, c3  )
     & -     F1260*coarse( c1-2, c2-1, c3+1)
     & -     F1260*coarse( c1-1, c2-2, c3+1)
     & -      F972*coarse( c1-1, c2-1, c3+2)
     & -      F675*coarse( c1-2, c2  , c3+2)
     & -      F675*coarse( c1  , c2-2, c3+2)
     & +      F525*coarse( c1-2, c2-2, c3  )
     & +      F350*coarse( c1-2, c2-2, c3+1)
     & +      F270*coarse( c1-2, c2-1, c3+2)
     & +      F270*coarse( c1-1, c2-2, c3+2)
     & -       F75*coarse( c1-2, c2-2, c3+2)
     & )
          end if
c
c Set cint(5)-cint(4) edge
          f31 = f3i
          do c3 = c3i,c3f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     & +    F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1-1, c2  , c3  )
     & +     F8820*coarse( c1  , c2-1, c3  )
     & +     F5880*coarse( c1-1, c2-1, c3  )
     & +     F2205*coarse( c1  , c2  , c3-1)
     & -     F1890*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3  )
     & +     F1470*coarse( c1-1, c2  , c3-1)
     & +     F1470*coarse( c1  , c2-1, c3-1)
     & -     F1323*coarse( c1  , c2  , c3+1)
     & -     F1260*coarse( c1-2, c2-1, c3  )
     & -     F1260*coarse( c1-1, c2-2, c3  )
     & +      F980*coarse( c1-1, c2-1, c3-1)
     & -      F882*coarse( c1-1, c2  , c3+1)
     & -      F882*coarse( c1  , c2-1, c3+1)
     & -      F588*coarse( c1-1, c2-1, c3+1)
     & -      F315*coarse( c1-2, c2  , c3-1)
     & -      F315*coarse( c1  , c2-2, c3-1)
     & +      F270*coarse( c1-2, c2-2, c3  )
     & -      F210*coarse( c1-2, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2-2, c3-1)
     & +      F189*coarse( c1-2, c2  , c3+1)
     & +      F189*coarse( c1  , c2-2, c3+1)
     & +      F126*coarse( c1-2, c2-1, c3+1)
     & +      F126*coarse( c1-1, c2-2, c3+1)
     & +       F45*coarse( c1-2, c2-2, c3-1)
     & -       F27*coarse( c1-2, c2-2, c3+1)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3  )
     & -     F7560*coarse( c1-1, c2-1, c3  )
     & +     F4725*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1  , c2-1, c3-1)
     & +     F3150*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3+1)
     & +     F2100*coarse( c1-2, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3-1)
     & -     F1890*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1-1, c2-2, c3  )
     & -     F1260*coarse( c1-1, c2-1, c3-1)
     & +     F1134*coarse( c1-1, c2  , c3+1)
     & +      F756*coarse( c1-1, c2-1, c3+1)
     & -      F675*coarse( c1  , c2-2, c3-1)
     & +      F525*coarse( c1-2, c2  , c3-1)
     & -      F450*coarse( c1-2, c2-2, c3  )
     & +      F405*coarse( c1  , c2-2, c3+1)
     & +      F350*coarse( c1-2, c2-1, c3-1)
     & -      F315*coarse( c1-2, c2  , c3+1)
     & +      F270*coarse( c1-1, c2-2, c3-1)
     & -      F210*coarse( c1-2, c2-1, c3+1)
     & -      F162*coarse( c1-1, c2-2, c3+1)
     & -       F75*coarse( c1-2, c2-2, c3-1)
     & +       F45*coarse( c1-2, c2-2, c3+1)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1-1, c2  , c3  )
     & -    F11340*coarse( c1  , c2-1, c3  )
     & -     F7560*coarse( c1-1, c2-1, c3  )
     & +     F4725*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1-1, c2  , c3-1)
     & +     F3150*coarse( c1  , c2-2, c3  )
     & -     F2835*coarse( c1  , c2  , c3+1)
     & +     F2100*coarse( c1-1, c2-2, c3  )
     & -     F1890*coarse( c1  , c2-1, c3-1)
     & -     F1890*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1-2, c2-1, c3  )
     & -     F1260*coarse( c1-1, c2-1, c3-1)
     & +     F1134*coarse( c1  , c2-1, c3+1)
     & +      F756*coarse( c1-1, c2-1, c3+1)
     & -      F675*coarse( c1-2, c2  , c3-1)
     & +      F525*coarse( c1  , c2-2, c3-1)
     & -      F450*coarse( c1-2, c2-2, c3  )
     & +      F405*coarse( c1-2, c2  , c3+1)
     & +      F350*coarse( c1-1, c2-2, c3-1)
     & -      F315*coarse( c1  , c2-2, c3+1)
     & +      F270*coarse( c1-2, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2-2, c3+1)
     & -      F162*coarse( c1-2, c2-1, c3+1)
     & -       F75*coarse( c1-2, c2-2, c3-1)
     & +       F45*coarse( c1-2, c2-2, c3+1)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     & +    F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1-1, c2  , c3  )
     & -    F24300*coarse( c1  , c2-1, c3  )
     & +    F10125*coarse( c1  , c2  , c3-1)
     & +     F9720*coarse( c1-1, c2-1, c3  )
     & +     F6750*coarse( c1-2, c2  , c3  )
     & +     F6750*coarse( c1  , c2-2, c3  )
     & -     F6075*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1-1, c2  , c3-1)
     & -     F4050*coarse( c1  , c2-1, c3-1)
     & -     F2700*coarse( c1-2, c2-1, c3  )
     & -     F2700*coarse( c1-1, c2-2, c3  )
     & +     F2430*coarse( c1-1, c2  , c3+1)
     & +     F2430*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1-1, c2-1, c3-1)
     & +     F1125*coarse( c1-2, c2  , c3-1)
     & +     F1125*coarse( c1  , c2-2, c3-1)
     & -      F972*coarse( c1-1, c2-1, c3+1)
     & +      F750*coarse( c1-2, c2-2, c3  )
     & -      F675*coarse( c1-2, c2  , c3+1)
     & -      F675*coarse( c1  , c2-2, c3+1)
     & -      F450*coarse( c1-2, c2-1, c3-1)
     & -      F450*coarse( c1-1, c2-2, c3-1)
     & +      F270*coarse( c1-2, c2-1, c3+1)
     & +      F270*coarse( c1-1, c2-2, c3+1)
     & +      F125*coarse( c1-2, c2-2, c3-1)
     & -       F75*coarse( c1-2, c2-2, c3+1)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     & +    F13230*coarse( c1  , c2  , c3  )
     & +     F8820*coarse( c1-1, c2  , c3  )
     & +     F8820*coarse( c1  , c2-1, c3  )
     & +     F5880*coarse( c1-1, c2-1, c3  )
     & +     F2205*coarse( c1  , c2  , c3+1)
     & -     F1890*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3  )
     & +     F1470*coarse( c1-1, c2  , c3+1)
     & +     F1470*coarse( c1  , c2-1, c3+1)
     & -     F1323*coarse( c1  , c2  , c3-1)
     & -     F1260*coarse( c1-2, c2-1, c3  )
     & -     F1260*coarse( c1-1, c2-2, c3  )
     & +      F980*coarse( c1-1, c2-1, c3+1)
     & -      F882*coarse( c1-1, c2  , c3-1)
     & -      F882*coarse( c1  , c2-1, c3-1)
     & -      F588*coarse( c1-1, c2-1, c3-1)
     & -      F315*coarse( c1-2, c2  , c3+1)
     & -      F315*coarse( c1  , c2-2, c3+1)
     & +      F270*coarse( c1-2, c2-2, c3  )
     & -      F210*coarse( c1-2, c2-1, c3+1)
     & -      F210*coarse( c1-1, c2-2, c3+1)
     & +      F189*coarse( c1-2, c2  , c3-1)
     & +      F189*coarse( c1  , c2-2, c3-1)
     & +      F126*coarse( c1-2, c2-1, c3-1)
     & +      F126*coarse( c1-1, c2-2, c3-1)
     & +       F45*coarse( c1-2, c2-2, c3+1)
     & -       F27*coarse( c1-2, c2-2, c3-1)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3  )
     & -     F7560*coarse( c1-1, c2-1, c3  )
     & +     F4725*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1  , c2-1, c3+1)
     & -     F2835*coarse( c1  , c2  , c3-1)
     & +     F2100*coarse( c1-2, c2-1, c3  )
     & -     F1890*coarse( c1  , c2-1, c3-1)
     & -     F1890*coarse( c1-1, c2  , c3+1)
     & +     F1620*coarse( c1-1, c2-2, c3  )
     & -     F1260*coarse( c1-1, c2-1, c3+1)
     & +     F1134*coarse( c1-1, c2  , c3-1)
     & +      F756*coarse( c1-1, c2-1, c3-1)
     & -      F675*coarse( c1  , c2-2, c3+1)
     & +      F525*coarse( c1-2, c2  , c3+1)
     & -      F450*coarse( c1-2, c2-2, c3  )
     & +      F405*coarse( c1  , c2-2, c3-1)
     & +      F350*coarse( c1-2, c2-1, c3+1)
     & -      F315*coarse( c1-2, c2  , c3-1)
     & +      F270*coarse( c1-1, c2-2, c3+1)
     & -      F210*coarse( c1-2, c2-1, c3-1)
     & -      F162*coarse( c1-1, c2-2, c3-1)
     & -       F75*coarse( c1-2, c2-2, c3+1)
     & +       F45*coarse( c1-2, c2-2, c3-1)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     & +    F28350*coarse( c1  , c2  , c3  )
     & +    F18900*coarse( c1-1, c2  , c3  )
     & -    F11340*coarse( c1  , c2-1, c3  )
     & -     F7560*coarse( c1-1, c2-1, c3  )
     & +     F4725*coarse( c1  , c2  , c3+1)
     & -     F4050*coarse( c1-2, c2  , c3  )
     & +     F3150*coarse( c1  , c2-2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3+1)
     & -     F2835*coarse( c1  , c2  , c3-1)
     & +     F2100*coarse( c1-1, c2-2, c3  )
     & -     F1890*coarse( c1-1, c2  , c3-1)
     & -     F1890*coarse( c1  , c2-1, c3+1)
     & +     F1620*coarse( c1-2, c2-1, c3  )
     & -     F1260*coarse( c1-1, c2-1, c3+1)
     & +     F1134*coarse( c1  , c2-1, c3-1)
     & +      F756*coarse( c1-1, c2-1, c3-1)
     & -      F675*coarse( c1-2, c2  , c3+1)
     & +      F525*coarse( c1  , c2-2, c3+1)
     & -      F450*coarse( c1-2, c2-2, c3  )
     & +      F405*coarse( c1-2, c2  , c3-1)
     & +      F350*coarse( c1-1, c2-2, c3+1)
     & -      F315*coarse( c1  , c2-2, c3-1)
     & +      F270*coarse( c1-2, c2-1, c3+1)
     & -      F210*coarse( c1-1, c2-2, c3-1)
     & -      F162*coarse( c1-2, c2-1, c3-1)
     & -       F75*coarse( c1-2, c2-2, c3+1)
     & +       F45*coarse( c1-2, c2-2, c3-1)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     & +    F60750*coarse( c1  , c2  , c3  )
     & -    F24300*coarse( c1-1, c2  , c3  )
     & -    F24300*coarse( c1  , c2-1, c3  )
     & +    F10125*coarse( c1  , c2  , c3+1)
     & +     F9720*coarse( c1-1, c2-1, c3  )
     & +     F6750*coarse( c1-2, c2  , c3  )
     & +     F6750*coarse( c1  , c2-2, c3  )
     & -     F6075*coarse( c1  , c2  , c3-1)
     & -     F4050*coarse( c1-1, c2  , c3+1)
     & -     F4050*coarse( c1  , c2-1, c3+1)
     & -     F2700*coarse( c1-2, c2-1, c3  )
     & -     F2700*coarse( c1-1, c2-2, c3  )
     & +     F2430*coarse( c1-1, c2  , c3-1)
     & +     F2430*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1-1, c2-1, c3+1)
     & +     F1125*coarse( c1-2, c2  , c3+1)
     & +     F1125*coarse( c1  , c2-2, c3+1)
     & -      F972*coarse( c1-1, c2-1, c3-1)
     & +      F750*coarse( c1-2, c2-2, c3  )
     & -      F675*coarse( c1-2, c2  , c3-1)
     & -      F675*coarse( c1  , c2-2, c3-1)
     & -      F450*coarse( c1-2, c2-1, c3+1)
     & -      F450*coarse( c1-1, c2-2, c3+1)
     & +      F270*coarse( c1-2, c2-1, c3-1)
     & +      F270*coarse( c1-1, c2-2, c3-1)
     & +      F125*coarse( c1-2, c2-2, c3+1)
     & -       F75*coarse( c1-2, c2-2, c3-1)
     & )
            f31 = f31 + 2
          end do
c
c Set cint(5)-cint(4)-cint(6) corner
          if(c3f.lt.cint(6)) then
            c3 = cint(6)
            f31 = 2*c3
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &       F9261*coarse( c1  , c2  , c3  )
     & +     F6174*coarse( c1-1, c2  , c3  )
     & +     F6174*coarse( c1  , c2-1, c3  )
     & +     F6174*coarse( c1  , c2  , c3-1)
     & +     F4116*coarse( c1-1, c2-1, c3  )
     & +     F4116*coarse( c1-1, c2  , c3-1)
     & +     F4116*coarse( c1  , c2-1, c3-1)
     & +     F2744*coarse( c1-1, c2-1, c3-1)
     & -     F1323*coarse( c1-2, c2  , c3  )
     & -     F1323*coarse( c1  , c2-2, c3  )
     & -     F1323*coarse( c1  , c2  , c3-2)
     & -      F882*coarse( c1-2, c2-1, c3  )
     & -      F882*coarse( c1-1, c2-2, c3  )
     & -      F882*coarse( c1-2, c2  , c3-1)
     & -      F882*coarse( c1  , c2-2, c3-1)
     & -      F882*coarse( c1-1, c2  , c3-2)
     & -      F882*coarse( c1  , c2-1, c3-2)
     & -      F588*coarse( c1-2, c2-1, c3-1)
     & -      F588*coarse( c1-1, c2-2, c3-1)
     & -      F588*coarse( c1-1, c2-1, c3-2)
     & +      F189*coarse( c1-2, c2-2, c3  )
     & +      F189*coarse( c1-2, c2  , c3-2)
     & +      F189*coarse( c1  , c2-2, c3-2)
     & +      F126*coarse( c1-2, c2-2, c3-1)
     & +      F126*coarse( c1-2, c2-1, c3-2)
     & +      F126*coarse( c1-1, c2-2, c3-2)
     & -       F27*coarse( c1-2, c2-2, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1  , c2-1, c3  )
     & +    F13230*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1  , c2-1, c3-1)
     & -     F7938*coarse( c1-1, c2  , c3  )
     & -     F5292*coarse( c1-1, c2-1, c3  )
     & -     F5292*coarse( c1-1, c2  , c3-1)
     & -     F3528*coarse( c1-1, c2-1, c3-1)
     & -     F2835*coarse( c1  , c2-2, c3  )
     & -     F2835*coarse( c1  , c2  , c3-2)
     & +     F2205*coarse( c1-2, c2  , c3  )
     & -     F1890*coarse( c1  , c2-2, c3-1)
     & -     F1890*coarse( c1  , c2-1, c3-2)
     & +     F1470*coarse( c1-2, c2-1, c3  )
     & +     F1470*coarse( c1-2, c2  , c3-1)
     & +     F1134*coarse( c1-1, c2-2, c3  )
     & +     F1134*coarse( c1-1, c2  , c3-2)
     & +      F980*coarse( c1-2, c2-1, c3-1)
     & +      F756*coarse( c1-1, c2-2, c3-1)
     & +      F756*coarse( c1-1, c2-1, c3-2)
     & +      F405*coarse( c1  , c2-2, c3-2)
     & -      F315*coarse( c1-2, c2-2, c3  )
     & -      F315*coarse( c1-2, c2  , c3-2)
     & -      F210*coarse( c1-2, c2-2, c3-1)
     & -      F210*coarse( c1-2, c2-1, c3-2)
     & -      F162*coarse( c1-1, c2-2, c3-2)
     & +       F45*coarse( c1-2, c2-2, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1-1, c2  , c3  )
     & +    F13230*coarse( c1  , c2  , c3-1)
     & +     F8820*coarse( c1-1, c2  , c3-1)
     & -     F7938*coarse( c1  , c2-1, c3  )
     & -     F5292*coarse( c1-1, c2-1, c3  )
     & -     F5292*coarse( c1  , c2-1, c3-1)
     & -     F3528*coarse( c1-1, c2-1, c3-1)
     & -     F2835*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2  , c3-2)
     & +     F2205*coarse( c1  , c2-2, c3  )
     & -     F1890*coarse( c1-2, c2  , c3-1)
     & -     F1890*coarse( c1-1, c2  , c3-2)
     & +     F1470*coarse( c1-1, c2-2, c3  )
     & +     F1470*coarse( c1  , c2-2, c3-1)
     & +     F1134*coarse( c1-2, c2-1, c3  )
     & +     F1134*coarse( c1  , c2-1, c3-2)
     & +      F980*coarse( c1-1, c2-2, c3-1)
     & +      F756*coarse( c1-2, c2-1, c3-1)
     & +      F756*coarse( c1-1, c2-1, c3-2)
     & +      F405*coarse( c1-2, c2  , c3-2)
     & -      F315*coarse( c1-2, c2-2, c3  )
     & -      F315*coarse( c1  , c2-2, c3-2)
     & -      F210*coarse( c1-2, c2-2, c3-1)
     & -      F210*coarse( c1-1, c2-2, c3-2)
     & -      F162*coarse( c1-2, c2-1, c3-2)
     & +       F45*coarse( c1-2, c2-2, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2  , c3-1)
     & -    F17010*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2-1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3-1)
     & -    F11340*coarse( c1  , c2-1, c3-1)
     & +     F6804*coarse( c1-1, c2-1, c3  )
     & -     F6075*coarse( c1  , c2  , c3-2)
     & +     F4725*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2-2, c3  )
     & +     F4536*coarse( c1-1, c2-1, c3-1)
     & +     F3150*coarse( c1-2, c2  , c3-1)
     & +     F3150*coarse( c1  , c2-2, c3-1)
     & +     F2430*coarse( c1-1, c2  , c3-2)
     & +     F2430*coarse( c1  , c2-1, c3-2)
     & -     F1890*coarse( c1-2, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2-2, c3  )
     & -     F1260*coarse( c1-2, c2-1, c3-1)
     & -     F1260*coarse( c1-1, c2-2, c3-1)
     & -      F972*coarse( c1-1, c2-1, c3-2)
     & -      F675*coarse( c1-2, c2  , c3-2)
     & -      F675*coarse( c1  , c2-2, c3-2)
     & +      F525*coarse( c1-2, c2-2, c3  )
     & +      F350*coarse( c1-2, c2-2, c3-1)
     & +      F270*coarse( c1-2, c2-1, c3-2)
     & +      F270*coarse( c1-1, c2-2, c3-2)
     & -       F75*coarse( c1-2, c2-2, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F19845*coarse( c1  , c2  , c3  )
     & +    F13230*coarse( c1-1, c2  , c3  )
     & +    F13230*coarse( c1  , c2-1, c3  )
     & +     F8820*coarse( c1-1, c2-1, c3  )
     & -     F7938*coarse( c1  , c2  , c3-1)
     & -     F5292*coarse( c1-1, c2  , c3-1)
     & -     F5292*coarse( c1  , c2-1, c3-1)
     & -     F3528*coarse( c1-1, c2-1, c3-1)
     & -     F2835*coarse( c1-2, c2  , c3  )
     & -     F2835*coarse( c1  , c2-2, c3  )
     & +     F2205*coarse( c1  , c2  , c3-2)
     & -     F1890*coarse( c1-2, c2-1, c3  )
     & -     F1890*coarse( c1-1, c2-2, c3  )
     & +     F1470*coarse( c1-1, c2  , c3-2)
     & +     F1470*coarse( c1  , c2-1, c3-2)
     & +     F1134*coarse( c1-2, c2  , c3-1)
     & +     F1134*coarse( c1  , c2-2, c3-1)
     & +      F980*coarse( c1-1, c2-1, c3-2)
     & +      F756*coarse( c1-2, c2-1, c3-1)
     & +      F756*coarse( c1-1, c2-2, c3-1)
     & +      F405*coarse( c1-2, c2-2, c3  )
     & -      F315*coarse( c1-2, c2  , c3-2)
     & -      F315*coarse( c1  , c2-2, c3-2)
     & -      F210*coarse( c1-2, c2-1, c3-2)
     & -      F210*coarse( c1-1, c2-2, c3-2)
     & -      F162*coarse( c1-2, c2-2, c3-1)
     & +       F45*coarse( c1-2, c2-2, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1  , c2-1, c3  )
     & -    F17010*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1-1, c2-1, c3  )
     & -    F11340*coarse( c1  , c2-1, c3-1)
     & +     F6804*coarse( c1-1, c2  , c3-1)
     & -     F6075*coarse( c1  , c2-2, c3  )
     & +     F4725*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2  , c3-2)
     & +     F4536*coarse( c1-1, c2-1, c3-1)
     & +     F3150*coarse( c1-2, c2-1, c3  )
     & +     F3150*coarse( c1  , c2-1, c3-2)
     & +     F2430*coarse( c1-1, c2-2, c3  )
     & +     F2430*coarse( c1  , c2-2, c3-1)
     & -     F1890*coarse( c1-2, c2  , c3-1)
     & -     F1890*coarse( c1-1, c2  , c3-2)
     & -     F1260*coarse( c1-2, c2-1, c3-1)
     & -     F1260*coarse( c1-1, c2-1, c3-2)
     & -      F972*coarse( c1-1, c2-2, c3-1)
     & -      F675*coarse( c1-2, c2-2, c3  )
     & -      F675*coarse( c1  , c2-2, c3-2)
     & +      F525*coarse( c1-2, c2  , c3-2)
     & +      F350*coarse( c1-2, c2-1, c3-2)
     & +      F270*coarse( c1-2, c2-2, c3-1)
     & +      F270*coarse( c1-1, c2-2, c3-2)
     & -       F75*coarse( c1-2, c2-2, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F42525*coarse( c1  , c2  , c3  )
     & +    F28350*coarse( c1-1, c2  , c3  )
     & -    F17010*coarse( c1  , c2-1, c3  )
     & -    F17010*coarse( c1  , c2  , c3-1)
     & -    F11340*coarse( c1-1, c2-1, c3  )
     & -    F11340*coarse( c1-1, c2  , c3-1)
     & +     F6804*coarse( c1  , c2-1, c3-1)
     & -     F6075*coarse( c1-2, c2  , c3  )
     & +     F4725*coarse( c1  , c2-2, c3  )
     & +     F4725*coarse( c1  , c2  , c3-2)
     & +     F4536*coarse( c1-1, c2-1, c3-1)
     & +     F3150*coarse( c1-1, c2-2, c3  )
     & +     F3150*coarse( c1-1, c2  , c3-2)
     & +     F2430*coarse( c1-2, c2-1, c3  )
     & +     F2430*coarse( c1-2, c2  , c3-1)
     & -     F1890*coarse( c1  , c2-2, c3-1)
     & -     F1890*coarse( c1  , c2-1, c3-2)
     & -     F1260*coarse( c1-1, c2-2, c3-1)
     & -     F1260*coarse( c1-1, c2-1, c3-2)
     & -      F972*coarse( c1-2, c2-1, c3-1)
     & -      F675*coarse( c1-2, c2-2, c3  )
     & -      F675*coarse( c1-2, c2  , c3-2)
     & +      F525*coarse( c1  , c2-2, c3-2)
     & +      F350*coarse( c1-1, c2-2, c3-2)
     & +      F270*coarse( c1-2, c2-2, c3-1)
     & +      F270*coarse( c1-2, c2-1, c3-2)
     & -       F75*coarse( c1-2, c2-2, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F91125*coarse( c1  , c2  , c3  )
     & -    F36450*coarse( c1-1, c2  , c3  )
     & -    F36450*coarse( c1  , c2-1, c3  )
     & -    F36450*coarse( c1  , c2  , c3-1)
     & +    F14580*coarse( c1-1, c2-1, c3  )
     & +    F14580*coarse( c1-1, c2  , c3-1)
     & +    F14580*coarse( c1  , c2-1, c3-1)
     & +    F10125*coarse( c1-2, c2  , c3  )
     & +    F10125*coarse( c1  , c2-2, c3  )
     & +    F10125*coarse( c1  , c2  , c3-2)
     & -     F5832*coarse( c1-1, c2-1, c3-1)
     & -     F4050*coarse( c1-2, c2-1, c3  )
     & -     F4050*coarse( c1-1, c2-2, c3  )
     & -     F4050*coarse( c1-2, c2  , c3-1)
     & -     F4050*coarse( c1  , c2-2, c3-1)
     & -     F4050*coarse( c1-1, c2  , c3-2)
     & -     F4050*coarse( c1  , c2-1, c3-2)
     & +     F1620*coarse( c1-2, c2-1, c3-1)
     & +     F1620*coarse( c1-1, c2-2, c3-1)
     & +     F1620*coarse( c1-1, c2-1, c3-2)
     & +     F1125*coarse( c1-2, c2-2, c3  )
     & +     F1125*coarse( c1-2, c2  , c3-2)
     & +     F1125*coarse( c1  , c2-2, c3-2)
     & -      F450*coarse( c1-2, c2-2, c3-1)
     & -      F450*coarse( c1-2, c2-1, c3-2)
     & -      F450*coarse( c1-1, c2-2, c3-2)
     & +      F125*coarse( c1-2, c2-2, c3-2)
     & )
          end if
        end if
      end if
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Set cint(6) boundary
      if(c3f.lt.cint(6)) then
        c3 = cint(6)
        f31 = 2*c3
        f21 = f2i
        do c2 = c2i,c2f
          f11 = f1i
          do c1 = c1i,c1f
            fine(f11-1,f21-1,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1  , c2  , c3-2)
     & +     F2100*coarse( c1  , c2-1, c3-1)
     & +     F2100*coarse( c1-1, c2  , c3-1)
     & -     F1890*coarse( c1+1, c2  , c3  )
     & -     F1890*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1+1, c2  , c3-1)
     & -     F1260*coarse( c1  , c2+1, c3-1)
     & +      F525*coarse( c1-1, c2-1, c3  )
     & -      F450*coarse( c1  , c2-1, c3-2)
     & -      F450*coarse( c1-1, c2  , c3-2)
     & +      F350*coarse( c1-1, c2-1, c3-1)
     & -      F315*coarse( c1+1, c2-1, c3  )
     & -      F315*coarse( c1-1, c2+1, c3  )
     & +      F270*coarse( c1+1, c2  , c3-2)
     & +      F270*coarse( c1  , c2+1, c3-2)
     & -      F210*coarse( c1+1, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3-1)
     & +      F189*coarse( c1+1, c2+1, c3  )
     & +      F126*coarse( c1+1, c2+1, c3-1)
     & -       F75*coarse( c1-1, c2-1, c3-2)
     & +       F45*coarse( c1-1, c2+1, c3-2)
     & +       F45*coarse( c1+1, c2-1, c3-2)
     & -       F27*coarse( c1+1, c2+1, c3-2)
     & )
            fine(f11  ,f21-1,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1  , c2-1, c3  )
     & +     F3150*coarse( c1+1, c2  , c3  )
     & -     F2700*coarse( c1  , c2  , c3-2)
     & +     F2100*coarse( c1  , c2-1, c3-1)
     & +     F2100*coarse( c1+1, c2  , c3-1)
     & -     F1890*coarse( c1-1, c2  , c3  )
     & -     F1890*coarse( c1  , c2+1, c3  )
     & -     F1260*coarse( c1-1, c2  , c3-1)
     & -     F1260*coarse( c1  , c2+1, c3-1)
     & +      F525*coarse( c1+1, c2-1, c3  )
     & -      F450*coarse( c1  , c2-1, c3-2)
     & -      F450*coarse( c1+1, c2  , c3-2)
     & +      F350*coarse( c1+1, c2-1, c3-1)
     & -      F315*coarse( c1-1, c2-1, c3  )
     & -      F315*coarse( c1+1, c2+1, c3  )
     & +      F270*coarse( c1-1, c2  , c3-2)
     & +      F270*coarse( c1  , c2+1, c3-2)
     & -      F210*coarse( c1-1, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3-1)
     & +      F189*coarse( c1-1, c2+1, c3  )
     & +      F126*coarse( c1-1, c2+1, c3-1)
     & -       F75*coarse( c1+1, c2-1, c3-2)
     & +       F45*coarse( c1-1, c2-1, c3-2)
     & +       F45*coarse( c1+1, c2+1, c3-2)
     & -       F27*coarse( c1-1, c2+1, c3-2)
     & )
            fine(f11-1,f21  ,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1-1, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2  , c3-2)
     & +     F2100*coarse( c1-1, c2  , c3-1)
     & +     F2100*coarse( c1  , c2+1, c3-1)
     & -     F1890*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1+1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-1, c3-1)
     & -     F1260*coarse( c1+1, c2  , c3-1)
     & +      F525*coarse( c1-1, c2+1, c3  )
     & -      F450*coarse( c1-1, c2  , c3-2)
     & -      F450*coarse( c1  , c2+1, c3-2)
     & +      F350*coarse( c1-1, c2+1, c3-1)
     & -      F315*coarse( c1-1, c2-1, c3  )
     & -      F315*coarse( c1+1, c2+1, c3  )
     & +      F270*coarse( c1  , c2-1, c3-2)
     & +      F270*coarse( c1+1, c2  , c3-2)
     & -      F210*coarse( c1-1, c2-1, c3-1)
     & -      F210*coarse( c1+1, c2+1, c3-1)
     & +      F189*coarse( c1+1, c2-1, c3  )
     & +      F126*coarse( c1+1, c2-1, c3-1)
     & -       F75*coarse( c1-1, c2+1, c3-2)
     & +       F45*coarse( c1-1, c2-1, c3-2)
     & +       F45*coarse( c1+1, c2+1, c3-2)
     & -       F27*coarse( c1+1, c2-1, c3-2)
     & )
            fine(f11  ,f21  ,f31-1) = F1o32768*(
     &      F18900*coarse( c1  , c2  , c3  )
     & +    F12600*coarse( c1  , c2  , c3-1)
     & +     F3150*coarse( c1+1, c2  , c3  )
     & +     F3150*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2  , c3-2)
     & +     F2100*coarse( c1+1, c2  , c3-1)
     & +     F2100*coarse( c1  , c2+1, c3-1)
     & -     F1890*coarse( c1  , c2-1, c3  )
     & -     F1890*coarse( c1-1, c2  , c3  )
     & -     F1260*coarse( c1  , c2-1, c3-1)
     & -     F1260*coarse( c1-1, c2  , c3-1)
     & +      F525*coarse( c1+1, c2+1, c3  )
     & -      F450*coarse( c1+1, c2  , c3-2)
     & -      F450*coarse( c1  , c2+1, c3-2)
     & +      F350*coarse( c1+1, c2+1, c3-1)
     & -      F315*coarse( c1+1, c2-1, c3  )
     & -      F315*coarse( c1-1, c2+1, c3  )
     & +      F270*coarse( c1  , c2-1, c3-2)
     & +      F270*coarse( c1-1, c2  , c3-2)
     & -      F210*coarse( c1+1, c2-1, c3-1)
     & -      F210*coarse( c1-1, c2+1, c3-1)
     & +      F189*coarse( c1-1, c2-1, c3  )
     & +      F126*coarse( c1-1, c2-1, c3-1)
     & -       F75*coarse( c1+1, c2+1, c3-2)
     & +       F45*coarse( c1+1, c2-1, c3-2)
     & +       F45*coarse( c1-1, c2+1, c3-2)
     & -       F27*coarse( c1-1, c2-1, c3-2)
     & )
            fine(f11-1,f21-1,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1-1, c2  , c3  )
     & +     F4500*coarse( c1  , c2  , c3-2)
     & -     F4050*coarse( c1+1, c2  , c3  )
     & -     F4050*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2-1, c3-1)
     & -     F2700*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2+1, c3-1)
     & +     F1125*coarse( c1-1, c2-1, c3  )
     & +      F750*coarse( c1  , c2-1, c3-2)
     & +      F750*coarse( c1-1, c2  , c3-2)
     & -      F675*coarse( c1+1, c2-1, c3  )
     & -      F675*coarse( c1-1, c2+1, c3  )
     & -      F450*coarse( c1-1, c2-1, c3-1)
     & -      F450*coarse( c1+1, c2  , c3-2)
     & -      F450*coarse( c1  , c2+1, c3-2)
     & +      F405*coarse( c1+1, c2+1, c3  )
     & +      F270*coarse( c1+1, c2-1, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3-1)
     & -      F162*coarse( c1+1, c2+1, c3-1)
     & +      F125*coarse( c1-1, c2-1, c3-2)
     & -       F75*coarse( c1+1, c2-1, c3-2)
     & -       F75*coarse( c1-1, c2+1, c3-2)
     & +       F45*coarse( c1+1, c2+1, c3-2)
     & )
            fine(f11  ,f21-1,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1  , c2-1, c3  )
     & +     F6750*coarse( c1+1, c2  , c3  )
     & +     F4500*coarse( c1  , c2  , c3-2)
     & -     F4050*coarse( c1-1, c2  , c3  )
     & -     F4050*coarse( c1  , c2+1, c3  )
     & -     F2700*coarse( c1  , c2-1, c3-1)
     & -     F2700*coarse( c1+1, c2  , c3-1)
     & +     F1620*coarse( c1-1, c2  , c3-1)
     & +     F1620*coarse( c1  , c2+1, c3-1)
     & +     F1125*coarse( c1+1, c2-1, c3  )
     & +      F750*coarse( c1  , c2-1, c3-2)
     & +      F750*coarse( c1+1, c2  , c3-2)
     & -      F675*coarse( c1-1, c2-1, c3  )
     & -      F675*coarse( c1+1, c2+1, c3  )
     & -      F450*coarse( c1+1, c2-1, c3-1)
     & -      F450*coarse( c1-1, c2  , c3-2)
     & -      F450*coarse( c1  , c2+1, c3-2)
     & +      F405*coarse( c1-1, c2+1, c3  )
     & +      F270*coarse( c1-1, c2-1, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3-1)
     & -      F162*coarse( c1-1, c2+1, c3-1)
     & +      F125*coarse( c1+1, c2-1, c3-2)
     & -       F75*coarse( c1-1, c2-1, c3-2)
     & -       F75*coarse( c1+1, c2+1, c3-2)
     & +       F45*coarse( c1-1, c2+1, c3-2)
     & )
            fine(f11-1,f21  ,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1-1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+1, c3  )
     & +     F4500*coarse( c1  , c2  , c3-2)
     & -     F4050*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1+1, c2  , c3  )
     & -     F2700*coarse( c1-1, c2  , c3-1)
     & -     F2700*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1+1, c2  , c3-1)
     & +     F1125*coarse( c1-1, c2+1, c3  )
     & +      F750*coarse( c1-1, c2  , c3-2)
     & +      F750*coarse( c1  , c2+1, c3-2)
     & -      F675*coarse( c1-1, c2-1, c3  )
     & -      F675*coarse( c1+1, c2+1, c3  )
     & -      F450*coarse( c1-1, c2+1, c3-1)
     & -      F450*coarse( c1  , c2-1, c3-2)
     & -      F450*coarse( c1+1, c2  , c3-2)
     & +      F405*coarse( c1+1, c2-1, c3  )
     & +      F270*coarse( c1-1, c2-1, c3-1)
     & +      F270*coarse( c1+1, c2+1, c3-1)
     & -      F162*coarse( c1+1, c2-1, c3-1)
     & +      F125*coarse( c1-1, c2+1, c3-2)
     & -       F75*coarse( c1-1, c2-1, c3-2)
     & -       F75*coarse( c1+1, c2+1, c3-2)
     & +       F45*coarse( c1+1, c2-1, c3-2)
     & )
            fine(f11  ,f21  ,f31  ) = F1o32768*(
     &      F40500*coarse( c1  , c2  , c3  )
     & -    F16200*coarse( c1  , c2  , c3-1)
     & +     F6750*coarse( c1+1, c2  , c3  )
     & +     F6750*coarse( c1  , c2+1, c3  )
     & +     F4500*coarse( c1  , c2  , c3-2)
     & -     F4050*coarse( c1  , c2-1, c3  )
     & -     F4050*coarse( c1-1, c2  , c3  )
     & -     F2700*coarse( c1+1, c2  , c3-1)
     & -     F2700*coarse( c1  , c2+1, c3-1)
     & +     F1620*coarse( c1  , c2-1, c3-1)
     & +     F1620*coarse( c1-1, c2  , c3-1)
     & +     F1125*coarse( c1+1, c2+1, c3  )
     & +      F750*coarse( c1+1, c2  , c3-2)
     & +      F750*coarse( c1  , c2+1, c3-2)
     & -      F675*coarse( c1+1, c2-1, c3  )
     & -      F675*coarse( c1-1, c2+1, c3  )
     & -      F450*coarse( c1+1, c2+1, c3-1)
     & -      F450*coarse( c1  , c2-1, c3-2)
     & -      F450*coarse( c1-1, c2  , c3-2)
     & +      F405*coarse( c1-1, c2-1, c3  )
     & +      F270*coarse( c1+1, c2-1, c3-1)
     & +      F270*coarse( c1-1, c2+1, c3-1)
     & -      F162*coarse( c1-1, c2-1, c3-1)
     & +      F125*coarse( c1+1, c2+1, c3-2)
     & -       F75*coarse( c1+1, c2-1, c3-2)
     & -       F75*coarse( c1-1, c2+1, c3-2)
     & +       F45*coarse( c1-1, c2-1, c3-2)
     & )
            f11 = f11 + 2
          end do
          f21 = f21 + 2
        end do
      end if
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
      return
      end
